@@ -25,6 +25,7 @@
 
 #include "ipa_kdb.h"
 #include "ipa_krb5.h"
+#include "ipa_hostname.h"
 
 #define IPADB_GLOBAL_CONFIG_CACHE_TIME 60
 
@@ -52,14 +53,16 @@ static void ipadb_context_free(krb5_context kcontext,
         free((*ctx)->base);
         free((*ctx)->realm_base);
         free((*ctx)->accounts_base);
-        free((*ctx)->kdc_hostname);
+        /* kdc_hostname is a pointer to a static char[] */
         /* ldap free lcontext */
         if ((*ctx)->lcontext) {
             ldap_unbind_ext_s((*ctx)->lcontext, NULL, NULL);
+            (*ctx)->lcontext = NULL;
         }
         free((*ctx)->supp_encs);
         free((*ctx)->def_encs);
         ipadb_mspac_struct_free(&(*ctx)->mspac);
+        krb5_free_principal(kcontext, (*ctx)->local_tgs);
         krb5_free_default_realm(kcontext, (*ctx)->realm);
 
         cfg = &(*ctx)->config;
@@ -495,6 +498,27 @@ done:
     return 0;
 }
 
+static krb5_principal ipadb_create_local_tgs(krb5_context kcontext,
+                                             struct ipadb_context *ipactx)
+{
+    krb5_principal tgtp;
+    unsigned int length = strlen(ipactx->realm);
+    krb5_error_code kerr = 0;
+
+    kerr = krb5_build_principal_ext(kcontext, &tgtp,
+                                    length,
+                                    ipactx->realm,
+                                    KRB5_TGS_NAME_SIZE,
+                                    KRB5_TGS_NAME,
+                                    length,
+                                    ipactx->realm, 0);
+    if (kerr != 0) {
+        return NULL;
+    }
+
+    return tgtp;
+}
+
 /* INTERFACE */
 
 static krb5_error_code ipadb_init_library(void)
@@ -515,7 +539,6 @@ static krb5_error_code ipadb_init_module(krb5_context kcontext,
     krb5_error_code kerr;
     int ret;
     int i;
-    struct utsname uname_data;
 
     /* make sure the context is freed to avoid leaking it */
     ipactx = ipadb_get_context(kcontext);
@@ -556,6 +579,12 @@ static krb5_error_code ipadb_init_module(krb5_context kcontext,
         goto fail;
     }
 
+    ipactx->local_tgs = ipadb_create_local_tgs(kcontext, ipactx);
+    if (!ipactx->local_tgs) {
+        ret = ENOMEM;
+        goto fail;
+    }
+
     ipactx->base = ipadb_get_base_from_realm(kcontext);
     if (!ipactx->base) {
         ret = ENOMEM;
@@ -575,15 +604,9 @@ static krb5_error_code ipadb_init_module(krb5_context kcontext,
         goto fail;
     }
 
-    ret = uname(&uname_data);
-    if (ret) {
-        ret = EINVAL;
-        goto fail;
-    }
-
-    ipactx->kdc_hostname = strdup(uname_data.nodename);
+    ipactx->kdc_hostname = ipa_gethostfqdn();
     if (!ipactx->kdc_hostname) {
-        ret = ENOMEM;
+        ret = errno;
         goto fail;
     }
 
