@@ -49,10 +49,6 @@
 #include "ipa_krb5.h"
 #include "ipa_pwd.h"
 
-#ifndef MAXHOSTNAMELEN
-#define MAXHOSTNAMELEN 64
-#endif
-
 /* easier to copy the defines here than to mess with kadm5/admin.h
  * for now */
 #define KMASK_PRINCIPAL         0x000001
@@ -83,6 +79,7 @@
 #define IPA_USER_AUTH_TYPE "ipaUserAuthType"
 
 struct ipadb_mspac;
+struct dom_sid;
 
 enum ipadb_user_auth {
   IPADB_USER_AUTH_NONE     = 0,
@@ -119,7 +116,7 @@ struct ipadb_context {
     char *realm;
     char *realm_base;
     char *accounts_base;
-    char *kdc_hostname;
+    const char *kdc_hostname;
     LDAP *lcontext;
     krb5_context kcontext;
     bool override_restrictions;
@@ -134,6 +131,8 @@ struct ipadb_context {
 
     /* Don't access this directly, use ipadb_get_global_config(). */
     struct ipadb_global_config config;
+
+    krb5_principal local_tgs;
 };
 
 struct ipadb_e_pol_limits {
@@ -147,6 +146,7 @@ struct ipadb_e_data {
     bool ipa_user;
     char *entry_dn;
     char *passwd;
+    char *user;
     time_t last_pwd_change;
     char *pw_policy_dn;
     char **pw_history;
@@ -156,6 +156,8 @@ struct ipadb_e_data {
     bool has_tktpolaux;
     enum ipadb_user_auth user_auth;
     struct ipadb_e_pol_limits pol_limits[IPADB_USER_AUTH_IDX_MAX];
+    bool has_sid;
+    struct dom_sid *sid;
 };
 
 struct ipadb_context *ipadb_get_context(krb5_context kcontext);
@@ -367,3 +369,8 @@ int ipadb_get_enc_salt_types(struct ipadb_context *ipactx, LDAPMessage *entry,
 /* CERTAUTH PLUGIN */
 void ipa_certauth_free_moddata(krb5_certauth_moddata *moddata);
 #endif
+
+int ipadb_string_to_sid(const char *str, struct dom_sid *sid);
+void alloc_sid(struct dom_sid **sid);
+void free_sid(struct dom_sid **sid);
+bool dom_sid_check(const struct dom_sid *sid1, const struct dom_sid *sid2, bool exact_check);
\ No newline at end of file
