@@ -28,16 +28,6 @@
  * During TGS request search by ipaKrbPrincipalName (case-insensitive)
  * and krbPrincipalName (case-sensitive)
  */
-#define PRINC_TGS_SEARCH_FILTER "(&(|(objectclass=krbprincipalaux)" \
-                                    "(objectclass=krbprincipal)" \
-                                    "(objectclass=ipakrbprincipal))" \
-                                    "(|(ipakrbprincipalalias=%s)" \
-                                      "(krbprincipalname:caseIgnoreIA5Match:=%s)))"
-
-#define PRINC_SEARCH_FILTER "(&(|(objectclass=krbprincipalaux)" \
-                                "(objectclass=krbprincipal))" \
-                              "(krbprincipalname=%s))"
-
 #define PRINC_TGS_SEARCH_FILTER_EXTRA "(&(|(objectclass=krbprincipalaux)" \
                                           "(objectclass=krbprincipal)" \
                                           "(objectclass=ipakrbprincipal))" \
@@ -49,6 +39,13 @@
                                       "(objectclass=krbprincipal))" \
                                     "(krbprincipalname=%s)" \
                                     "%s)"
+
+#define PRINC_TGS_SEARCH_FILTER_WILD_EXTRA "(&(|(objectclass=krbprincipalaux)" \
+                                               "(objectclass=krbprincipal)" \
+                                               "(objectclass=ipakrbprincipal))" \
+                                             "(|(ipakrbprincipalalias=*)" \
+                                               "(krbprincipalname=*))" \
+                                             "%s)"
 static char *std_principal_attrs[] = {
     "krbPrincipalName",
     "krbCanonicalName",
@@ -74,6 +71,7 @@ static char *std_principal_attrs[] = {
     "krbMaxRenewableAge",
 
     /* IPA SPECIFIC ATTRIBUTES */
+    "uid",
     "nsaccountlock",
     "passwordHistory",
     IPA_KRB_AUTHZ_DATA_ATTR,
@@ -81,6 +79,8 @@ static char *std_principal_attrs[] = {
     "ipatokenRadiusConfigLink",
     "krbAuthIndMaxTicketLife",
     "krbAuthIndMaxRenewableAge",
+    "ipaNTSecurityIdentifier",
+    "ipaUniqueID",
 
     "objectClass",
     NULL
@@ -335,6 +335,11 @@ static enum ipadb_user_auth ipadb_get_user_auth(struct ipadb_context *ipactx,
     if (gcfg != NULL)
         gua = gcfg->user_auth;
 
+    /* lcontext == NULL means ipadb_get_global_config() failed to load
+     * global config and cleared the ipactx */
+    if (ipactx->lcontext == NULL)
+        return IPADB_USER_AUTH_NONE;
+
     /* Get the user's user_auth settings if not disabled. */
     if ((gua & IPADB_USER_AUTH_DISABLED) == 0)
         ipadb_parse_user_auth(ipactx->lcontext, lentry, &ua);
@@ -491,7 +496,7 @@ static krb5_error_code ipadb_get_ldap_auth_ind(krb5_context kcontext,
     l = len;
     for (i = 0; i < count; i++) {
         ret = snprintf(ap, l, "%s ", authinds[i]);
-        if (ret <= 0 || ret > l) {
+        if (ret <= 0 || ret > (int) l) {
             ret = ENOMEM;
             goto cleanup;
         }
@@ -589,7 +594,9 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
     krb5_kvno mkvno = 0;
     char **restrlist;
     char *restring;
+    char *uidstring;
     char **authz_data_list;
+    char *princ_sid;
     krb5_timestamp restime;
     bool resbool;
     int result;
@@ -608,8 +615,16 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
         free(entry);
         return KRB5_KDB_DBNOTINITED;
     }
-    lcontext = ipactx->lcontext;
-    if (!lcontext) {
+
+    entry->magic = KRB5_KDB_MAGIC_NUMBER;
+    entry->len = KRB5_KDB_V1_BASE_LENGTH;
+
+    /* Get User Auth configuration. */
+    ua = ipadb_get_user_auth(ipactx, lentry);
+
+    /* ipadb_get_user_auth() calls into ipadb_get_global_config()
+     * and that might fail, causing lcontext to become NULL */
+    if (!ipactx->lcontext) {
         krb5_klog_syslog(LOG_INFO,
                          "No LDAP connection in ipadb_parse_ldap_entry(); retrying...\n");
         ret = ipadb_get_connection(ipactx);
@@ -621,11 +636,10 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
         }
     }
 
-    entry->magic = KRB5_KDB_MAGIC_NUMBER;
-    entry->len = KRB5_KDB_V1_BASE_LENGTH;
-
-    /* Get User Auth configuration. */
-    ua = ipadb_get_user_auth(ipactx, lentry);
+    /* If any code below would result in invalidating ipactx->lcontext,
+     * lcontext must be updated with the new ipactx->lcontext value.
+     * We rely on the fact that none of LDAP-parsing helpers does it. */
+    lcontext = ipactx->lcontext;
 
     /* ignore mask for now */
 
@@ -839,6 +853,13 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
     }
     if (ret == 0) {
         ied->ipa_user = true;
+        ret = ipadb_ldap_attr_to_str(lcontext, lentry,
+                                     "uid", &uidstring);
+        if (ret != 0 && ret != ENOENT) {
+            kerr = ret;
+            goto done;
+        }
+        ied->user = uidstring;
     }
 
     /* check if it has the krbTicketPolicyAux objectclass */
@@ -897,7 +918,7 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
             goto done;
         }
 
-        ied->last_pwd_change = restime;
+        ied->last_pwd_change = krb5_ts2tt(restime);
     }
 
     ret = ipadb_ldap_attr_to_krb5_timestamp(lcontext, lentry,
@@ -913,7 +934,7 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
             goto done;
         }
 
-        ied->last_admin_unlock = restime;
+        ied->last_admin_unlock = krb5_ts2tt(restime);
     }
 
     ret = ipadb_ldap_attr_to_strlist(lcontext, lentry,
@@ -945,6 +966,27 @@ static krb5_error_code ipadb_parse_ldap_entry(krb5_context kcontext,
         ipadb_parse_authind_policies(kcontext, lcontext, lentry, entry, ua);
     }
 
+    /* Add SID if it is associated with the principal account */
+    ied->has_sid = false;
+    ied->sid = NULL;
+    ret = ipadb_ldap_attr_to_str(ipactx->lcontext, lentry,
+                                 "ipaNTSecurityIdentifier", &princ_sid);
+    if (ret == 0 && princ_sid != NULL) {
+        alloc_sid(&ied->sid);
+        if (ied->sid == NULL) {
+            kerr = KRB5_KDB_INTERNAL_ERROR;
+            free(princ_sid);
+            goto done;
+        }
+        ret = ipadb_string_to_sid(princ_sid, ied->sid);
+        free(princ_sid);
+        if (ret != 0) {
+            kerr = ret;
+            goto done;
+        }
+        ied->has_sid = true;
+    }
+
     kerr = 0;
 
 done:
@@ -989,34 +1031,22 @@ ipadb_fetch_principals_with_extra_filter(struct ipadb_context *ipactx,
     /* Starting in DAL 8.0, aliases are always okay. */
 #ifdef KRB5_KDB_FLAG_ALIAS_OK
     if (!(flags & KRB5_KDB_FLAG_ALIAS_OK)) {
-        if (filter == NULL) {
-            ret = asprintf(&src_filter, PRINC_SEARCH_FILTER,
-                           esc_original_princ);
-        } else {
-            ret = asprintf(&src_filter, PRINC_SEARCH_FILTER_EXTRA,
-                           esc_original_princ, filter);
-        }
+        ret = asprintf(&src_filter, PRINC_SEARCH_FILTER_EXTRA,
+                       esc_original_princ,
+                       filter ? filter : "");
     } else
 #endif
     {
-        /* In case we've got a principal name as '*' we have to
-         * follow RFC 4515 section 3 and reencode it using
-         * <valueencoding> rule from RFC 4511 section 4.1.6 but
-         * only to the part of the filter that does use assertion
-         * value. */
-        const char *asterisk = "%x2A";
-        char *assertion_value = esc_original_princ;
-
+        /* In case we've got a principal name as '*', we don't need to specify
+         * the principal itself, use pre-defined filter for a wild-card search.
+         */
         if ((len == 1) && (esc_original_princ[0] == '*')) {
-            assertion_value = asterisk;
-        }
-
-        if (filter == NULL) {
-            ret = asprintf(&src_filter, PRINC_TGS_SEARCH_FILTER,
-                           esc_original_princ, assertion_value);
+            ret = asprintf(&src_filter, PRINC_TGS_SEARCH_FILTER_WILD_EXTRA,
+                           filter ? filter : "");
         } else {
             ret = asprintf(&src_filter, PRINC_TGS_SEARCH_FILTER_EXTRA,
-                           esc_original_princ, assertion_value, filter);
+                           esc_original_princ, esc_original_princ,
+                           filter ? filter : "");
         }
     }
 
@@ -1551,6 +1581,7 @@ void ipadb_free_principal_e_data(krb5_context kcontext, krb5_octet *e_data)
     if (ied->magic == IPA_E_DATA_MAGIC) {
 	ldap_memfree(ied->entry_dn);
 	free(ied->passwd);
+	free(ied->user);
 	free(ied->pw_policy_dn);
 	for (i = 0; ied->pw_history && ied->pw_history[i]; i++) {
 	    free(ied->pw_history[i]);
@@ -1561,6 +1592,7 @@ void ipadb_free_principal_e_data(krb5_context kcontext, krb5_octet *e_data)
 	}
 	free(ied->authz_data);
 	free(ied->pol);
+	free_sid(&ied->sid);
 	free(ied);
     }
 }
@@ -1780,7 +1812,7 @@ static krb5_error_code ipadb_get_ldap_mod_time(struct ipadb_mods *imods,
     time_t timeval;
     char v[20];
 
-    timeval = (time_t)value;
+    timeval = krb5_ts2tt(value);
     t = gmtime_r(&timeval, &date);
     if (t == NULL) {
         return EINVAL;
@@ -2079,7 +2111,7 @@ static krb5_error_code ipadb_get_ldap_mod_auth_ind(krb5_context kcontext,
     char *s = NULL;
     size_t ai_size = 0;
     int cnt = 0;
-    int i = 0;
+    size_t i = 0;
 
     ret = krb5_dbe_get_string(kcontext, entry, "require_auth", &ais);
     if (ret) {
@@ -2460,7 +2492,7 @@ static krb5_error_code ipadb_entry_default_attrs(struct ipadb_mods *imods)
 {
     krb5_error_code kerr;
     LDAPMod *m = NULL;
-    int i;
+    size_t i;
 
     kerr = ipadb_mods_new(imods, &m);
     if (kerr) {
