@@ -29,8 +29,11 @@ define([
     './aci',
     './automember',
     './automount',
+    './plugins/ca',
     './plugins/caacl',
     './plugins/certprofile',
+    './plugins/certmap',
+    './plugins/certmapmatch',
     './dns',
     './group',
     './hbac',
@@ -48,11 +51,13 @@ define([
     './selinux',
     './serverconfig',
     './service',
+    './stageuser',
+    './subid',
     './sudo',
     './trust',
     './topology',
     './user',
-    './stageuser',
+    './vault',
     'dojo/domReady!'
 ],function(app_container) {
     return app_container;
