@@ -538,6 +538,13 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
      */
     that.refresh_command_name = spec.refresh_command_name || 'show';
 
+    /**
+     * Name of url argument which will be added to refresh RPC command as option.
+     *
+     * @property {string}
+     */
+    that.refresh_attribute = spec.refresh_attribute || null;
+
     /**
      * Name of update command
      *
@@ -546,6 +553,13 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
      */
     that.update_command_name = spec.update_command_name || 'mod';
 
+    /**
+     * Name of url argument which will be added to update RPC command as option.
+     *
+     * @property {string}
+     */
+    that.update_attribute = spec.update_attribute || null;
+
     /**
      * Command mode
      * Command mode determines how update information on update is collected.
@@ -588,6 +602,12 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
      */
     that.facet_group = spec.facet_group || 'settings';
 
+    /**
+     * Indicates if the details facet depends on pkey
+     * @property {boolean}
+     */
+    that.require_pkey = spec.require_pkey !== undefined ? spec.require_pkey : true;
+
     /**
      * Widgets
      * @property {IPA.widget_container}
@@ -743,7 +763,8 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
         var fields = that.fields.get_fields();
         for (var i=0; i<fields.length; i++) {
             var field = fields[i];
-            field.load(data);
+
+            if (field.autoload_value) field.load(data);
         }
         that.policies.post_load(data);
         that.post_load.notify([data], that);
@@ -883,6 +904,23 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
         }
     };
 
+    /**
+     * Takes url argument which is named arg_name and its value. Creates object
+     * from this information {arg_name: arg_name_value} and attach it as option
+     * to command.
+     *
+     * @protected
+     * @param {rpc.command} command
+     * @param {string} argumnent name
+     */
+    that.add_url_arg_to_command = function(command, arg_name) {
+        var additional_opt = {};
+        command.options = command.options || {};
+
+        additional_opt[arg_name] = that.state[arg_name];
+        $.extend(command.options, additional_opt);
+    };
+
     /**
      * Create update command based on field part of update info
      * @protected
@@ -904,6 +942,10 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
             options: options
         });
 
+        if (that.update_attribute) {
+            that.add_url_arg_to_command(command, that.update_attribute);
+        }
+
         //set command options
         that.add_fields_to_command(update_info, command);
 
@@ -1033,6 +1075,10 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
             command.args = that.get_pkeys();
         }
 
+        if (that.refresh_attribute) {
+            that.add_url_arg_to_command(command, that.refresh_attribute);
+        }
+
         return command;
     };
 
@@ -1065,7 +1111,7 @@ exp.details_facet = IPA.details_facet = function(spec, no_init) {
      */
     that.refresh = function(on_success, on_error) {
 
-        if (!that.get_pkey() && that.entity.redirect_facet) {
+        if (that.require_pkey && !that.get_pkey() && that.entity.redirect_facet) {
             that.redirect();
             return;
         }
@@ -1496,8 +1542,6 @@ exp.boolean_state_evaluator = IPA.boolean_state_evaluator = function(spec) {
 
     that.param = spec.param || that.field_name;
 
-    that.adapter = builder.build('adapter', spec.adapter || 'adapter', { context: that });
-
     /**
      * State to set when value is `true`
      * @property {string}
@@ -1603,8 +1647,6 @@ exp.acl_state_evaluator = IPA.acl_state_evaluator = function(spec) {
 
     that.param = spec.param || 'attributelevelrights';
 
-    that.adapter = builder.build('adapter', spec.adapter || 'adapter', { context: that });
-
     /**
      * @inheritDoc
      */
@@ -1647,7 +1689,6 @@ exp.value_state_evaluator = IPA.value_state_evaluator = function(spec) {
 
     spec.name = spec.name || 'value_state_evaluator';
     spec.event = spec.event || 'post_load';
-    spec.adapter = spec.adapter || {};
 
     var that = IPA.state_evaluator(spec);
 
@@ -1672,9 +1713,6 @@ exp.value_state_evaluator = IPA.value_state_evaluator = function(spec) {
      */
     that.representation = spec.representation;
 
-    that.adapter = builder.build('adapter',
-                    spec.adapter, { context: that });
-
     that.param = spec.param;
 
     /**
@@ -1758,7 +1796,6 @@ exp.has_keytab_evaluator = IPA.has_keytab_evaluator = function(spec) {
     spec.value = spec.value || [true];
     spec.representation = spec.representation || 'has_keytab';
     spec.param = spec.param || 'has_keytab';
-    spec.adapter = spec.adapter || { $type: 'adapter' };
 
     var that = IPA.value_state_evaluator(spec);
 
@@ -1783,7 +1820,6 @@ exp.object_class_evaluator = IPA.object_class_evaluator = function(spec) {
 
     var that = IPA.state_evaluator(spec);
 
-
     /**
      * @inheritDoc
      */
@@ -1792,7 +1828,7 @@ exp.object_class_evaluator = IPA.object_class_evaluator = function(spec) {
         var old_state, classes, i;
 
         old_state = that.state;
-        classes = data.result.result.objectclass;
+        classes = that.adapter.load(data, 'objectclass');
 
         that.state = [];
 
