@@ -5,7 +5,7 @@
  *    John Dennis <jdennis@redhat.com>
  *    Petr Vobornik <pvoborni@redhat.com>
  *
- * Copyright (C) 2010 Red Hat
+ * Copyright (C) 2010-2016 Red Hat
  * see file 'COPYING' for use and warranty information
  *
  * This program is free software; you can redistribute it and/or modify
@@ -62,7 +62,7 @@ var IPA = function () {
     that.jsonrpc_id = 0;
 
     // live server path
-    that.url = '/ipa/ui/';
+    that.url = config.url;
 
     /**
      * jQuery AJAX options used by RPC commands
@@ -76,17 +76,11 @@ var IPA = function () {
         processData: false
     };
 
-    /**
-     * i18n messages
-     * @deprecated
-     * @property {Object}
-     */
-    that.messages = {};
-
     /**
      * User information
      *
-     * - output of ipa user-find --whoami
+     * - output of ipa whoami in that.whoami.metadata and then object_show method
+     * in that.whoami.data
      */
     that.whoami = {};
 
@@ -128,7 +122,7 @@ var IPA = function () {
 
         // if current path matches live server path, use live data
         if (that.url && window.location.pathname.substring(0, that.url.length) === that.url) {
-            that.json_url = params.url || '/ipa/session/json';
+            that.json_url = params.url || config.json_url;
 
         } else { // otherwise use fixtures
             that.json_path = params.url || "test/data";
@@ -174,14 +168,6 @@ var IPA = function () {
             }
         });
 
-        batch.add_command(rpc.command({
-            method: 'i18n_messages',
-            on_success: function(data, text_status, xhr) {
-                that.messages = data.texts;
-                i18n.source = that.messages;
-            }
-        }));
-
         batch.add_command(rpc.command({
             entity: 'config',
             method: 'show',
@@ -190,7 +176,7 @@ var IPA = function () {
             }
         }));
 
-        batch.add_command(that.get_whoami_command(true));
+        batch.add_command(that.get_whoami_command());
 
         batch.add_command(rpc.command({
             method: 'env',
@@ -240,26 +226,60 @@ var IPA = function () {
             }
         }));
 
+        batch.add_command(rpc.command({
+            entity: 'vaultconfig',
+            method: 'show',
+            retry: false,
+            on_success: function(data, text_status, xhr) {
+                that.vault_enabled = true;
+            },
+            on_error: function(xhr, text_status, error_thrown) {
+                that.vault_enabled = false;
+            }
+        }));
+
         batch.execute();
     };
 
     /**
      * Prepares `user-find --whoami` command
      * @protected
-     * @param {boolean} batch - Specifies if it will be used as single command or
-     *                          in a batch.
      */
-    that.get_whoami_command = function(batch) {
+    that.get_whoami_command = function() {
         return rpc.command({
-            entity: 'user',
-            method: 'find',
-            options: {
-                whoami: true,
-                all: true
-            },
+            method: 'whoami',
             on_success: function(data, text_status, xhr) {
-                that.whoami = batch ? data.result[0] : data.result.result[0];
-                that.principal = that.whoami.krbprincipalname[0];
+                that.whoami.metadata = data.result || data;
+                var wa_data = that.whoami.metadata;
+                // This AJAX request has no synchronization point,
+                // so we set async = false to make sure that init_metadata
+                // doesn't start before we get whoami response.
+                $.ajaxSetup({async: false});
+                rpc.command({
+                    method: wa_data.details || wa_data.command,
+                    args: wa_data.arguments,
+                    options: function() {
+                        var options = wa_data.options || [];
+                        $.extend(options, {all: true});
+                        return options;
+                    }(),
+                    on_success: function(data, text_status, xhr) {
+                        that.whoami.data = data.result.result;
+                        var entity = that.whoami.metadata.object;
+
+                        if (entity === 'user') {
+                            var cn = that.whoami.data.krbcanonicalname;
+                            if (cn) that.principal = cn[0];
+                            if (!that.principal) {
+                                that.principal = that.whoami.data.krbprincipalname[0];
+                            }
+                        } else if (entity === 'idoverrideuser') {
+                            that.principal = that.whoami.data.ipaoriginaluid[0];
+                        }
+                    }
+                }).execute();
+                // Restore AJAX options
+                $.ajaxSetup(that.ajax_options);
             }
         });
     };
@@ -272,6 +292,9 @@ var IPA = function () {
      * @param {Function} params.on_error
      */
     that.init_metadata = function(params) {
+        var loading = text.get('@i18n:login.loading_md');
+
+        topic.publish('set-activity', loading);
 
         var objects = rpc.command({
             name: 'ipa_init_objects',
@@ -336,29 +359,6 @@ var IPA = function () {
         return reg.entity.get(name);
     };
 
-    /**
-     * Display network activity indicator
-     */
-    that.display_activity_icon = function() {
-        that.network_call_count++;
-        if (that.network_call_count === 1) {
-            topic.publish('network-activity-start');
-        }
-    };
-
-    /**
-     * Hide network activity indicator
-     *
-     * - based on network_call_count
-     */
-    that.hide_activity_icon = function() {
-        that.network_call_count--;
-
-        if (0 === that.network_call_count) {
-            topic.publish('network-activity-end');
-        }
-    };
-
     that.obj_cls = declare([Evented]);
 
     return that;
@@ -387,13 +387,13 @@ IPA.get_credentials = function() {
 
     function error_handler(xhr, text_status, error_thrown) {
         d.resolve(xhr.status);
-        IPA.hide_activity_icon();
+        topic.publish('rpc-end');
     }
 
     function success_handler(data, text_status, xhr) {
         auth.current.set_authenticated(true, 'kerberos');
         d.resolve(xhr.status);
-        IPA.hide_activity_icon();
+        topic.publish('rpc-end');
     }
 
     var request = {
@@ -403,7 +403,9 @@ IPA.get_credentials = function() {
         success: success_handler,
         error: error_handler
     };
-    IPA.display_activity_icon();
+
+    topic.publish('rpc-start');
+
     $.ajax(request);
 
     return d.promise;
@@ -435,7 +437,8 @@ IPA.logout = function() {
     }
 
     function success_handler(data, text_status, xhr) {
-        IPA.hide_activity_icon();
+        topic.publish('rpc-end');
+
         if (data && data.error) {
             show_error(data.error.message);
         } else {
@@ -444,7 +447,8 @@ IPA.logout = function() {
     }
 
     function error_handler(xhr, text_status, error_thrown) {
-        IPA.hide_activity_icon();
+        topic.publish('rpc-end');
+
         if (xhr.status === 401) {
             reload();
         } else {
@@ -463,7 +467,8 @@ IPA.logout = function() {
         success: success_handler,
         error: error_handler
     };
-    IPA.display_activity_icon();
+    topic.publish('rpc-start');
+
     $.ajax(request);
 };
 
@@ -481,7 +486,8 @@ IPA.login_password = function(username, password) {
     var d = new Deferred();
 
     function success_handler(data, text_status, xhr) {
-        IPA.hide_activity_icon();
+        topic.publish('rpc-end');
+
         result = 'success';
         auth.current.set_authenticated(true, 'password');
         d.resolve(result);
@@ -489,13 +495,18 @@ IPA.login_password = function(username, password) {
 
     function error_handler(xhr, text_status, error_thrown) {
 
-        IPA.hide_activity_icon();
+        topic.publish('rpc-end');
+
         if (xhr.status === 401) {
             var reason = xhr.getResponseHeader("X-IPA-Rejection-Reason");
 
             //change result from invalid only if we have a header which we
             //understand
-            if (reason === 'password-expired' || reason === 'denied') {
+            if (reason === 'password-expired' ||
+                reason === 'denied' ||
+                reason === 'krbprincipal-expired' ||
+                reason === 'invalid-password' ||
+                reason === 'user-locked') {
                 result = reason;
             }
         }
@@ -508,7 +519,7 @@ IPA.login_password = function(username, password) {
     };
 
     var request = {
-        url: '/ipa/session/login_password',
+        url: config.forms_login_url,
         data: data,
         contentType: 'application/x-www-form-urlencoded',
         processData: true,
@@ -518,7 +529,8 @@ IPA.login_password = function(username, password) {
         error: error_handler
     };
 
-    IPA.display_activity_icon();
+    topic.publish('rpc-start');
+
     $.ajax(request);
 
     return d.promise;
@@ -550,6 +562,8 @@ IPA.reset_password = function(username, old_password, new_password, otp) {
 
     function success_handler(data, text_status, xhr) {
 
+        topic.publish('rpc-end');
+
         result.status = xhr.getResponseHeader("X-IPA-Pwchange-Result") || status;
 
         if (result.status === 'policy-error') {
@@ -563,6 +577,7 @@ IPA.reset_password = function(username, old_password, new_password, otp) {
     }
 
     function error_handler(xhr, text_status, error_thrown) {
+        topic.publish('rpc-end');
         return result;
     }
 
@@ -577,7 +592,7 @@ IPA.reset_password = function(username, old_password, new_password, otp) {
     }
 
     request = {
-        url: '/ipa/session/change_password',
+        url: config.reset_psw_url,
         data: data,
         contentType: 'application/x-www-form-urlencoded',
         processData: true,
@@ -588,9 +603,8 @@ IPA.reset_password = function(username, old_password, new_password, otp) {
         error: error_handler
     };
 
-    IPA.display_activity_icon();
+    topic.publish('rpc-start');
     $.ajax(request);
-    IPA.hide_activity_icon();
 
     return result;
 };
@@ -606,7 +620,7 @@ IPA.update_password_expiration = function() {
 
     var now, expires, notify_days, diff, message, container, notify;
 
-    expires = rpc.extract_objects(IPA.whoami.krbpasswordexpiration);
+    expires = rpc.extract_objects(IPA.whoami.data.krbpasswordexpiration);
     expires = expires ? datetime.parse(expires[0]) : null;
 
     notify_days = IPA.server_config.ipapwdexpadvnotify;
@@ -640,13 +654,13 @@ IPA.update_password_expiration = function() {
 IPA.password_selfservice = function() {
     var reset_dialog = builder.build('dialog', {
         $type: 'user_password',
-        args: [IPA.whoami.uid[0]]
+        args: [IPA.whoami.data.uid[0]]
     });
     reset_dialog.succeeded.attach(function() {
         var command = IPA.get_whoami_command();
         var orig_on_success = command.on_success;
         command.on_success = function(data, text_status, xhr) {
-            orig_on_success.call(this, data, text_status, xhr);
+            orig_on_success.call(this, data.result, text_status, xhr);
             IPA.update_password_expiration();
         };
         command.execute();
