@@ -81,7 +81,8 @@ return {
                             name: 'krbprincipalname',
                             item_name: 'principal',
                             child_spec: {
-                                $type: 'krb_principal'
+                                $type: 'non_editable_row',
+                                data_name: 'krb-principal'
                             }
                         },
                         {
@@ -126,19 +127,25 @@ return {
                             ]
                         },
                         {
-                            $type: 'custom_checkboxes',
+                            $type: 'checkboxes',
                             label: '@i18n:authtype.auth_indicators',
                             name: 'krbprincipalauthind',
-                            add_dialog_title: '@i18n:authtype.custom_auth_ind_title',
-                            add_field_label: '@i18n:authtype.auth_indicator',
                             options: [
                                 {
-                                    label: '@i18n:authtype.otp',
+                                    label: '@i18n:authtype.type_otp',
                                     value: 'otp'
                                 },
                                 {
                                     label: '@i18n:authtype.type_radius',
                                     value: 'radius'
+                                },
+                                {
+                                    label: '@i18n:authtype.type_pkinit',
+                                    value: 'pkinit'
+                                },
+                                {
+                                    label: '@i18n:authtype.type_hardened',
+                                    value: 'hardened'
                                 }
                             ],
                             tooltip: {
@@ -200,10 +207,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_read_keys_user',
                             name: 'ipaallowedtoperform_read_keys_user',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_retrieve_keytab',
                             remove_method: 'disallow_retrieve_keytab',
-                            add_title: '@i18n:keytab.add_retrive',
-                            remove_title: '@i18n:keytab.remove_retrieve',
+                            add_title: '@i18n:keytab.add_users_retrieve',
+                            remove_title: '@i18n:keytab.remove_users_retrieve',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_read_keys_user',
@@ -216,10 +224,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_read_keys_group',
                             name: 'ipaallowedtoperform_read_keys_group',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_retrieve_keytab',
                             remove_method: 'disallow_retrieve_keytab',
-                            add_title: '@i18n:keytab.add_retrive',
-                            remove_title: '@i18n:keytab.remove_retrieve',
+                            add_title: '@i18n:keytab.add_groups_retrieve',
+                            remove_title: '@i18n:keytab.remove_groups_retrieve',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_read_keys_group',
@@ -232,10 +241,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_read_keys_host',
                             name: 'ipaallowedtoperform_read_keys_host',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_retrieve_keytab',
                             remove_method: 'disallow_retrieve_keytab',
-                            add_title: '@i18n:keytab.add_retrive',
-                            remove_title: '@i18n:keytab.remove_retrieve',
+                            add_title: '@i18n:keytab.add_hosts_retrieve',
+                            remove_title: '@i18n:keytab.remove_hosts_retrieve',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_read_keys_host',
@@ -248,10 +258,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_read_keys_hostgroup',
                             name: 'ipaallowedtoperform_read_keys_hostgroup',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_retrieve_keytab',
                             remove_method: 'disallow_retrieve_keytab',
-                            add_title: '@i18n:keytab.add_retrive',
-                            remove_title: '@i18n:keytab.remove_retrieve',
+                            add_title: '@i18n:keytab.add_hostgroups_retrieve',
+                            remove_title: '@i18n:keytab.remove_hostgroups_retrieve',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_read_keys_hostgroup',
@@ -271,10 +282,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_write_keys_user',
                             name: 'ipaallowedtoperform_write_keys_user',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_create_keytab',
                             remove_method: 'disallow_create_keytab',
-                            add_title: '@i18n:keytab.add_create',
-                            remove_title: '@i18n:keytab.remove_create',
+                            add_title: '@i18n:keytab.add_users_create',
+                            remove_title: '@i18n:keytab.remove_users_create',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_write_keys_user',
@@ -287,10 +299,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_write_keys_group',
                             name: 'ipaallowedtoperform_write_keys_group',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_create_keytab',
                             remove_method: 'disallow_create_keytab',
-                            add_title: '@i18n:keytab.add_create',
-                            remove_title: '@i18n:keytab.remove_create',
+                            add_title: '@i18n:keytab.add_groups_create',
+                            remove_title: '@i18n:keytab.remove_groups_create',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_write_keys_group',
@@ -303,10 +316,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_write_keys_host',
                             name: 'ipaallowedtoperform_write_keys_host',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_create_keytab',
                             remove_method: 'disallow_create_keytab',
-                            add_title: '@i18n:keytab.add_create',
-                            remove_title: '@i18n:keytab.remove_create',
+                            add_title: '@i18n:keytab.add_hosts_create',
+                            remove_title: '@i18n:keytab.remove_hosts_create',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_write_keys_host',
@@ -319,10 +333,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_write_keys_hostgroup',
                             name: 'ipaallowedtoperform_write_keys_hostgroup',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_create_keytab',
                             remove_method: 'disallow_create_keytab',
-                            add_title: '@i18n:keytab.add_create',
-                            remove_title: '@i18n:keytab.remove_create',
+                            add_title: '@i18n:keytab.add_hostgroups_create',
+                            remove_title: '@i18n:keytab.remove_hostgroups_create',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_write_keys_hostgroup',
@@ -336,7 +351,10 @@ return {
             ],
             actions: [
                 'service_unprovision',
-                'cert_request'
+                {
+                    $type: 'cert_request',
+                    title: '@i18n:objects.cert.issue_for_service'
+                }
             ],
             header_actions: [
                 'unprovision',
@@ -357,15 +375,24 @@ return {
                 IPA.service.certificate_policy
             ]
         },
+        {
+            $type: 'association',
+            name: 'memberof_role',
+            add_title: '@i18n:objects.service.add_into_roles',
+            remove_title: '@i18n:objects.service.remove_from_roles'
+        },
         {
             $type: 'association',
             name: 'managedby_host',
             add_method: 'add_host',
-            remove_method: 'remove_host'
+            add_title: '@i18n:objects.service.add_hosts_managing',
+            remove_method: 'remove_host',
+            remove_title: '@i18n:objects.service.remove_hosts_managing'
         }
     ],
     standard_association_facets: true,
     adder_dialog: {
+        title: '@i18n:objects.service.add',
         $factory: IPA.service_adder_dialog,
         height: 350,
         sections: [
@@ -398,6 +425,7 @@ return {
                         other_entity: 'host',
                         other_field: 'fqdn',
                         label: '@i18n:objects.service.host',
+                        editable: true,
                         required: true,
                         z_index: 1
                     },
@@ -405,10 +433,18 @@ return {
                         $type: 'checkbox',
                         name: 'force',
                         metadata: '@mc-opt:service_add:force'
+                    },
+                    {
+                        $type: 'checkbox',
+                        name: 'skip_host_check',
+                        metadata: '@mc-opt:service_add:skip_host_check'
                     }
                 ]
             }
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.service.remove'
     }
 };};
 
@@ -429,6 +465,7 @@ IPA.service.search_facet = function(spec) {
     that.get_records_map = function(data) {
 
         var records_map = $.ordered_map();
+        var pkeys_map = $.ordered_map();
 
         var result = data.result.result;
         var pkey_name = that.managed_entity.metadata.primary_key ||
@@ -442,11 +479,16 @@ IPA.service.search_facet = function(spec) {
                 pkey = adapter.load(record, that.alternative_pkey)[0];
             }
             if (that.filter_records(records_map, pkey, record)) {
-                records_map.put(pkey, record);
+                var compound_pkey = pkey + i;
+                records_map.put(compound_pkey, record);
+                pkeys_map.put(compound_pkey, pkey);
             }
         }
 
-        return records_map;
+        return {
+            records_map: records_map,
+            pkeys_map: pkeys_map
+        };
     };
 
     return that;
@@ -475,6 +517,7 @@ IPA.service.details_facet = function(spec, no_init) {
             retry: false,
             options: {
                 service: [ pkey ],
+                sizelimit: 0,
                 all: true
             }
         });
@@ -484,6 +527,12 @@ IPA.service.details_facet = function(spec, no_init) {
         return batch;
     };
 
+    that.update_on_success = function(data, text_status, xhr) {
+        that.on_update.notify();
+        that.nofify_update_success();
+        that.refresh();
+    };
+
     if (!no_init) that.init_details_facet();
 
     return that;
@@ -521,6 +570,9 @@ IPA.service_adder_dialog = function(spec) {
 
         field = that.fields.get_field('force');
         record['force'] = field.save();
+
+        field = that.fields.get_field('skip_host_check');
+        record['skip_host_check'] = field.save();
     };
 
     init();
@@ -614,9 +666,6 @@ IPA.service.unprovision_dialog = function(spec) {
     var that = IPA.dialog(spec);
     that.facet = spec.facet;
 
-    var entity_singular = that.entity.metadata.label_singular;
-    that.title = that.title.replace('${entity}', entity_singular);
-
     that.create_content = function() {
         that.container.append(text.get('@i18n:objects.service.unprovision_confirmation'));
     };
