@@ -23,23 +23,22 @@ All constants centralised in one file.
 """
 
 import os
-import socket
+import string
+
+from ipaplatform.constants import constants as _constants
 from ipapython.dn import DN
+from ipapython.fqdn import gethostfqdn
 from ipapython.version import VERSION, API_VERSION
 
-try:
-    FQDN = socket.getfqdn()
-except Exception:
-    try:
-        FQDN = socket.gethostname()
-    except Exception:
-        FQDN = None
+
+FQDN = gethostfqdn()
 
 # TLS related constants
 # * SSL2 and SSL3 are broken.
 # * TLS1.0 and TLS1.1 are no longer state of the art.
-# * TLS1.3 support is not yet stable, e.g. issues with PHA.
-# Therefore only TLS 1.2 is enabled by default.
+# * TLS1.2 and 1.3 are secure and working properly
+# * Crypto policies restrict TLS range to 1.2 and 1.3. Python 3.6 cannot
+#   override the crypto policy.
 
 TLS_VERSIONS = [
     "ssl2",
@@ -49,9 +48,10 @@ TLS_VERSIONS = [
     "tls1.2",
     "tls1.3",
 ]
-TLS_VERSION_MINIMAL = "tls1.0"
-TLS_VERSION_DEFAULT_MIN = "tls1.2"
-TLS_VERSION_DEFAULT_MAX = "tls1.2"
+TLS_VERSION_MINIMAL = "tls1.2"
+TLS_VERSION_MAXIMAL = "tls1.3"
+TLS_VERSION_DEFAULT_MIN = None
+TLS_VERSION_DEFAULT_MAX = None
 
 # regular expression NameSpace member names must match:
 NAME_REGEX = r'^[a-z][_a-z0-9]*[a-z0-9]$|^[a-z]$'
@@ -132,6 +132,9 @@ DEFAULT_CONFIG = (
     ('container_ranges', DN(('cn', 'ranges'), ('cn', 'etc'))),
     ('container_dna', DN(('cn', 'dna'), ('cn', 'ipa'), ('cn', 'etc'))),
     ('container_dna_posix_ids', DN(('cn', 'posix-ids'), ('cn', 'dna'), ('cn', 'ipa'), ('cn', 'etc'))),
+    ('container_dna_subordinate_ids', DN(
+        ('cn', 'subordinate-ids'), ('cn', 'dna'), ('cn', 'ipa'), ('cn', 'etc')
+    )),
     ('container_realm_domains', DN(('cn', 'Realm Domains'), ('cn', 'ipa'), ('cn', 'etc'))),
     ('container_otp', DN(('cn', 'otp'))),
     ('container_radiusproxy', DN(('cn', 'radiusproxy'))),
@@ -149,6 +152,7 @@ DEFAULT_CONFIG = (
     ('container_certmaprules', DN(('cn', 'certmaprules'), ('cn', 'certmap'))),
     ('container_ca_renewal',
         DN(('cn', 'ca_renewal'), ('cn', 'ipa'), ('cn', 'etc'))),
+    ('container_subids', DN(('cn', 'subids'), ('cn', 'accounts'))),
 
     # Ports, hosts, and URIs:
     # Following values do not have any reasonable default.
@@ -161,6 +165,9 @@ DEFAULT_CONFIG = (
 
     ('rpc_protocol', 'jsonrpc'),
 
+    ('ldap_cache', True),
+    ('ldap_cache_size', 100),
+
     # Define an inclusive range of SSL/TLS version support
     ('tls_version_min', TLS_VERSION_DEFAULT_MIN),
     ('tls_version_max', TLS_VERSION_DEFAULT_MAX),
@@ -173,6 +180,11 @@ DEFAULT_CONFIG = (
     ('http_timeout', 30),
     # How long to wait for an entry to appear on a replica
     ('replication_wait_timeout', 300),
+    # How long to wait for a certmonger request to finish
+    ('certmonger_wait_timeout', 300),
+
+    # Number of seconds before client should check for schema update.
+    ('schema_ttl', 3600),
 
     # Web Application mount points
     ('mount_ipa', '/ipa/'),
@@ -194,12 +206,9 @@ DEFAULT_CONFIG = (
     ('ca_host', FQDN),  # Set in Env._finalize_core()
     ('ca_port', 80),
     ('ca_agent_port', 443),
-    ('ca_ee_port', 443),
     # For the following ports, None means a default specific to the installed
     # Dogtag version.
     ('ca_install_port', None),
-    ('ca_agent_install_port', None),
-    ('ca_ee_install_port', None),
 
     # Topology plugin
     ('recommended_max_agmts', 4),  # Recommended maximum number of replication
@@ -306,6 +315,7 @@ IPA_CA_RECORD = "ipa-ca"
 IPA_CA_NICKNAME = 'caSigningCert cert-pki-ca'
 RENEWAL_CA_NAME = 'dogtag-ipa-ca-renew-agent'
 RENEWAL_REUSE_CA_NAME = 'dogtag-ipa-ca-renew-agent-reuse'
+RENEWAL_SELFSIGNED_CA_NAME = 'dogtag-ipa-ca-renew-agent-selfsigned'
 # The RA agent cert is used for client cert authentication. In the past IPA
 # used caServerCert profile, which adds clientAuth and serverAuth EKU. The
 # serverAuth EKU caused trouble with NamedConstraints, see RHBZ#1670239.
@@ -314,7 +324,10 @@ RA_AGENT_PROFILE = 'caSubsystemCert'
 CA_DBUS_TIMEOUT = 120
 
 # Maximum hostname length in Linux
+# It's the max length of uname's nodename and return value of gethostname().
 MAXHOSTNAMELEN = 64
+# DNS name is 255 octets, effectively 253 ASCII characters.
+MAXHOSTFQDNLEN = 253
 
 # regexp definitions
 PATTERN_GROUPUSER_NAME = (
@@ -325,8 +338,8 @@ PATTERN_GROUPUSER_NAME = (
 ANON_USER = 'WELLKNOWN/ANONYMOUS'
 
 # IPA API Framework user
-IPAAPI_USER = 'ipaapi'
-IPAAPI_GROUP = 'ipaapi'
+IPAAPI_USER = _constants.IPAAPI_USER
+IPAAPI_GROUP = _constants.IPAAPI_GROUP
 
 # Use cache path
 USER_CACHE_PATH = (
@@ -338,3 +351,26 @@ SOFTHSM_DNSSEC_TOKEN_LABEL = u'ipaDNSSEC'
 # Apache's mod_ssl SSLVerifyDepth value (Maximum depth of CA
 # Certificates in Client Certificate verification)
 MOD_SSL_VERIFY_DEPTH = '5'
+
+# subuid / subgid counts are hard-coded
+# An interval of 65536 uids/gids is required to map nobody (65534).
+SUBID_COUNT = 65536
+
+# upper half of uid_t (uint32_t)
+SUBID_RANGE_START = 2 ** 31
+# theoretical max limit is UINT32_MAX-1 ((2 ** 32) - 2)
+# We use a smaller value to keep the topmost subid interval unused.
+SUBID_RANGE_MAX = (2 ** 32) - (2 * SUBID_COUNT)
+SUBID_RANGE_SIZE = SUBID_RANGE_MAX - SUBID_RANGE_START
+# threshold before DNA plugin requests a new range
+SUBID_DNA_THRESHOLD = 500
+
+# moved from ipaserver/install/krainstance.py::KRAInstance to avoid duplication
+# as per https://pagure.io/freeipa/issue/8795
+KRA_TRACKING_REQS = {
+    'auditSigningCert cert-pki-kra': 'caAuditSigningCert',
+    'transportCert cert-pki-kra': 'caTransportCert',
+    'storageCert cert-pki-kra': 'caStorageCert',
+}
+
+ALLOWED_NETBIOS_CHARS = string.ascii_uppercase + string.digits + '-'
