@@ -35,6 +35,7 @@ import six
 
 from ipapython import ipautil
 from ipaplatform.paths import paths
+from ipaplatform.tasks import tasks
 
 # pylint: disable=no-name-in-module, import-error
 if six.PY3:
@@ -49,11 +50,16 @@ logger = logging.getLogger(__name__)
 # *some* naming, set them as in Red Hat distributions. Actual implementation
 # should make them available through knownservices.<name> and take care of
 # re-mapping internally, if needed
-wellknownservices = ['certmonger', 'dirsrv', 'httpd', 'ipa', 'krb5kdc',
-                     'dbus', 'nslcd', 'nscd', 'ntpd', 'portmap',
-                     'rpcbind', 'kadmin', 'sshd', 'autofs', 'rpcgssd',
-                     'rpcidmapd', 'pki_tomcatd', 'chronyd', 'domainname',
-                     'named', 'ods_enforcerd', 'ods_signerd', 'gssproxy']
+wellknownservices = [
+    'certmonger', 'dirsrv', 'httpd', 'ipa', 'krb5kdc',
+    'dbus', 'nslcd', 'nscd', 'ntpd', 'portmap',
+    'rpcbind', 'kadmin', 'sshd', 'autofs', 'rpcgssd',
+    'rpcidmapd', 'pki_tomcatd', 'chronyd', 'domainname',
+    'named', 'ods_enforcerd', 'ods_signerd', 'gssproxy',
+    'nfs-utils', 'sssd', 'NetworkManager', 'ipa-custodia',
+    'ipa-dnskeysyncd', 'ipa-otpd', 'ipa-ods-exporter',
+    'systemd-resolved',
+]
 
 # The common ports for these services. This is used to wait for the
 # service to become available.
@@ -96,7 +102,7 @@ class KnownServices(Mapping):
             raise AttributeError(name)
 
 
-class PlatformService(object):
+class PlatformService:
     """
     PlatformService abstracts out external process running on the system
     which is possible to administer (start, stop, check status, etc).
@@ -168,6 +174,9 @@ class PlatformService(object):
     def restart(self, instance_name="", capture_output=True, wait=True):
         pass
 
+    def try_restart(self, instance_name="", capture_output=True, wait=True):
+        pass
+
     def is_running(self, instance_name="", wait=True):
         return False
 
@@ -333,6 +342,10 @@ class SystemdService(PlatformService):
         self._restart_base(instance_name, "restart",
                            capture_output, wait)
 
+    def try_restart(self, instance_name="", capture_output=True, wait=True):
+        self._restart_base(instance_name, "try-restart",
+                           capture_output, wait)
+
     def is_running(self, instance_name="", wait=True):
         instance = self.service_instance(instance_name, 'is-active')
 
@@ -451,7 +464,7 @@ class SystemdService(PlatformService):
                         # Link exists and it is broken, make new one
                         os.unlink(srv_lnk)
                         os.symlink(self.lib_path, srv_lnk)
-                ipautil.run([paths.SYSTEMCTL, "--system", "daemon-reload"])
+                tasks.systemd_daemon_reload()
             except Exception:
                 pass
         else:
@@ -474,7 +487,7 @@ class SystemdService(PlatformService):
                 if os.path.isdir(srv_tgt):
                     if os.path.islink(srv_lnk):
                         os.unlink(srv_lnk)
-                ipautil.run([paths.SYSTEMCTL, "--system", "daemon-reload"])
+                tasks.systemd_daemon_reload()
             except Exception:
                 pass
         else:
