@@ -3,7 +3,10 @@
 #
 """Fedora container constants
 """
-from ipaplatform.fedora.constants import FedoraConstantsNamespace
+from ipaplatform.fedora.constants import FedoraConstantsNamespace, User, Group
+
+
+__all__ = ("constants", "User", "Group")
 
 
 class FedoraContainerConstantsNamespace(FedoraConstantsNamespace):
