@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-'''
+r'''
 
 Goal
 ----
@@ -423,10 +423,16 @@ import sys
 import functools
 
 import cryptography.x509
-from ldap.dn import str2dn, dn2str
-from ldap import DECODING_ERROR
 import six
 
+try:
+    from ldap import DECODING_ERROR
+except ImportError:
+    from ipapython.dn_ctypes import str2dn, dn2str, DECODING_ERROR
+else:
+    from ldap.dn import str2dn, dn2str
+
+
 if six.PY3:
     unicode = str
 
@@ -453,8 +459,8 @@ def _adjust_indices(start, end, length):
 def _normalize_ava_input(val):
     if six.PY3 and isinstance(val, bytes):
         raise TypeError('expected str, got bytes: %r' % val)
-    elif not isinstance(val, six.string_types):
-        val = val_encode(six.text_type(val))
+    elif not isinstance(val, str):
+        val = val_encode(str(val))
     elif six.PY2 and isinstance(val, unicode):
         val = val.encode('utf-8')
     return val
@@ -500,7 +506,7 @@ def get_ava(*args):
             if len(arg) != 2:
                 raise ValueError("tuple or list must be 2-valued, not \"%s\"" % (arg))
             ava = [_normalize_ava_input(arg[0]), _normalize_ava_input(arg[1]), 0]
-        elif isinstance(arg, six.string_types):
+        elif isinstance(arg, str):
             rdn = str2rdn(arg)
             if len(rdn) > 1:
                 raise TypeError("multiple AVA's specified by \"%s\"" % (arg))
@@ -557,7 +563,7 @@ else:
 
 
 @functools.total_ordering
-class AVA(object):
+class AVA:
     '''
     AVA(arg0, ...)
 
@@ -679,7 +685,7 @@ class AVA(object):
         caseIgnoreMatch.
         '''
         # Try coercing string to AVA, if successful compare to coerced object
-        if isinstance(other, six.string_types):
+        if isinstance(other, str):
             try:
                 other_ava = AVA(other)
                 return self.__eq__(other_ava)
@@ -706,7 +712,7 @@ class AVA(object):
 
 
 @functools.total_ordering
-class RDN(object):
+class RDN:
     '''
     RDN(arg0, ...)
 
@@ -824,7 +830,7 @@ class RDN(object):
 
         if raw:  # fast raw mode
             avas = args
-        elif ava_count == 1 and isinstance(args[0], six.string_types):
+        elif ava_count == 1 and isinstance(args[0], str):
             avas = str2rdn(args[0])
             sort = 1
         elif ava_count == 1 and isinstance(args[0], RDN):
@@ -860,11 +866,11 @@ class RDN(object):
         return len(self._avas)
 
     def __getitem__(self, key):
-        if isinstance(key, six.integer_types):
+        if isinstance(key, int):
             return self._get_ava(self._avas[key])
         if isinstance(key, slice):
             return [self._get_ava(ava) for ava in self._avas[key]]
-        elif isinstance(key, six.string_types):
+        elif isinstance(key, str):
             for ava in self._avas:
                 if key == val_decode(ava[0]):
                     return val_decode(ava[1])
@@ -882,7 +888,7 @@ class RDN(object):
         if len(self._avas) == 0:
             raise IndexError("No AVA's in this RDN")
 
-        self._avas[0][0] = val_encode(six.text_type(new_attr))
+        self._avas[0][0] = val_encode(str(new_attr))
 
     attr  = property(_get_attr)
 
@@ -894,7 +900,7 @@ class RDN(object):
     def _set_value(self, new_value):
         if len(self._avas) == 0:
             raise IndexError("No AVA's in this RDN")
-        self._avas[0][1] = val_encode(six.text_type(new_value))
+        self._avas[0][1] = val_encode(str(new_value))
 
     value = property(_get_value)
 
@@ -909,7 +915,7 @@ class RDN(object):
 
     def __eq__(self, other):
         # Try coercing string to RDN, if successful compare to coerced object
-        if isinstance(other, six.string_types):
+        if isinstance(other, str):
             try:
                 other_rdn = RDN(other)
                 return self.__eq__(other_rdn)
@@ -939,7 +945,7 @@ class RDN(object):
                 result._avas.append((ava[0], ava[1], ava[2]))
         elif isinstance(other, AVA):
             result._avas.append(other.to_openldap())
-        elif isinstance(other, six.string_types):
+        elif isinstance(other, str):
             rdn = self.__class__(other)
             for ava in rdn._avas:
                 result._avas.append((ava[0], ava[1], ava[2]))
@@ -951,7 +957,7 @@ class RDN(object):
 
 
 @functools.total_ordering
-class DN(object):
+class DN:
     '''
     DN(arg0, ...)
 
@@ -1112,9 +1118,9 @@ class DN(object):
         return [[list(a) for a in rdn] for rdn in rdns]
 
     def _rdns_from_value(self, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             try:
-                if isinstance(value, six.text_type):
+                if isinstance(value, str):
                     value = val_encode(value)
                 rdns = str2dn(value)
             except DECODING_ERROR:
@@ -1132,9 +1138,11 @@ class DN(object):
             rdns = list(reversed([
                 [get_ava(
                     ATTR_NAME_BY_OID.get(ava.oid, ava.oid.dotted_string),
-                    ava.value)]
-                for ava in value
+                    ava.value) for ava in rdn]
+                for rdn in value.rdns
             ]))
+            for rdn in rdns:
+                sort_avas(rdn)
         else:
             raise TypeError(
                 "must be str, unicode, tuple, Name, RDN or DN, got %s instead"
@@ -1178,14 +1186,14 @@ class DN(object):
         return len(self.rdns)
 
     def __getitem__(self, key):
-        if isinstance(key, six.integer_types):
+        if isinstance(key, int):
             return self._get_rdn(self.rdns[key])
         if isinstance(key, slice):
             cls = self.__class__
             new_dn = cls.__new__(cls)
             new_dn.rdns = self.rdns[key]
             return new_dn
-        elif isinstance(key, six.string_types):
+        elif isinstance(key, str):
             for rdn in self.rdns:
                 for ava in rdn:
                     if key == val_decode(ava[0]):
@@ -1212,7 +1220,7 @@ class DN(object):
 
     def __eq__(self, other):
         # Try coercing to DN, if successful compare to coerced object
-        if isinstance(other, (six.string_types, RDN, AVA)):
+        if isinstance(other, (str, RDN, AVA)):
             try:
                 other_dn = DN(other)
                 return self.__eq__(other_dn)
