@@ -20,6 +20,7 @@
 import collections
 import gzip
 import io
+import json
 import logging
 from urllib.parse import urlencode
 import xml.dom.minidom
@@ -56,6 +57,9 @@ INCLUDED_PROFILES = {
     Profile(u'KDCs_PKINIT_Certs',
             u'Profile for PKINIT support by KDCs',
             False),
+    Profile(u'acmeIPAServerCert',
+            u'ACME IPA service certificate profile',
+            False),
     }
 
 DEFAULT_PROFILE = u'caIPAserviceCert'
@@ -97,6 +101,10 @@ def get_ca_certchain(ca_host=None):
         data = res.read()
         conn.close()
         try:
+            doc = json.loads(data)
+            chain = doc['Response']['ChainBase64']
+        except (json.JSONDecodeError, KeyError):
+            logger.debug("Response is not valid JSON, try XML")
             doc = xml.dom.minidom.parseString(data)
             try:
                 item_node = doc.getElementsByTagName("ChainBase64")
@@ -104,9 +112,9 @@ def get_ca_certchain(ca_host=None):
             except IndexError:
                 raise error_from_xml(
                     doc, _("Retrieving CA cert chain failed: %s"))
-        finally:
-            if doc:
-                doc.unlink()
+            finally:
+                if doc:
+                    doc.unlink()
     else:
         raise errors.RemoteRetrieveError(
             reason=_("request failed with HTTP status %d") % res.status)
@@ -115,13 +123,18 @@ def get_ca_certchain(ca_host=None):
 
 
 def _parse_ca_status(body):
-    doc = xml.dom.minidom.parseString(body)
     try:
-        item_node = doc.getElementsByTagName("XMLResponse")[0]
-        item_node = item_node.getElementsByTagName("Status")[0]
-        return item_node.childNodes[0].data
-    except IndexError:
-        raise error_from_xml(doc, _("Retrieving CA status failed: %s"))
+        doc = json.loads(body)
+        return doc['Response']['Status']
+    except (json.JSONDecodeError, KeyError):
+        logger.debug("Response is not valid JSON, try XML")
+        doc = xml.dom.minidom.parseString(body)
+        try:
+            item_node = doc.getElementsByTagName("XMLResponse")[0]
+            item_node = item_node.getElementsByTagName("Status")[0]
+            return item_node.childNodes[0].data
+        except IndexError:
+            raise error_from_xml(doc, _("Retrieving CA status failed: %s"))
 
 
 def ca_status(ca_host=None):
@@ -147,6 +160,35 @@ def ca_status(ca_host=None):
     return _parse_ca_status(body)
 
 
+def acme_status(ca_host=None):
+    """Return the status of ACME
+
+    Returns a boolean.
+
+    If the proxy is not working or the CA is not running then this could
+    return a false negative.
+    """
+    if ca_host is None:
+        ca_host = api.env.ca_host
+    status, _headers, _body = https_request(
+        ca_host, 443,
+        url='/acme/directory',
+        cafile=api.env.tls_ca_cert,
+        client_certfile=None,
+        client_keyfile=None,
+        method='GET',
+        timeout=api.env.http_timeout)
+    if status == 200:
+        return True
+    elif status == 503:
+        # This is what it should return when disabled
+        return False
+    else:
+        # Unexpected status code, log and return False
+        logger.error('ACME status request returned %d', status)
+        return False
+
+
 def https_request(
         host, port, url, cafile, client_certfile, client_keyfile,
         method='POST', headers=None, body=None, **kw):
