@@ -15,13 +15,13 @@ from dns import (
     rdatatype,
     zone,
 )
-from dns.exception import DNSException
 
 from time import sleep, time
 
 from ipalib import errors
 from ipalib.dns import record_name_format
-from ipapython.dnsutil import DNSName, resolve_rrsets
+from ipapython.dnsutil import DNSName
+from ipaserver.install import installutils
 
 if six.PY3:
     unicode=str
@@ -31,23 +31,38 @@ logger = logging.getLogger(__name__)
 
 IPA_DEFAULT_MASTER_SRV_REC = (
     # srv record name, port
-    (DNSName(u'_ldap._tcp'), 389),
-    (DNSName(u'_kerberos._tcp'), 88),
-    (DNSName(u'_kerberos._udp'), 88),
-    (DNSName(u'_kerberos-master._tcp'), 88),
-    (DNSName(u'_kerberos-master._udp'), 88),
-    (DNSName(u'_kpasswd._tcp'), 464),
-    (DNSName(u'_kpasswd._udp'), 464),
+    (DNSName('_ldap._tcp'), 389),
+    # Kerberos records are provided for MIT KRB5 < 1.15 and AD
+    (DNSName('_kerberos._tcp'), 88),
+    (DNSName('_kerberos._udp'), 88),
+    (DNSName('_kerberos-master._tcp'), 88),
+    (DNSName('_kerberos-master._udp'), 88),
+    (DNSName('_kpasswd._tcp'), 464),
+    (DNSName('_kpasswd._udp'), 464),
+)
+
+IPA_DEFAULT_MASTER_URI_REC = (
+    # URI record name, URI template
+
+    # MIT KRB5 1.15+ prefers URI records for service discovery
+    # scheme: always krb5srv
+    # flags: empty or 'm' for primary server
+    # transport: 'tcp', 'udp', or 'kkdcp')
+    # residual: 'hostname', 'hostname:port', or 'https://' URL
+    (DNSName('_kerberos'), "krb5srv:m:tcp:{hostname}"),
+    (DNSName('_kerberos'), "krb5srv:m:udp:{hostname}"),
+    (DNSName('_kpasswd'), "krb5srv:m:tcp:{hostname}"),
+    (DNSName('_kpasswd'), "krb5srv:m:udp:{hostname}"),
 )
 
 IPA_DEFAULT_ADTRUST_SRV_REC = (
     # srv record name, port
-    (DNSName(u'_ldap._tcp.Default-First-Site-Name._sites.dc._msdcs'), 389),
-    (DNSName(u'_ldap._tcp.dc._msdcs'), 389),
-    (DNSName(u'_kerberos._tcp.Default-First-Site-Name._sites.dc._msdcs'), 88),
-    (DNSName(u'_kerberos._udp.Default-First-Site-Name._sites.dc._msdcs'), 88),
-    (DNSName(u'_kerberos._tcp.dc._msdcs'), 88),
-    (DNSName(u'_kerberos._udp.dc._msdcs'), 88),
+    (DNSName('_ldap._tcp.Default-First-Site-Name._sites.dc._msdcs'), 389),
+    (DNSName('_ldap._tcp.dc._msdcs'), 389),
+    (DNSName('_kerberos._tcp.Default-First-Site-Name._sites.dc._msdcs'), 88),
+    (DNSName('_kerberos._udp.Default-First-Site-Name._sites.dc._msdcs'), 88),
+    (DNSName('_kerberos._tcp.dc._msdcs'), 88),
+    (DNSName('_kerberos._udp.dc._msdcs'), 88),
 )
 
 IPA_DEFAULT_NTP_SRV_REC = (
@@ -55,18 +70,20 @@ IPA_DEFAULT_NTP_SRV_REC = (
     (DNSName("_ntp._udp"), 123),
 )
 
-CA_RECORDS_DNS_TIMEOUT = 30  # timeout in seconds
+CA_RECORDS_DNS_TIMEOUT = 15  # timeout in seconds
 
 
 class IPADomainIsNotManagedByIPAError(Exception):
     pass
 
 
-class IPASystemRecords(object):
+class IPASystemRecords:
 
     # fixme do it configurable
     PRIORITY_HIGH = 0
     PRIORITY_LOW = 50
+    # FIXME: use TTL from config
+    TTL = 3600
 
     def __init__(self, api_instance, all_servers=False):
         self.api_instance = api_instance
@@ -83,7 +100,7 @@ class IPASystemRecords(object):
         self.__init_data()
 
     def __get_server_attrs(self, server_result):
-        weight = int(server_result.get('ipaserviceweight', [u'100'])[0])
+        weight = int(server_result.get('ipaserviceweight', ['100'])[0])
         location = server_result.get('ipalocation_location', [None])[0]
         roles = set(server_result.get('enabled_role_servrole', ()))
 
@@ -98,7 +115,7 @@ class IPASystemRecords(object):
         kwargs = dict(no_members=False)
         if not all_servers:
             # only active, fully installed masters]
-            kwargs["servrole"] = u"IPA master"
+            kwargs["servrole"] = "IPA master"
         servers = self.api_instance.Command.server_find(**kwargs)
 
         for s in servers['result']:
@@ -134,21 +151,53 @@ class IPASystemRecords(object):
 
             rdataset = zone_obj.get_rdataset(
                 r_name, rdatatype.SRV, create=True)
-            rdataset.add(rd, ttl=86400)  # FIXME: use TTL from config
+            rdataset.add(rd, ttl=self.TTL)
+
+    def __add_uri_records(
+        self, zone_obj, hostname, rname_uri_map,
+        weight=100, priority=0, location=None
+    ):
+        assert isinstance(hostname, DNSName)
+        assert isinstance(priority, int)
+        assert isinstance(weight, int)
+
+        if location:
+            suffix = self.__get_location_suffix(location)
+        else:
+            suffix = self.domain_abs
+
+        for name, uri_template in rname_uri_map:
+            uri = uri_template.format(hostname=hostname.make_absolute())
+            rd = rdata.from_text(
+                rdataclass.IN, rdatatype.URI,
+                '{0} {1} {2}'.format(
+                    priority, weight, uri
+                )
+            )
+
+            r_name = name.derelativize(suffix)
+
+            rdataset = zone_obj.get_rdataset(
+                r_name, rdatatype.URI, create=True)
+            rdataset.add(rd, ttl=self.TTL)
 
     def __add_ca_records_from_hostname(self, zone_obj, hostname):
         assert isinstance(hostname, DNSName) and hostname.is_absolute()
         r_name = DNSName('ipa-ca') + self.domain_abs
-        rrsets = []
+        rrsets = None
         end_time = time() + CA_RECORDS_DNS_TIMEOUT
-        while time() < end_time:
+        while True:
             try:
-                rrsets = resolve_rrsets(hostname, (rdatatype.A, rdatatype.AAAA))
-            except DNSException:  # logging is done inside resolve_rrsets
+                # function logs errors
+                rrsets = installutils.resolve_rrsets_nss(hostname)
+            except OSError:
+                # also retry on EAI_AGAIN, EAI_FAIL
                 pass
             if rrsets:
                 break
-            sleep(5)
+            if time() >= end_time:
+                break
+            sleep(3)
 
         if not rrsets:
             logger.error('unable to resolve host name %s to IP address, '
@@ -159,7 +208,7 @@ class IPASystemRecords(object):
             for rd in rrset:
                 rdataset = zone_obj.get_rdataset(
                     r_name, rd.rdtype, create=True)
-                rdataset.add(rd, ttl=86400)  # FIXME: use TTL from config
+                rdataset.add(rd, ttl=self.TTL)
 
     def __add_kerberos_txt_rec(self, zone_obj):
         # FIXME: with external DNS, this should generate records for all
@@ -170,7 +219,7 @@ class IPASystemRecords(object):
         rdataset = zone_obj.get_rdataset(
             r_name, rdatatype.TXT, create=True
         )
-        rdataset.add(rd, ttl=86400)  # FIXME: use TTL from config
+        rdataset.add(rd, ttl=self.TTL)
 
     def _add_base_dns_records_for_server(
             self, zone_obj, hostname, roles=None, include_master_role=True,
@@ -194,6 +243,12 @@ class IPASystemRecords(object):
                 IPA_DEFAULT_MASTER_SRV_REC,
                 weight=server['weight']
             )
+            self.__add_uri_records(
+                zone_obj,
+                hostname_abs,
+                IPA_DEFAULT_MASTER_URI_REC,
+                weight=server['weight']
+            )
 
         if 'CA server' in eff_roles:
             self.__add_ca_records_from_hostname(zone_obj, hostname_abs)
@@ -240,6 +295,14 @@ class IPASystemRecords(object):
                     priority=priority,
                     location=location
                 )
+                self.__add_uri_records(
+                    zone_obj,
+                    hostname_abs,
+                    IPA_DEFAULT_MASTER_URI_REC,
+                    weight=server['weight'],
+                    priority=priority,
+                    location=location
+                )
 
             if 'AD trust controller' in eff_roles:
                 self.__add_srv_records(
@@ -277,10 +340,10 @@ class IPASystemRecords(object):
     ):
         update_dict = self.__prepare_records_update_dict(nodes)
         cname_template = {
-            'addattr': [u'objectclass=idnsTemplateObject'],
+            'addattr': ['objectclass=idnsTemplateObject'],
             'setattr': [
-                u'idnsTemplateAttribute;cnamerecord=%s'
-                u'.\{substitutionvariable_ipalocation\}._locations' %
+                r'idnsTemplateAttribute;cnamerecord=%s'
+                r'.\{substitutionvariable_ipalocation\}._locations' %
                 record_name.relativize(self.domain_abs)
             ]
         }
@@ -474,7 +537,7 @@ class IPASystemRecords(object):
         for rdataset in node:
             for rd in rdataset:
                 records.append(
-                    u'{name} {ttl} {rdclass} {rdtype} {rdata}'.format(
+                    '{name} {ttl} {rdclass} {rdtype} {rdata}'.format(
                         name=name.ToASCII(),
                         ttl=rdataset.ttl,
                         rdclass=rdataclass.to_text(rd.rdclass),
