@@ -12,13 +12,9 @@ from __future__ import print_function
 import enum
 import logging
 import os
-
-# absolute import is necessary because IPA module dns clashes with python-dns
-from dns import resolver
-import six
-
 import sys
 
+import six
 from subprocess import CalledProcessError
 
 from ipalib import api
@@ -26,6 +22,7 @@ from ipalib import errors
 from ipalib import util
 from ipalib.install import hostname, sysrestore
 from ipalib.install.service import enroll_only, prepare_only
+from ipalib.install import dnsforwarders
 from ipaplatform.paths import paths
 from ipaplatform.constants import constants
 from ipaplatform import services
@@ -111,14 +108,20 @@ def _disable_dnssec():
             conn.update_entry(entry)
 
 
+def package_check(exception):
+    if not os.path.isfile(paths.IPA_DNS_INSTALL):
+        raise exception(
+            "Integrated DNS requires '%s' package"
+            % constants.IPA_DNS_PACKAGE_NAME
+        )
+
+
 def install_check(standalone, api, replica, options, hostname):
     global ip_addresses
     global reverse_zones
     fstore = sysrestore.FileStore(paths.SYSRESTORE)
 
-    if not os.path.isfile(paths.IPA_DNS_INSTALL):
-        raise RuntimeError("Integrated DNS requires '%s' package" %
-                           constants.IPA_DNS_PACKAGE_NAME)
+    package_check(RuntimeError)
 
     # when installing first DNS instance we need to check zone overlap
     if replica or standalone:
@@ -151,7 +154,7 @@ def install_check(standalone, api, replica, options, hostname):
 
     if standalone:
         print("==============================================================================")
-        print("This program will setup DNS for the FreeIPA Server.")
+        print("This program will setup DNS for the IPA Server.")
         print("")
         print("This includes:")
         print("  * Configure DNS (bind)")
@@ -288,7 +291,7 @@ def install_check(standalone, api, replica, options, hostname):
         if not options.forwarders:
             options.forwarders = []
         if options.auto_forwarders:
-            options.forwarders += resolver.get_default_resolver().nameservers
+            options.forwarders.extend(dnsforwarders.get_nameservers())
     elif standalone or not replica:
         options.forwarders = read_dns_forwarders()
 
