@@ -22,9 +22,7 @@ from __future__ import print_function, absolute_import
 
 import logging
 import shutil
-import pwd
 import os
-import time
 import tempfile
 import fnmatch
 
@@ -47,6 +45,7 @@ from ipaserver.install import certs
 from ipaserver.install import replication
 from ipaserver.install import sysupgrade
 from ipaserver.install import upgradeinstance
+from ipaserver.install import ldapupdate
 from ipalib import api
 from ipalib import errors
 from ipalib import constants
@@ -67,6 +66,7 @@ IPA_SCHEMA_FILES = ("60kerberos.ldif",
                     "60ipaconfig.ldif",
                     "60basev2.ldif",
                     "60basev3.ldif",
+                    "60basev4.ldif",
                     "60ipapk11.ldif",
                     "60ipadns.ldif",
                     "60certificate-profiles.ldif",
@@ -215,6 +215,8 @@ class DsInstance(service.Service):
         if realm_name:
             self.suffix = ipautil.realm_to_suffix(self.realm)
             self.serverid = ipaldap.realm_to_serverid(self.realm)
+            if self.domain is None:
+                self.domain = self.realm.lower()
             self.__setup_sub_dict()
         else:
             self.suffix = DN()
@@ -225,11 +227,13 @@ class DsInstance(service.Service):
     def __common_setup(self):
 
         self.step("creating directory server instance", self.__create_instance)
-        self.step("configure autobind for root", self.__root_autobind)
         self.step("tune ldbm plugin", self.__tune_ldbm)
-        self.step("stopping directory server", self.__stop_instance)
-        self.step("updating configuration in dse.ldif", self.__update_dse_ldif)
-        self.step("starting directory server", self.__start_instance)
+        if self.config_ldif is not None:
+            self.step("stopping directory server", self.__stop_instance)
+            self.step(
+                "updating configuration in dse.ldif", self.__update_dse_ldif
+            )
+            self.step("starting directory server", self.__start_instance)
         self.step("adding default schema", self.__add_default_schemas)
         self.step("enabling memberof plugin", self.__add_memberof_module)
         self.step("enabling winsync plugin", self.__add_winsync_module)
@@ -496,34 +500,22 @@ class DsInstance(service.Service):
 
     def __setup_sub_dict(self):
         server_root = find_server_root()
-        try:
-            idrange_size = self.idmax - self.idstart + 1
-        except TypeError:
-            idrange_size = None
-        self.sub_dict = dict(
-            FQDN=self.fqdn, SERVERID=self.serverid,
+        self.sub_dict = ldapupdate.get_sub_dict(
+            realm=self.realm,
+            domain=self.domain,
+            suffix=self.suffix,
+            fqdn=self.fqdn,
+            idstart=self.idstart,
+            idmax=self.idmax,
+        )
+        self.sub_dict.update(
+            DOMAIN_LEVEL=self.domainlevel,
+            SERVERID=self.serverid,
             PASSWORD=self.dm_password,
             RANDOM_PASSWORD=ipautil.ipa_generate_password(),
-            SUFFIX=self.suffix,
-            REALM=self.realm, USER=DS_USER,
-            SERVER_ROOT=server_root, DOMAIN=self.domain,
-            TIME=int(time.time()), IDSTART=self.idstart,
-            IDMAX=self.idmax, HOST=self.fqdn,
-            ESCAPED_SUFFIX=str(self.suffix),
+            USER=DS_USER,
             GROUP=DS_GROUP,
-            IDRANGE_SIZE=idrange_size,
-            DOMAIN_LEVEL=self.domainlevel,
-            MAX_DOMAIN_LEVEL=constants.MAX_DOMAIN_LEVEL,
-            MIN_DOMAIN_LEVEL=constants.MIN_DOMAIN_LEVEL,
-            STRIP_ATTRS=" ".join(replication.STRIP_ATTRS),
-            EXCLUDES='(objectclass=*) $ EXCLUDE ' +
-            ' '.join(replication.EXCLUDES),
-            TOTAL_EXCLUDES='(objectclass=*) $ EXCLUDE ' +
-            ' '.join(replication.TOTAL_EXCLUDES),
-            DEFAULT_SHELL=platformconstants.DEFAULT_SHELL,
-            DEFAULT_ADMIN_SHELL=platformconstants.DEFAULT_ADMIN_SHELL,
-            SELINUX_USERMAP_DEFAULT=platformconstants.SELINUX_USERMAP_DEFAULT,
-            SELINUX_USERMAP_ORDER=platformconstants.SELINUX_USERMAP_ORDER,
+            SERVER_ROOT=server_root,
         )
 
     def __create_instance(self):
@@ -545,6 +537,7 @@ class DsInstance(service.Service):
         slapd_options = Slapd2Base(logger)
         slapd_options.set('instance_name', self.serverid)
         slapd_options.set('root_password', self.dm_password)
+        slapd_options.set('self_sign_cert', False)
         slapd_options.verify()
         slapd = slapd_options.collect()
 
@@ -560,27 +553,22 @@ class DsInstance(service.Service):
         sds.create_from_args(general, slapd, backends, None)
 
         # Now create the new domain root object in the format that IPA expects.
-        # Get the instance ....
-
+        # Get the instance and setup LDAPI with root autobind.
         inst = DirSrv(verbose=True, external_log=logger)
         inst.local_simple_allocate(
             serverid=self.serverid,
             ldapuri=ipaldap.get_ldap_uri(realm=self.realm, protocol='ldapi'),
-            password=self.dm_password
         )
-
-        # local_simple_allocate() configures LDAPI but doesn't set up the
-        # DirSrv object to use LDAPI. Modify the DirSrv() object to use
-        # LDAPI with password bind. autobind is not available, yet.
-        inst.ldapi_enabled = 'on'
-        inst.ldapi_socket = paths.SLAPD_INSTANCE_SOCKET_TEMPLATE % (
-            self.serverid
-        )
-        inst.ldapi_autobind = 'off'
-
-        # This actually opens the conn and binds.
+        inst.setup_ldapi()
         inst.open()
 
+        def get_entry(dn, attrs):
+            return inst.getEntry(str(dn), attrlist=attrs)
+
+        self.sub_dict['REPLICATION_PLUGIN'] = (
+            installutils.get_replication_plugin_name(get_entry)
+        )
+
         try:
             ipadomain = IpaDomain(inst, dn=self.suffix.ldap_text())
             ipadomain.create(properties={
@@ -628,14 +616,13 @@ class DsInstance(service.Service):
             logger.debug("Failed to clean temporary file: %s", e)
 
     def __add_default_schemas(self):
-        pent = pwd.getpwnam(DS_USER)
         for schema_fname in IPA_SCHEMA_FILES:
             target_fname = schema_dirname(self.serverid) + schema_fname
             shutil.copyfile(
                 os.path.join(paths.USR_SHARE_IPA_DIR, schema_fname),
                 target_fname)
             os.chmod(target_fname, 0o440)    # read access for dirsrv user/group
-            os.chown(target_fname, pent.pw_uid, pent.pw_gid)
+            DS_USER.chown(target_fname)
 
         try:
             shutil.move(schema_dirname(self.serverid) + "05rfc2247.ldif",
@@ -646,7 +633,7 @@ class DsInstance(service.Service):
                 os.path.join(paths.USR_SHARE_IPA_DIR, "05rfc2247.ldif"),
                 target_fname)
             os.chmod(target_fname, 0o440)
-            os.chown(target_fname, pent.pw_uid, pent.pw_gid)
+            DS_USER.chown(target_fname)
         except IOError:
             # Does not apply with newer DS releases
             pass
@@ -666,7 +653,8 @@ class DsInstance(service.Service):
         )
 
     def restart(self, instance_name="", capture_output=True, wait=True):
-        api.Backend.ldap2.disconnect()
+        if api.Backend.ldap2.isconnected():
+            api.Backend.ldap2.disconnect()
         try:
             super(DsInstance, self).restart(
                 instance_name, capture_output=capture_output, wait=wait
@@ -745,7 +733,7 @@ class DsInstance(service.Service):
         self._ldap_mod("pw-logging-conf.ldif")
 
     def __config_version_module(self):
-        self._ldap_mod("version-conf.ldif")
+        self._ldap_mod("version-conf.ldif", self.sub_dict)
 
     def __config_uuid_module(self):
         self._ldap_mod("uuid-conf.ldif")
@@ -770,13 +758,12 @@ class DsInstance(service.Service):
         self._ldap_mod("repoint-managed-entries.ldif", self.sub_dict)
 
     def configure_systemd_ipa_env(self):
-        pent = pwd.getpwnam(platformconstants.DS_USER)
         template = os.path.join(
             paths.USR_SHARE_IPA_DIR, "ds-ipa-env.conf.template"
         )
         sub_dict = dict(
             KRB5_KTNAME=paths.DS_KEYTAB,
-            KRB5CCNAME=paths.TMP_KRB5CC % pent.pw_uid
+            KRB5CCNAME=paths.TMP_KRB5CC % platformconstants.DS_USER.uid
         )
         conf = ipautil.template_file(template, sub_dict)
 
@@ -1025,7 +1012,7 @@ class DsInstance(service.Service):
         __add_principal('ipa-ldap-delegation-targets', 'ldap', self)
 
     def __create_indices(self):
-        self._ldap_mod("indices.ldif")
+        self._ldap_update(["20-indices.update"])
 
     def __certmap_conf(self):
         write_certmap_conf(self.realm, self.ca_subject)
@@ -1249,14 +1236,6 @@ class DsInstance(service.Service):
 
         return status
 
-    def __root_autobind(self):
-        self._ldap_mod(
-            "root-autobind.ldif",
-            ldap_uri=ipaldap.get_ldap_uri(realm=self.realm, protocol='ldapi'),
-            # must simple bind until auto bind is configured
-            dm_password=self.dm_password
-        )
-
     def __add_sudo_binduser(self):
         self._ldap_mod("sudobind.ldif", self.sub_dict)
 
