@@ -39,7 +39,7 @@ from contextlib import contextmanager
 from configparser import ConfigParser as SafeConfigParser
 from configparser import NoOptionError
 
-from dns import rdatatype
+from dns import rrset, rdatatype, rdataclass
 from dns.exception import DNSException
 import ldap
 import six
@@ -50,12 +50,12 @@ import ipaplatform
 from ipapython import ipautil, admintool, version, ipaldap
 from ipapython.admintool import ScriptError, SERVER_NOT_CONFIGURED  # noqa: E402
 from ipapython.certdb import EXTERNAL_CA_TRUST_FLAGS
-from ipalib.constants import MAXHOSTNAMELEN
+from ipalib.constants import FQDN, MAXHOSTNAMELEN
 from ipalib.util import validate_hostname
 from ipalib import api, errors, x509
 from ipalib.install import dnsforwarders
 from ipapython.dn import DN
-from ipapython.dnsutil import resolve
+from ipapython.dnsutil import DNSName, resolve
 from ipaserver.install import certs, service, sysupgrade
 from ipaplatform import services
 from ipaplatform.paths import paths
@@ -118,16 +118,16 @@ class ReplicaConfig:
 
     subject_base = ipautil.dn_attribute_property('_subject_base')
 
+
 def get_fqdn():
-    fqdn = ""
-    try:
-        fqdn = socket.getfqdn()
-    except Exception:
-        try:
-            fqdn = socket.gethostname()
-        except Exception:
-            fqdn = ""
-    return fqdn
+    """Get fully qualified domain name of current host
+
+    :note: used by ansible_freeipa
+    :deprecated: use ipalib.constants.FQDN
+    :return: str
+    """
+    return FQDN
+
 
 def verify_fqdn(host_name, no_host_dns=False, local_hostname=True):
     """
@@ -479,18 +479,47 @@ def resolve_ip_addresses_nss(fqdn):
     logger.debug('Name %s resolved to %s', fqdn, ip_addresses)
     return ip_addresses
 
-def get_host_name(no_host_dns):
-    """
-    Get the current FQDN from the socket and verify that it is valid.
 
-    no_host_dns is a boolean that determines whether we enforce that the
-    hostname is resolvable.
+def resolve_rrsets_nss(fqdn):
+    """Get list of dnspython RRsets from NSS"""
+    if not isinstance(fqdn, DNSName):
+        fqdn = DNSName.from_text(fqdn)
+
+    ip_addresses = resolve_ip_addresses_nss(fqdn.to_text())
+
+    # split IP addresses into IPv4 and IPv6
+    ipv4 = []
+    ipv6 = []
+    for ip_address in ip_addresses:
+        # Skip reserved or link-local addresses
+        try:
+            ipautil.CheckedIPAddress(ip_address)
+        except ValueError as e:
+            logger.warning("Invalid IP address %s for %s: %s",
+                           ip_address, fqdn, unicode(e))
+            continue
+        if ip_address.version == 4:
+            ipv4.append(str(ip_address))
+        elif ip_address.version == 6:
+            ipv6.append(str(ip_address))
+
+    # construct an RRset for each address type. TTL is irrelevant
+    ttl = 3600
+    rrs = []
+    if ipv4:
+        rrs.append(
+            rrset.from_text_list(
+                fqdn, ttl, rdataclass.IN, rdatatype.A, ipv4
+            )
+        )
+    if ipv6:
+        rrs.append(
+            rrset.from_text_list(
+                fqdn, ttl, rdataclass.IN, rdatatype.AAAA, ipv6
+            )
+        )
+    return rrs
 
-    Will raise a RuntimeError on error, returns hostname on success
-    """
-    hostname = get_fqdn()
-    verify_fqdn(hostname, no_host_dns)
-    return hostname
 
 def get_server_ip_address(host_name, unattended, setup_dns, ip_addresses):
     hostaddr = resolve_ip_addresses_nss(host_name)
@@ -1050,27 +1079,55 @@ def check_available_memory(ca=False):
 
     Use Kb instead of KiB to leave a bit of slush for the OS
     """
+    available = None
     minimum_suggested = 1000 * 1000 * 1000 * 1.2
     if not ca:
         minimum_suggested -= 150 * 1000 * 1000
     if in_container():
+        logger.debug("container detected")
+        # cgroup v1
         if os.path.exists(
             '/sys/fs/cgroup/memory/memory.limit_in_bytes'
         ) and os.path.exists('/sys/fs/cgroup/memory/memory.usage_in_bytes'):
-            with open('/sys/fs/cgroup/memory/memory.limit_in_bytes') as fd:
-                limit = int(fd.readline())
-            with open('/sys/fs/cgroup/memory/memory.usage_in_bytes') as fd:
-                used = int(fd.readline())
-            available = limit - used
+            logger.debug("cgroup v1")
+            limit_file = '/sys/fs/cgroup/memory/memory.limit_in_bytes'
+            usage_file = '/sys/fs/cgroup/memory/memory.usage_in_bytes'
+        # cgroup v2
+        elif os.path.exists(
+            '/sys/fs/cgroup/memory.current'
+        ) and os.path.exists('/sys/fs/cgroup/memory.max'):
+            logger.debug("cgroup v2")
+            limit_file = '/sys/fs/cgroup/memory.max'
+            usage_file = '/sys/fs/cgroup/memory.current'
         else:
             raise ScriptError(
                 "Unable to determine the amount of available RAM"
             )
-    else:
+
+        with open(limit_file) as fd:
+            limit = fd.readline().rstrip()
+        with open(usage_file) as fd:
+            used = int(fd.readline())
+
+        # In cgroup v2 if there is no limit on the container then
+        # the maximum host memory is available. Fall back to the psutil
+        # method for determining availability.
+        if limit != 'max':
+            available = int(limit) - used
+
+        logger.debug("Max RAM %s, used RAM %s", limit, used)
+
+    if available is None:
         # delay import of psutil. On import it opens files in /proc and
         # can trigger a SELinux violation.
         import psutil
         available = psutil.virtual_memory().available
+        logger.debug(psutil.virtual_memory())
+
+    if available is None:
+        raise ScriptError(
+            "Unable to determine the amount of available RAM"
+        )
     logger.debug("Available memory is %sB", available)
     if available < minimum_suggested:
         raise ScriptError(
@@ -1502,3 +1559,31 @@ def validate_mask():
     if mask & 0b111101101 > 0:
         mask_str = "{:04o}".format(mask)
     return mask_str
+
+
+def get_replication_plugin_name(dirsrv_get_entry):
+    # Support renaming of a replication plugin in 389-ds
+    # IPA topology plugin depends on the replication plugin but
+    # 389-ds cannot handle older alias querying in the plugin
+    # configuration with 'nsslapd-plugin-depends-on-named: ..' attribute
+    #
+    # dirsrv_get_entry: function (dn, attrs) can return different types
+    # depending on the function. The 389-ds connection returns bytes
+    # and ipaldap will return a list of string values.
+    try:
+        entry = dirsrv_get_entry(
+            DN('cn=Multisupplier Replication Plugin,cn=plugins,cn=config'),
+            ['cn'])
+    except (errors.NotFound, ldap.NO_SUCH_OBJECT):
+        return 'Multimaster Replication Plugin'
+    else:
+        cn = entry['cn']
+        if isinstance(cn, list):
+            return cn[0]
+        elif isinstance(cn, bytes):
+            return cn.decode('utf-8')
+        else:
+            raise RuntimeError(
+                'LDAP query returned unknown type for cn %s: %s' %
+                (cn, type(cn))
+            )
