@@ -24,13 +24,16 @@ import os
 from optparse import OptionGroup  # pylint: disable=deprecated-module
 import gssapi
 
-from ipalib.constants import RENEWAL_CA_NAME, RENEWAL_REUSE_CA_NAME, IPA_CA_CN
+from ipalib.constants import (
+    RENEWAL_CA_NAME, RENEWAL_REUSE_CA_NAME, RENEWAL_SELFSIGNED_CA_NAME,
+    IPA_CA_CN)
 from ipalib.install import certmonger, certstore
 from ipapython import admintool, ipautil
 from ipapython.certdb import (EMPTY_TRUST_FLAGS,
                               EXTERNAL_CA_TRUST_FLAGS,
                               TrustFlags,
-                              parse_trust_flags)
+                              parse_trust_flags,
+                              get_ca_nickname)
 from ipapython.dn import DN
 from ipaplatform.paths import paths
 from ipalib import api, errors, x509
@@ -42,7 +45,8 @@ logger = logging.getLogger(__name__)
 class CACertManage(admintool.AdminTool):
     command_name = 'ipa-cacert-manage'
 
-    usage = "%prog renew [options]\n%prog install [options] CERTFILE"
+    usage = "%prog renew [options]\n%prog install [options] CERTFILE\n" \
+            "%prog delete [options] NICKNAME\n%prog list"
 
     description = "Manage CA certificates."
 
@@ -65,7 +69,7 @@ class CACertManage(admintool.AdminTool):
             "--external-ca", dest='self_signed',
             action='store_false',
             help="Sign the renewed certificate by external CA")
-        ext_cas = tuple(x.value for x in cainstance.ExternalCAType)
+        ext_cas = tuple(x.value for x in x509.ExternalCAType)
         renew_group.add_option(
             "--external-ca-type", dest="external_ca_type",
             type="choice", choices=ext_cas,
@@ -73,7 +77,7 @@ class CACertManage(admintool.AdminTool):
             help="Type of the external CA. Default: generic")
         renew_group.add_option(
             "--external-ca-profile", dest="external_ca_profile",
-            type='constructor', constructor=cainstance.ExternalCAProfile,
+            type='constructor', constructor=x509.ExternalCAProfile,
             default=None, metavar="PROFILE-SPEC",
             help="Specify the certificate profile/template to use "
                  "at the external CA")
@@ -93,6 +97,12 @@ class CACertManage(admintool.AdminTool):
             help="Trust flags for the certificate in certutil format")
         parser.add_option_group(install_group)
 
+        delete_group = OptionGroup(parser, "Delete options")
+        delete_group.add_option(
+            "-f", "--force", action='store_true',
+            help="Force removing the CA even if chain validation fails")
+        parser.add_option_group(delete_group)
+
     def validate_options(self):
         super(CACertManage, self).validate_options(needs_root=True)
 
@@ -112,6 +122,9 @@ class CACertManage(admintool.AdminTool):
                 parser.error("certificate file name not provided")
         elif command == 'list':
             pass
+        elif command == 'delete':
+            if len(self.args) < 2:
+                parser.error("nickname not provided")
         else:
             parser.error("unknown command \"%s\"" % command)
 
@@ -130,6 +143,8 @@ class CACertManage(admintool.AdminTool):
                 return self.install()
             elif command == 'list':
                 return self.list()
+            elif command == 'delete':
+                return self.delete()
             else:
                 raise NotImplementedError
         finally:
@@ -210,7 +225,7 @@ class CACertManage(admintool.AdminTool):
         except errors.NotFound:
             raise admintool.ScriptError("CA renewal master not found")
 
-        self.resubmit_request()
+        self.resubmit_request(RENEWAL_SELFSIGNED_CA_NAME)
 
         db = certs.CertDB(api.env.realm, nssdir=paths.PKI_TOMCAT_ALIAS_DIR)
         cert = db.get_cert_from_db(self.cert_nickname)
@@ -224,11 +239,11 @@ class CACertManage(admintool.AdminTool):
         options = self.options
 
         if not options.external_ca_type:
-            options.external_ca_type = cainstance.ExternalCAType.GENERIC.value
+            options.external_ca_type = x509.ExternalCAType.GENERIC.value
 
-        if options.external_ca_type == cainstance.ExternalCAType.MS_CS.value \
+        if options.external_ca_type == x509.ExternalCAType.MS_CS.value \
                 and options.external_ca_profile is None:
-            options.external_ca_profile = cainstance.MSCSTemplateV1(u"SubCA")
+            options.external_ca_profile = x509.MSCSTemplateV1(u"SubCA")
 
         if options.external_ca_profile is not None:
             # check that profile is valid for the external ca type
@@ -352,11 +367,11 @@ class CACertManage(admintool.AdminTool):
         timeout = api.env.startup_timeout + 60
 
         cm_profile = None
-        if isinstance(profile, cainstance.MSCSTemplateV1):
+        if isinstance(profile, x509.MSCSTemplateV1):
             cm_profile = profile.unparsed_input
 
         cm_template = None
-        if isinstance(profile, cainstance.MSCSTemplateV2):
+        if isinstance(profile, x509.MSCSTemplateV2):
             cm_template = profile.unparsed_input
 
         logger.debug("resubmitting certmonger request '%s'", self.request_id)
@@ -466,6 +481,71 @@ class CACertManage(admintool.AdminTool):
         for _ca_cert, ca_nickname, _ca_trust_flags in ca_certs:
             print(ca_nickname)
 
+    def delete(self):
+        options = self.options
+        nickname = self.args[1]
+        conn = api.Backend.ldap2
+
+        ca_certs = certstore.get_ca_certs_nss(api.Backend.ldap2,
+                                              api.env.basedn,
+                                              api.env.realm,
+                                              False)
+
+        ipa_ca_nickname = get_ca_nickname(api.env.realm)
+
+        found = False
+        for _ca_cert, ca_nickname, _ca_trust_flags in ca_certs:
+            if ca_nickname == nickname:
+                if ca_nickname == ipa_ca_nickname:
+                    raise admintool.ScriptError(
+                        'The IPA CA cannot be removed with this tool'
+                    )
+                else:
+                    found = True
+                    break
+
+        if not found:
+            raise admintool.ScriptError(
+                'Unknown CA \'{}\''.format(nickname)
+            )
+
+        with certs.NSSDatabase() as tmpdb:
+            tmpdb.create_db()
+            for ca_cert, ca_nickname, ca_trust_flags in ca_certs:
+                tmpdb.add_cert(ca_cert, ca_nickname, ca_trust_flags)
+            loaded = tmpdb.list_certs()
+            logger.debug("loaded raw certs '%s'", loaded)
+
+            tmpdb.delete_cert(nickname)
+
+            for ca_nickname, _trust_flags in loaded:
+                if ca_nickname == nickname:
+                    continue
+                if ipa_ca_nickname == nickname:
+                    raise admintool.ScriptError(
+                        "The IPA CA cannot be removed")
+                logger.debug("Verifying %s", ca_nickname)
+                try:
+                    tmpdb.verify_ca_cert_validity(ca_nickname)
+                except ValueError as e:
+                    msg = "Verifying \'%s\' failed. Removing part of the " \
+                          "chain? %s" % (nickname, e)
+                    if options.force:
+                        print(msg)
+                        continue
+                    raise admintool.ScriptError(msg)
+                else:
+                    logger.debug("Verified %s", ca_nickname)
+
+        for _ca_cert, ca_nickname, _ca_trust_flags in ca_certs:
+            if ca_nickname == nickname:
+                container_dn = DN(('cn', 'certificates'), ('cn', 'ipa'),
+                                  ('cn', 'etc'), api.env.basedn)
+                dn = DN(('cn', nickname), container_dn)
+                logger.debug("Deleting %s", ca_nickname)
+                conn.delete_entry(dn)
+                return
+
 
 def update_ipa_ca_entry(api, cert):
     """
