@@ -8,6 +8,7 @@ KRA installer module
 
 from __future__ import absolute_import
 
+import logging
 import os
 
 from ipalib import api
@@ -16,13 +17,15 @@ from ipaplatform import services
 from ipaplatform.paths import paths
 from ipapython import ipautil
 from ipapython.install.core import group
-from ipaserver.install import cainstance
+from ipaserver.install import ca, cainstance
 from ipaserver.install import krainstance
 from ipaserver.install import dsinstance
 from ipaserver.install import service as _service
 
 from . import dogtag
 
+logger = logging.getLogger(__name__)
+
 
 def install_check(api, replica_config, options):
     if replica_config is not None and not replica_config.setup_kra:
@@ -50,6 +53,11 @@ def install_check(api, replica_config, options):
                 "KRA is not installed on the master system. Please use "
                 "'ipa-kra-install' command to install the first instance.")
 
+    if api.env.ca_host is not None and api.env.ca_host != api.env.host:
+        raise RuntimeError(
+            "KRA can not be installed when 'ca_host' is overriden in "
+            "IPA configuration file.")
+
 
 def install(api, replica_config, options, custodia):
     if replica_config is None:
@@ -86,10 +94,13 @@ def install(api, replica_config, options, custodia):
         master_host = replica_config.kra_host_name
         promote = True
 
+    ca_subject = ca.lookup_ca_subject(api, subject_base)
+
     kra = krainstance.KRAInstance(realm_name)
     kra.configure_instance(
         realm_name, host_name, dm_password, dm_password,
         subject_base=subject_base,
+        ca_subject=ca_subject,
         pkcs12_info=pkcs12_info,
         master_host=master_host,
         promote=promote,
@@ -103,13 +114,29 @@ def install(api, replica_config, options, custodia):
 
     # Restart apache for new proxy config file
     services.knownservices.httpd.restart(capture_output=True)
-    # Restarted named-pkcs11 to restore bind-dyndb-ldap operation, see
+    # Restarted named to restore bind-dyndb-ldap operation, see
     # https://pagure.io/freeipa/issue/5813
-    named = services.knownservices.named  # alias for named-pkcs11
+    named = services.knownservices.named  # alias for current named
     if named.is_running():
         named.restart(capture_output=True)
 
 
+def uninstall_check(options):
+    """IPA needs to be running so pkidestroy can unregister KRA"""
+    kra = krainstance.KRAInstance(api.env.realm)
+    if not kra.is_installed():
+        return
+
+    result = ipautil.run([paths.IPACTL, 'status'],
+                         raiseonerr=False)
+
+    if result.returncode not in [0, 4]:
+        try:
+            ipautil.run([paths.IPACTL, 'start'])
+        except Exception:
+            logger.info("Re-starting IPA failed, continuing uninstall")
+
+
 def uninstall():
     kra = krainstance.KRAInstance(api.env.realm)
     kra.stop_tracking_certificates()
