@@ -29,13 +29,12 @@ import sys
 import uuid
 import time
 import os
-import pwd
 import fnmatch
+import warnings
 
-import ldap
+from pysss_murmur import murmurhash3  # pylint: disable=no-name-in-module
 import six
 
-from ipaserver.install import installutils
 from ipapython import ipautil, ipaldap
 from ipalib import errors
 from ipalib import api, create_api
@@ -44,6 +43,7 @@ from ipaplatform.constants import constants as platformconstants
 from ipaplatform.paths import paths
 from ipaplatform.tasks import tasks
 from ipapython.dn import DN
+from ipaserver.install import installutils, replication
 
 if six.PY3:
     unicode = str
@@ -54,7 +54,66 @@ UPDATES_DIR=paths.UPDATES_DIR
 UPDATE_SEARCH_TIME_LIMIT = 30  # seconds
 
 
-def connect(ldapi=False, realm=None, fqdn=None, dm_password=None):
+def get_sub_dict(realm, domain, suffix, fqdn, idstart=None, idmax=None):
+    """LDAP template substitution dict for installer and updater
+    """
+    if idstart is None:
+        idrange_size = None
+        subid_base_rid = None
+    else:
+        idrange_size = idmax - idstart + 1
+        subid_base_rid = constants.SUBID_RANGE_START - idrange_size
+
+    # uid / gid for autobind
+    # user is only defined when ipa-server-dns and bind are installed
+    try:
+        named_uid = platformconstants.NAMED_USER.uid
+        named_gid = platformconstants.NAMED_GROUP.gid
+    except ValueError:
+        named_uid = None
+        named_gid = None
+
+    return dict(
+        REALM=realm,
+        DOMAIN=domain,
+        SUFFIX=suffix,
+        ESCAPED_SUFFIX=str(suffix),
+        FQDN=fqdn,
+        HOST=fqdn,
+        LIBARCH=paths.LIBARCH,
+        TIME=int(time.time()),
+        FIPS="#" if tasks.is_fips_enabled() else "",
+        # idstart, idmax, and idrange_size may be None
+        IDSTART=idstart,
+        IDMAX=idmax,
+        IDRANGE_SIZE=idrange_size,
+        SUBID_COUNT=constants.SUBID_COUNT,
+        SUBID_RANGE_START=constants.SUBID_RANGE_START,
+        SUBID_RANGE_SIZE=constants.SUBID_RANGE_SIZE,
+        SUBID_RANGE_MAX=constants.SUBID_RANGE_MAX,
+        SUBID_DNA_THRESHOLD=constants.SUBID_DNA_THRESHOLD,
+        SUBID_BASE_RID=subid_base_rid,
+        DOMAIN_HASH=murmurhash3(domain, len(domain), 0xdeadbeef),
+        MAX_DOMAIN_LEVEL=constants.MAX_DOMAIN_LEVEL,
+        MIN_DOMAIN_LEVEL=constants.MIN_DOMAIN_LEVEL,
+        STRIP_ATTRS=" ".join(replication.STRIP_ATTRS),
+        EXCLUDES=(
+            '(objectclass=*) $ EXCLUDE ' + ' '.join(replication.EXCLUDES)
+        ),
+        TOTAL_EXCLUDES=(
+            '(objectclass=*) $ EXCLUDE '
+            + ' '.join(replication.TOTAL_EXCLUDES)
+        ),
+        DEFAULT_SHELL=platformconstants.DEFAULT_SHELL,
+        DEFAULT_ADMIN_SHELL=platformconstants.DEFAULT_ADMIN_SHELL,
+        SELINUX_USERMAP_DEFAULT=platformconstants.SELINUX_USERMAP_DEFAULT,
+        SELINUX_USERMAP_ORDER=platformconstants.SELINUX_USERMAP_ORDER,
+        NAMED_UID=named_uid,
+        NAMED_GID=named_gid,
+    )
+
+
+def connect(ldapi=False, realm=None, fqdn=None):
     """Create a connection for updates"""
     if ldapi:
         conn = ipaldap.LDAPClient.from_realm(realm, decode_attrs=False)
@@ -63,10 +122,7 @@ def connect(ldapi=False, realm=None, fqdn=None, dm_password=None):
             fqdn, decode_attrs=False
         )
     try:
-        if dm_password:
-            conn.simple_bind(bind_dn=ipaldap.DIRMAN_DN,
-                             bind_password=dm_password)
-        elif os.getegid() == 0:
+        if os.getegid() == 0:
             try:
                 # autobind
                 conn.external_bind()
@@ -75,13 +131,12 @@ def connect(ldapi=False, realm=None, fqdn=None, dm_password=None):
                 conn.gssapi_bind()
         else:
             conn.gssapi_bind()
-    except (ldap.CONNECT_ERROR, ldap.SERVER_DOWN):
-        raise RuntimeError("Unable to connect to LDAP server %s" % fqdn)
-    except ldap.INVALID_CREDENTIALS:
+    except (errors.DatabaseError, errors.NetworkError) as e:
+        raise RuntimeError("Unable to connect to LDAP server: %s" % e)
+    except errors.ACIError as e:
         raise RuntimeError(
-            "The password provided is incorrect for LDAP server %s" % fqdn)
-    except ldap.LOCAL_ERROR as e:
-        raise RuntimeError('%s' % e.args[0].get('info', '').strip())
+            "The password provided is incorrect for LDAP server %s: %s" %
+            (fqdn, e))
     return conn
 
 
@@ -139,28 +194,33 @@ def safe_output(attr, values):
     return values
 
 
+_sentinel = object()
+
+
 class LDAPUpdate:
-    action_keywords = [
+    action_keywords = {
         "default", "add", "remove", "only", "onlyifexist", "deleteentry",
         "replace", "addifnew", "addifexist"
-    ]
+    }
     index_suffix = DN(
         ('cn', 'index'), ('cn', 'userRoot'), ('cn', 'ldbm database'),
         ('cn', 'plugins'), ('cn', 'config')
     )
 
-    def __init__(self, dm_password=None, sub_dict={},
-                 online=True, ldapi=False):
+    def __init__(self, dm_password=_sentinel, sub_dict=None,
+                 online=_sentinel, ldapi=_sentinel, api=api):
         '''
         :parameters:
             dm_password
-                Directory Manager password
+                deprecated and no longer used
             sub_dict
                 substitution dictionary
             online
-                Do an online LDAP update or use an experimental LDIF updater
+                deprecated and no longer used
             ldapi
-                Bind using ldapi. This assumes autobind is enabled.
+                deprecated and no longer used
+            api
+                bootstrapped API object (for configuration)
 
         Data Structure Example:
         -----------------------
@@ -265,82 +325,54 @@ class LDAPUpdate:
         update format.
 
         '''
-        self.sub_dict = sub_dict
-        self.dm_password = dm_password
+        if any(arg is not _sentinel for arg in (dm_password, online, ldapi)):
+            warnings.warn(
+                "dm_password, online, and ldapi arguments are deprecated",
+                DeprecationWarning,
+                stacklevel=2
+            )
+        self.sub_dict = sub_dict if sub_dict is not None else {}
         self.conn = None
         self.modified = False
-        self.online = online
-        self.ldapi = ldapi
-        self.pw_name = pwd.getpwuid(os.geteuid()).pw_name
-        self.realm = None
-        self.socket_name = (
-            paths.SLAPD_INSTANCE_SOCKET_TEMPLATE %
-            api.env.realm.replace('.', '-')
-        )
-        suffix = None
+        self.ldapuri = ipaldap.realm_to_ldapi_uri(api.env.realm)
 
-        if sub_dict.get("REALM"):
-            self.realm = sub_dict["REALM"]
-        else:
-            self.realm = api.env.realm
-            suffix = ipautil.realm_to_suffix(self.realm) if self.realm else None
-
-        self.ldapuri = ipaldap.realm_to_ldapi_uri(self.realm)
-        if suffix is not None:
-            assert isinstance(suffix, DN)
-
-        fqdn = installutils.get_fqdn()
-        if fqdn is None:
-            raise RuntimeError("Unable to determine hostname")
-
-        if not self.sub_dict.get("REALM") and self.realm is not None:
-            self.sub_dict["REALM"] = self.realm
-        if not self.sub_dict.get("FQDN"):
-            self.sub_dict["FQDN"] = fqdn
-        if not self.sub_dict.get("DOMAIN"):
-            self.sub_dict["DOMAIN"] = api.env.domain
-        if not self.sub_dict.get("SUFFIX") and suffix is not None:
-            self.sub_dict["SUFFIX"] = suffix
-        if not self.sub_dict.get("ESCAPED_SUFFIX"):
-            self.sub_dict["ESCAPED_SUFFIX"] = str(suffix)
-        if not self.sub_dict.get("LIBARCH"):
-            self.sub_dict["LIBARCH"] = paths.LIBARCH
-        if not self.sub_dict.get("TIME"):
-            self.sub_dict["TIME"] = int(time.time())
-        if not self.sub_dict.get("MIN_DOMAIN_LEVEL"):
-            self.sub_dict["MIN_DOMAIN_LEVEL"] = str(constants.MIN_DOMAIN_LEVEL)
-        if not self.sub_dict.get("MAX_DOMAIN_LEVEL"):
-            self.sub_dict["MAX_DOMAIN_LEVEL"] = str(constants.MAX_DOMAIN_LEVEL)
-        if not self.sub_dict.get("STRIP_ATTRS"):
-            self.sub_dict["STRIP_ATTRS"] = "%s" % (
-                " ".join(constants.REPL_AGMT_STRIP_ATTRS),)
-        if not self.sub_dict.get("EXCLUDES"):
-            self.sub_dict["EXCLUDES"] = "(objectclass=*) $ EXCLUDE %s" % (
-                " ".join(constants.REPL_AGMT_EXCLUDES),)
-        if not self.sub_dict.get("TOTAL_EXCLUDES"):
-            self.sub_dict["TOTAL_EXCLUDES"] = "(objectclass=*) $ EXCLUDE " + \
-                " ".join(constants.REPL_AGMT_TOTAL_EXCLUDES)
-        if not self.sub_dict.get("SELINUX_USERMAP_DEFAULT"):
-            self.sub_dict["SELINUX_USERMAP_DEFAULT"] = \
-                platformconstants.SELINUX_USERMAP_DEFAULT
-        if not self.sub_dict.get("SELINUX_USERMAP_ORDER"):
-            self.sub_dict["SELINUX_USERMAP_ORDER"] = \
-                platformconstants.SELINUX_USERMAP_ORDER
-        if "FIPS" not in self.sub_dict:
-            self.sub_dict["FIPS"] = '#' if tasks.is_fips_enabled() else ''
         self.api = create_api(mode=None)
-        self.api.bootstrap(in_server=True,
-                           context='updates',
-                           confdir=paths.ETC_IPA,
-                           ldap_uri=self.ldapuri)
+        self.api.bootstrap(
+            in_server=True,
+            context='updates',
+            confdir=paths.ETC_IPA,
+            ldap_uri=self.ldapuri
+        )
         self.api.finalize()
-        if online:
-            # Try out the connection/password
-            # (This will raise if the server is not available)
-            self.create_connection()
-            self.close_connection()
+        self.create_connection()
+
+        # get ipa-local domain idrange settings
+        domain_range = f"{self.api.env.realm}_id_range"
+        try:
+            result = self.api.Command.idrange_show(domain_range)["result"]
+        except errors.NotFound:
+            idstart = None
+            idmax = None
         else:
-            raise RuntimeError("Offline updates are not supported.")
+            idstart = int(result['ipabaseid'][0])
+            idrange_size = int(result['ipaidrangesize'][0])
+            idmax = idstart + idrange_size - 1
+
+        default_sub = get_sub_dict(
+            realm=api.env.realm,
+            domain=api.env.domain,
+            suffix=api.env.basedn,
+            fqdn=api.env.host,
+            idstart=idstart,
+            idmax=idmax,
+        )
+        replication_plugin = (
+            installutils.get_replication_plugin_name(self.conn.get_entry)
+        )
+        default_sub["REPLICATION_PLUGIN"] = replication_plugin
+
+        for k, v in default_sub.items():
+            self.sub_dict.setdefault(k, v)
 
     def _template_str(self, s):
         try:
@@ -544,10 +576,6 @@ class LDAPUpdate:
 
     def create_index_task(self, *attributes):
         """Create a task to update an index for attributes"""
-
-        # Sleep a bit to ensure previous operations are complete
-        time.sleep(5)
-
         cn_uuid = uuid.uuid1()
         # cn_uuid.time is in nanoseconds, but other users of LDAPUpdate expect
         # seconds in 'TIME' so scale the value down
@@ -647,7 +675,7 @@ class LDAPUpdate:
         assert isinstance(dn, DN)
         searchfilter="objectclass=*"
         sattrs = ["*", "aci", "attributeTypes", "objectClasses"]
-        scope = ldap.SCOPE_BASE
+        scope = self.conn.SCOPE_BASE
 
         return self.conn.get_entries(dn, scope, searchfilter, sattrs)
 
@@ -918,13 +946,11 @@ class LDAPUpdate:
             self.create_connection()
 
     def create_connection(self):
-        if self.online:
+        if self.conn is None:
             self.api.Backend.ldap2.connect(
                 time_limit=UPDATE_SEARCH_TIME_LIMIT,
                 size_limit=0)
             self.conn = self.api.Backend.ldap2
-        else:
-            raise RuntimeError("Offline updates are not supported.")
 
     def _run_updates(self, all_updates):
         index_attributes = set()
@@ -954,8 +980,6 @@ class LDAPUpdate:
         """
         self.modified = False
         try:
-            self.create_connection()
-
             upgrade_files = files
             if ordered:
                 upgrade_files = sorted(files)
