@@ -17,16 +17,16 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+import logging
 import pprint
 
 import ldap.schema
 
 import ipapython.version
 from ipalib import api
-from ipapython.ipa_log_manager import log_mgr
+from ipalib.constants import FQDN
 from ipapython.dn import DN
 from ipaserver.install.ldapupdate import connect
-from ipaserver.install import installutils
 
 
 SCHEMA_ELEMENT_CLASSES = (
@@ -38,7 +38,7 @@ SCHEMA_ELEMENT_CLASSES = (
 
 ORIGIN = 'IPA v%s' % ipapython.version.VERSION
 
-log = log_mgr.get_logger(__name__)
+logger = logging.getLogger(__name__)
 
 
 def _get_oid_dependency_order(schema, cls):
@@ -83,7 +83,7 @@ def _get_oid_dependency_order(schema, cls):
     return ordered_oid_groups
 
 
-def update_schema(schema_files, ldapi=False, dm_password=None,):
+def update_schema(schema_files, ldapi=False):
     """Update schema to match the given ldif files
 
     Schema elements present in the LDIF files but missing from the DS schema
@@ -105,9 +105,7 @@ def update_schema(schema_files, ldapi=False, dm_password=None,):
     """
     SCHEMA_ELEMENT_CLASSES_KEYS = [x[0] for x in SCHEMA_ELEMENT_CLASSES]
 
-    conn = connect(ldapi=ldapi, dm_password=dm_password,
-                   realm=api.env.realm,
-                   fqdn=installutils.get_fqdn())
+    conn = connect(ldapi=ldapi, realm=api.env.realm, fqdn=FQDN)
 
     old_schema = conn.schema
 
@@ -119,13 +117,14 @@ def update_schema(schema_files, ldapi=False, dm_password=None,):
 
     # The exact representation the DS gives us for each OID
     # (for debug logging)
-    old_entries_by_oid = {cls(str(attr)).oid: str(attr)
+    old_entries_by_oid = {cls(attr).oid: attr.decode('utf-8')
                           for (attrname, cls) in SCHEMA_ELEMENT_CLASSES
                           for attr in schema_entry[attrname]}
 
     for filename in schema_files:
-        log.debug('Processing schema LDIF file %s', filename)
-        dn, new_schema = ldap.schema.subentry.urlfetch(filename)
+        logger.debug('Processing schema LDIF file %s', filename)
+        url = "file://{}".format(filename)
+        _dn, new_schema = ldap.schema.subentry.urlfetch(url)
 
         for attrname, cls in SCHEMA_ELEMENT_CLASSES:
             for oids_set in _get_oid_dependency_order(new_schema, cls):
@@ -143,14 +142,15 @@ def update_schema(schema_files, ldapi=False, dm_password=None,):
                         # Note: An add will automatically replace any existing
                         # schema with the same OID. So, we only add.
                         value = add_x_origin(new_obj)
-                        new_elements.append(value)
 
                         if old_obj:
                             old_attr = old_entries_by_oid.get(oid)
-                            log.debug('Replace: %s', old_attr)
-                            log.debug('   with: %s', value)
+                            logger.debug('Replace: %s', old_attr)
+                            logger.debug('   with: %s', value)
                         else:
-                            log.debug('Add: %s', value)
+                            logger.debug('Add: %s', value)
+
+                        new_elements.append(value.encode('utf-8'))
 
                 modified = modified or new_elements
                 schema_entry[attrname].extend(new_elements)
@@ -159,11 +159,12 @@ def update_schema(schema_files, ldapi=False, dm_password=None,):
                 # so updates must be executed with groups of independent OIDs
                 if new_elements:
                     modlist = schema_entry.generate_modlist()
-                    log.debug("Schema modlist:\n%s", pprint.pformat(modlist))
+                    logger.debug("Schema modlist:\n%s",
+                                 pprint.pformat(modlist))
                     conn.update_entry(schema_entry)
 
     if not modified:
-        log.debug('Not updating schema')
+        logger.debug('Not updating schema')
 
     return modified
 
