@@ -31,7 +31,7 @@ from ipaplatform import services
 from ipaplatform.paths import paths
 from ipaplatform.tasks import tasks
 from ipalib import api, errors, x509
-from ipalib.constants import DOMAIN_LEVEL_0
+from ipalib.constants import DOMAIN_LEVEL_0, FQDN
 from ipalib.facts import is_ipa_configured, is_ipa_client_configured
 from ipalib.util import (
     validate_domain_name,
@@ -42,9 +42,9 @@ from ipaserver.install import (
     adtrust, adtrustinstance, bindinstance, ca, dns, dsinstance,
     httpinstance, installutils, kra, krbinstance,
     otpdinstance, custodiainstance, replication, service,
-    sysupgrade)
+    sysupgrade, cainstance)
 from ipaserver.install.installutils import (
-    BadHostError, get_fqdn, get_server_ip_address,
+    BadHostError, get_server_ip_address,
     load_pkcs12, read_password, verify_fqdn, update_hosts_file,
     validate_mask)
 
@@ -76,7 +76,7 @@ def validate_dm_password(password):
     # Actual behavior of setup-ds.pl is that it does not accept white
     # space characters in password when called interactively but does when
     # provided such password in INF file. But it ignores leading and trailing
-    # white spaces in INF file.
+    # whitespaces in INF file.
 
     # Disallow leading/trailing whaitespaces
     if password.strip() != password:
@@ -178,7 +178,11 @@ def write_cache(options):
         shutil.rmtree(top_dir)
 
 
-def read_host_name(host_default, no_host_dns=False):
+def read_host_name(host_default):
+    """
+    Prompt user to input FQDN.  Does not verify it.
+
+    """
     print("Enter the fully qualified domain name of the computer")
     print("on which you're setting up server software. Using the form")
     print("<hostname>.<domainname>")
@@ -189,7 +193,6 @@ def read_host_name(host_default, no_host_dns=False):
         host_default = "master.example.com"
     host_name = user_input("Server host name", host_default, allow_empty=False)
     print("")
-    verify_fqdn(host_name, no_host_dns)
 
     return host_name
 
@@ -315,6 +318,19 @@ def remove_master_from_managed_topology(api_instance, options):
         logger.warning("Failed to delete master: %s", e)
 
 
+def cleanup_dogtag_server_specific_data():
+    """
+    There are data in Dogtag database related to specific servers.
+    Some of these data should be left alone, e.g. range assignments.
+    Some of these data should be cleaned up; that's what this
+    subroutine does.
+
+    """
+    # remove ACME user
+    acme_uid = cainstance.CAInstance.acme_uid(api.env.host)
+    cainstance.CAInstance.delete_user(acme_uid)
+
+
 @common_cleanup
 def install_check(installer):
     options = installer
@@ -411,7 +427,7 @@ def install_check(installer):
 
     print("======================================="
           "=======================================")
-    print("This program will set up the FreeIPA Server.")
+    print("This program will set up the IPA Server.")
     print("Version {}".format(version.VERSION))
     print("")
     print("This includes:")
@@ -427,6 +443,7 @@ def install_check(installer):
         print("  * Configure KRA (dogtag) for secret management")
     if options.setup_dns:
         print("  * Configure DNS (bind)")
+    print("  * Configure SID generation")
     if options.setup_adtrust:
         print("  * Configure Samba (smb) and winbind for managing AD trusts")
     if not options.no_pkinit:
@@ -477,14 +494,15 @@ def install_check(installer):
     if options.host_name:
         host_default = options.host_name
     else:
-        host_default = get_fqdn()
+        host_default = FQDN
+
+    if installer.interactive and not options.host_name:
+        host_name = read_host_name(host_default)
+    else:
+        host_name = host_default
 
     try:
-        if not installer.interactive or options.host_name:
-            verify_fqdn(host_default, options.no_host_dns)
-            host_name = host_default
-        else:
-            host_name = read_host_name(host_default, options.no_host_dns)
+        verify_fqdn(host_default, options.no_host_dns)
     except BadHostError as e:
         raise ScriptError(e)
 
@@ -686,8 +704,9 @@ def install_check(installer):
         logger.debug('Starting Directory Server')
         services.knownservices.dirsrv.start(instance_name)
 
-    if options.setup_adtrust:
-        adtrust.install_check(False, options, api)
+    # Always call adtrust.install_check
+    # if --setup-adtrust is not specified, only the SID part is executed
+    adtrust.install_check(False, options, api)
 
     # installer needs to update hosts file when DNS subsystem will be
     # installed or custom addresses are used
@@ -949,8 +968,9 @@ def install(installer):
     if options.setup_dns:
         dns.install(False, False, options)
 
-    if options.setup_adtrust:
-        adtrust.install(False, options, fstore, api)
+    # Always call adtrust installer to configure SID generation
+    # if --setup-adtrust is not specified, only the SID part is executed
+    adtrust.install(False, options, fstore, api)
 
     # Set the admin user kerberos password
     ds.change_admin_password(admin_password)
@@ -1081,6 +1101,8 @@ def uninstall_check(installer):
                           "uninstall procedure?", False):
             raise ScriptError("Aborting uninstall operation.")
 
+    kra.uninstall_check(options)
+
     try:
         api.Backend.ldap2.connect(autobind=True)
 
@@ -1104,6 +1126,8 @@ def uninstall_check(installer):
 
         ca.uninstall_check(options)
 
+        cleanup_dogtag_server_specific_data()
+
         if domain_level == DOMAIN_LEVEL_0:
             rm = replication.ReplicationManager(
                 realm=api.env.realm,
@@ -1146,6 +1170,10 @@ def uninstall(installer):
 
     rv = 0
 
+    # Uninstall the KRA prior to shutting the services down so it
+    # can un-register with the CA.
+    kra.uninstall()
+
     print("Shutting down all IPA services")
     try:
         services.knownservices.ipa.stop()
@@ -1158,8 +1186,6 @@ def uninstall(installer):
 
     restore_time_sync(sstore, fstore)
 
-    kra.uninstall()
-
     ca.uninstall()
 
     dns.uninstall()
