@@ -239,7 +239,7 @@ def _make_aci(ldap, current, aciname, kw):
 
     group = 'group' in kw
     permission = 'permission' in kw
-    selfaci = 'selfaci' in kw and kw['selfaci'] == True
+    selfaci = kw.get("selfaci", False)
     if group + permission + selfaci > 1:
         raise errors.ValidationError(name='target', error=_('group, permission and self are mutually exclusive'))
     elif group + permission + selfaci == 0:
@@ -271,7 +271,7 @@ def _make_aci(ldap, current, aciname, kw):
     try:
         a = ACI(current)
         a.name = _make_aci_name(kw['aciprefix'], aciname)
-        a.permissions = kw['permissions']
+        a.set_permissions(kw['permissions'])
         if 'selfaci' in kw and kw['selfaci']:
             a.set_bindrule('userdn = "ldap:///self"')
         else:
@@ -729,7 +729,7 @@ Search for ACIs.
 
         if kw.get('attrs'):
             for a in acis:
-                if not 'targetattr' in a.target:
+                if 'targetattr' not in a.target:
                     results.remove(a)
                     continue
                 alist1 = sorted(
@@ -945,7 +945,7 @@ class aci_rename(crud.Update):
         # a series of keywords. Then we replace any keywords that have been
         # updated and convert that back into an ACI and write it out.
         newkw =  _aci_to_kw(ldap, aci)
-        if 'selfaci' in newkw and newkw['selfaci'] == True:
+        if newkw.get("selfaci"):
             # selfaci is set in aci_to_kw to True only if the target is self
             kw['selfaci'] = True
         if 'aciname' in newkw:
