@@ -113,7 +113,7 @@ class delegation(Object):
         )
         json_dict['primary_key'] = self.primary_key.name
 
-        json_dict['methods'] = [m for m in self.methods]
+        json_dict['methods'] = list(self.methods)
         return json_dict
 
     def postprocess_result(self, result):
