@@ -241,7 +241,6 @@ digits and nothing else follows.
 
 from __future__ import absolute_import
 
-import datetime
 import json
 import logging
 
@@ -251,17 +250,16 @@ import contextlib
 
 import six
 
-from ipalib import Backend, api
+from ipalib import Backend, api, x509
 from ipapython.dn import DN
 import ipapython.cookie
-from ipapython import dogtag, ipautil, certdb
+from ipapython import dogtag, ipautil
 from ipaserver.masters import find_providing_server
 
-if api.env.in_server:
-    import pki
-    from pki.client import PKIConnection
-    import pki.crypto as cryptoutil
-    from pki.kra import KRAClient
+import pki
+from pki.client import PKIConnection
+import pki.crypto as cryptoutil
+from pki.kra import KRAClient
 
 if six.PY3:
     unicode = str
@@ -651,12 +649,14 @@ def parse_check_request_result_xml(doc):
 
     updated_on = doc.xpath('//xml/header/updatedOn[1]')
     if len(updated_on) == 1:
-        updated_on = datetime.datetime.utcfromtimestamp(int(updated_on[0].text))
+        updated_on = ipautil.datetime_from_utctimestamp(
+            int(updated_on[0].text), units=1)
         response['updated_on'] = updated_on
 
     created_on = doc.xpath('//xml/header/createdOn[1]')
     if len(created_on) == 1:
-        created_on = datetime.datetime.utcfromtimestamp(int(created_on[0].text))
+        created_on = ipautil.datetime_from_utctimestamp(
+            int(created_on[0].text), units=1)
         response['created_on'] = created_on
 
     request_notes = doc.xpath('//xml/header/requestNotes[1]')
@@ -685,118 +685,6 @@ def parse_check_request_result_xml(doc):
 
     return response
 
-def parse_display_cert_xml(doc):
-    '''
-    :param doc: The root node of the xml document to parse
-    :returns:   result dict
-    :except ValueError:
-
-    After parsing the results are returned in a result dict. The following
-    table illustrates the mapping from the CMS data item to what may be found in
-    the result dict. If a CMS data item is absent it will also be absent in the
-    result dict.
-
-    If the requestStatus is not SUCCESS then the response dict will have the
-    contents described in `parse_error_template_xml`.
-
-    +----------------+---------------+-----------------+---------------+
-    |cms name        |cms type       |result name      |result type    |
-    +================+===============+=================+===============+
-    |emailCert       |Boolean        |email_cert       |bool           |
-    +----------------+---------------+-----------------+---------------+
-    |noCertImport    |Boolean        |no_cert_import   |bool           |
-    +----------------+---------------+-----------------+---------------+
-    |revocationReason|int            |revocation_reason|int [1]_       |
-    +----------------+---------------+-----------------+---------------+
-    |certPrettyPrint |string         |cert_pretty      |unicode        |
-    +----------------+---------------+-----------------+---------------+
-    |authorityid     |string         |authority        |unicode        |
-    +----------------+---------------+-----------------+---------------+
-    |certFingerprint |string         |fingerprint      |unicode        |
-    +----------------+---------------+-----------------+---------------+
-    |certChainBase64 |string         |certificate      |unicode [2]_   |
-    +----------------+---------------+-----------------+---------------+
-    |serialNumber    |string         |serial_number    |int|long       |
-    +----------------+---------------+-----------------+---------------+
-    |pkcs7ChainBase64|string         |pkcs7_chain      |unicode [2]_   |
-    +----------------+---------------+-----------------+---------------+
-
-    .. [1] revocation reason may be one of:
-
-           - 0 = UNSPECIFIED
-           - 1 = KEY_COMPROMISE
-           - 2 = CA_COMPROMISE
-           - 3 = AFFILIATION_CHANGED
-           - 4 = SUPERSEDED
-           - 5 = CESSATION_OF_OPERATION
-           - 6 = CERTIFICATE_HOLD
-           - 8 = REMOVE_FROM_CRL
-           - 9 = PRIVILEGE_WITHDRAWN
-           - 10 = AA_COMPROMISE
-
-    .. [2] Base64 encoded
-
-    '''
-
-    request_status = get_request_status_xml(doc)
-
-    if request_status != CMS_STATUS_SUCCESS:
-        response = parse_error_template_xml(doc)
-        return response
-
-    response = {}
-    response['request_status'] = request_status
-
-    email_cert = doc.xpath('//xml/header/emailCert[1]')
-    if len(email_cert) == 1:
-        parse_and_set_boolean_xml(email_cert[0], response, 'email_cert')
-
-    no_cert_import = doc.xpath('//xml/header/noCertImport[1]')
-    if len(no_cert_import) == 1:
-        parse_and_set_boolean_xml(no_cert_import[0], response, 'no_cert_import')
-
-    revocation_reason = doc.xpath('//xml/header/revocationReason[1]')
-    if len(revocation_reason) == 1:
-        revocation_reason = int(revocation_reason[0].text)
-        response['revocation_reason'] = revocation_reason
-
-    cert_pretty = doc.xpath('//xml/header/certPrettyPrint[1]')
-    if len(cert_pretty) == 1:
-        cert_pretty = etree.tostring(cert_pretty[0], method='text',
-                                     encoding=unicode).strip()
-        response['cert_pretty'] = cert_pretty
-
-    authority = doc.xpath('//xml/header/authorityid[1]')
-    if len(authority) == 1:
-        authority = etree.tostring(authority[0], method='text',
-                                   encoding=unicode).strip()
-        response['authority'] = authority
-
-    fingerprint = doc.xpath('//xml/header/certFingerprint[1]')
-    if len(fingerprint) == 1:
-        fingerprint = etree.tostring(fingerprint[0], method='text',
-                                     encoding=unicode).strip()
-        response['fingerprint'] = fingerprint
-
-    certificate = doc.xpath('//xml/header/certChainBase64[1]')
-    if len(certificate) == 1:
-        certificate = etree.tostring(certificate[0], method='text',
-                                     encoding=unicode).strip()
-        response['certificate'] = certificate
-
-    serial_number = doc.xpath('//xml/header/serialNumber[1]')
-    if len(serial_number) == 1:
-        serial_number = int(serial_number[0].text, 16) # parse as hex
-        response['serial_number'] = serial_number
-        response['serial_number_hex'] = u'0x%X' % serial_number
-
-    pkcs7_chain = doc.xpath('//xml/header/pkcs7ChainBase64[1]')
-    if len(pkcs7_chain) == 1:
-        pkcs7_chain = etree.tostring(pkcs7_chain[0], method='text',
-                                     encoding=unicode).strip()
-        response['pkcs7_chain'] = pkcs7_chain
-
-    return response
 
 def parse_revoke_cert_xml(doc):
     '''
@@ -1212,9 +1100,17 @@ def parse_updateCRL_xml(doc):
 #-------------------------------------------------------------------------------
 
 from ipalib import Registry, errors, SkipPluginModule
-if api.env.ra_plugin != 'dogtag':
+
+# We only load the dogtag RA plugin if it is necessary to do so.
+# This is legacy code from when multiple RA backends were supported.
+#
+# If the plugins are loaded by the server then load the RA backend.
+#
+if api.isdone("finalize") and not (
+    api.env.ra_plugin == 'dogtag' or api.env.context == 'installer'
+):
     # In this case, abort loading this plugin module...
-    raise SkipPluginModule(reason='dogtag not selected as RA plugin')
+    raise SkipPluginModule(reason='Not loading dogtag RA plugin')
 import os
 from ipaserver.plugins import rabase
 from ipalib.constants import TYPE_ERROR
@@ -1531,13 +1427,10 @@ class ra(rabase.rabase, RestClient):
         """
         Retrieve an existing certificate.
 
-        :param serial_number: Certificate serial number. Must be a string value
-                              because serial numbers may be of any magnitude and
-                              XMLRPC cannot handle integers larger than 64-bit.
-                              The string value should be decimal, but may optionally
-                              be prefixed with a hex radix prefix if the integral value
-                              is represented as hexadecimal. If no radix prefix is
-                              supplied the string will be interpreted as decimal.
+        :param serial_number: Certificate serial number.  May be int,
+                              decimal string, or hex string with "0x"
+                              prefix.
+
 
         The command returns a dict with these possible key/value pairs.
         Some key/value pairs may be absent.
@@ -1574,49 +1467,51 @@ class ra(rabase.rabase, RestClient):
         """
         logger.debug('%s.get_certificate()', type(self).__name__)
 
-        # Convert serial number to integral type from string to properly handle
-        # radix issues. Note: the int object constructor will properly handle large
-        # magnitude integral values by returning a Python long type when necessary.
-        serial_number = int(serial_number, 0)
-
         # Call CMS
-        http_status, _http_headers, http_body = (
-            self._sslget('/ca/agent/ca/displayBySerial',
-                         self.env.ca_agent_port,
-                         serialNumber=str(serial_number),
-                         xml='true')
-        )
+        path = 'certs/{}'.format(serial_number)
+        try:
+            _http_status, _http_headers, http_body = self._ssldo(
+                'GET', path, use_session=False,
+                headers={
+                    'Accept': 'application/json',
+                },
+            )
+        except errors.HTTPRequestError as e:
+            self.raise_certificate_operation_error(
+                'get_certificate',
+                err_msg=e.msg,
+                detail=e.status  # pylint: disable=no-member
+            )
 
-
-        # Parse and handle errors
-        if http_status != 200:
-            self.raise_certificate_operation_error('get_certificate',
-                                                   detail=http_status)
-
-        parse_result = self.get_parse_result_xml(http_body, parse_display_cert_xml)
-        request_status = parse_result['request_status']
-        if request_status != CMS_STATUS_SUCCESS:
-            self.raise_certificate_operation_error('get_certificate',
-                                                   cms_request_status_to_string(request_status),
-                                                   parse_result.get('error_string'))
+        try:
+            resp = json.loads(ipautil.decode_json(http_body))
+        except ValueError:
+            raise errors.RemoteRetrieveError(
+                reason=_("Response from CA was not valid JSON"))
 
         # Return command result
         cmd_result = {}
 
-        if 'certificate' in parse_result:
-            cmd_result['certificate'] = parse_result['certificate']
+        if 'Encoded' in resp:
+            s = resp['Encoded']
+            # The 'cert' plugin expects the result to be base64-encoded
+            # X.509 DER.  We expect the result to be PEM.  We have to
+            # strip the PEM headers and we use PEM_CERT_REGEX to do it.
+            match = x509.PEM_CERT_REGEX.search(s.encode('utf-8'))
+            if match:
+                s = match.group(2).decode('utf-8')
+            cmd_result['certificate'] = s.strip()
 
-        if 'serial_number' in parse_result:
-            # see module documentation concerning serial numbers and XMLRPC
-            cmd_result['serial_number'] = unicode(parse_result['serial_number'])
-            cmd_result['serial_number_hex'] = u'0x%X' % int(cmd_result['serial_number'])
+        if 'id' in resp:
+            serial = int(resp['id'], 0)
+            cmd_result['serial_number'] = unicode(serial)
+            cmd_result['serial_number_hex'] = u'0x%X' % serial
 
-        if 'revocation_reason' in parse_result:
-            cmd_result['revocation_reason'] = parse_result['revocation_reason']
+        if 'RevocationReason' in resp and resp['RevocationReason'] is not None:
+            cmd_result['revocation_reason'] = resp['RevocationReason']
 
         return cmd_result
 
-
     def request_certificate(
             self, csr, profile_id, ca_id, request_type='pkcs10'):
         """
@@ -1890,6 +1785,10 @@ class ra(rabase.rabase, RestClient):
             node = etree.SubElement(page, 'serialTo')
             node.text = unicode(options['max_serial_number'])
 
+        if 'status' in options:
+            node = etree.SubElement(page, 'status')
+            node.text = unicode(options['status'])
+
         # date_types is a tuple that consists of:
         #   1. attribute name passed from IPA API
         #   2. attribute name used by REST API
@@ -1933,7 +1832,8 @@ class ra(rabase.rabase, RestClient):
             method='POST',
             headers={'Accept-Encoding': 'gzip, deflate',
                      'User-Agent': 'IPA',
-                     'Content-Type': 'application/xml'},
+                     'Content-Type': 'application/xml',
+                     'Accept': 'application/xml'},
             body=payload
         )
 
@@ -2070,11 +1970,10 @@ class kra(Backend):
             # TODO: replace this with a more specific exception
             raise RuntimeError('KRA service is not enabled')
 
-        tempdb = certdb.NSSDatabase()
-        tempdb.create_db()
-        crypto = cryptoutil.NSSCryptoProvider(
-            tempdb.secdir,
-            password_file=tempdb.pwd_file)
+        crypto = cryptoutil.CryptographyCryptoProvider(
+            transport_cert_nick="ra_agent",
+            transport_cert=x509.load_certificate_from_file(paths.RA_AGENT_PEM)
+        )
 
         # TODO: obtain KRA host & port from IPA service list or point to KRA load balancer
         # https://fedorahosted.org/freeipa/ticket/4557
@@ -2089,10 +1988,7 @@ class kra(Backend):
         connection.set_authentication_cert(paths.RA_AGENT_PEM,
                                            paths.RA_AGENT_KEY)
 
-        try:
-            yield KRAClient(connection, crypto)
-        finally:
-            tempdb.close()
+        yield KRAClient(connection, crypto)
 
 
 @register()
@@ -2222,3 +2118,23 @@ class ra_lightweight_ca(RestClient):
 
     def delete_ca(self, ca_id):
         self._ssldo('DELETE', ca_id)
+
+
+@register()
+class ra_securitydomain(RestClient):
+    """
+    Security domain management backend plugin.
+
+    Dogtag handles the creation of securitydomain entries
+    we need to clean them up when an IPA server is removed.
+    """
+    path = 'securityDomain/hosts'
+
+    def delete_domain(self, hostname, type):
+        """
+        Delete a security domain
+        """
+        self._ssldo(
+            'DELETE', f'{type}%20{hostname}%20443',
+            headers={'Accept': 'application/json'}
+        )
