@@ -27,7 +27,7 @@ import six
 from ipalib import api, errors, util
 from ipalib import messages
 from ipalib import Str, StrEnum, Flag
-from ipalib.parameters import Principal, Certificate
+from ipalib.parameters import Data, Principal, Certificate
 from ipalib.plugable import Registry
 from .baseldap import (LDAPQuery, LDAPObject, LDAPCreate,
                                      LDAPDelete, LDAPUpdate, LDAPSearch,
@@ -38,7 +38,7 @@ from .baseldap import (LDAPQuery, LDAPObject, LDAPCreate,
                                      LDAPAddAttributeViaOption,
                                      LDAPRemoveAttributeViaOption)
 from .service import (
-    validate_realm, normalize_principal,
+    validate_realm, validate_auth_indicator, normalize_principal,
     set_certificate_attrs, ticket_flags_params, update_krbticketflags,
     set_kerberos_attrs, rename_ipaallowedtoperform_from_ldap,
     rename_ipaallowedtoperform_to_ldap, revoke_certs)
@@ -252,6 +252,11 @@ def validate_ipaddr(ugettext, ipaddr):
     return None
 
 
+def resolve_fqdn(name):
+    hostentry = api.Command['host_show'](name)['result']
+    return hostentry['fqdn'][0]
+
+
 class HostPassword(Str):
     """
     A data type for host passwords to not log password values
@@ -260,6 +265,12 @@ class HostPassword(Str):
     setting a password on the command-line which would break
     backwards compatibility.
     """
+
+    kwargs = Data.kwargs + (
+        ('pattern', (str,), None),
+        ('noextrawhitespace', bool, False),
+    )
+
     def safe_value(self, value):
         return u'********'
 
@@ -584,7 +595,7 @@ class host(LDAPObject):
             'krbprincipalauthind*',
             cli_name='auth_ind',
             label=_('Authentication Indicators'),
-            doc=_("Defines a whitelist for Authentication Indicators."
+            doc=_("Defines an allow list for Authentication Indicators."
                   " Use 'otp' to allow OTP-based 2FA authentications."
                   " Use 'radius' to allow RADIUS-based 2FA authentications."
                   " Use 'pkinit' to allow PKINIT-based 2FA authentications."
@@ -724,6 +735,8 @@ class host_add(LDAPCreate):
         update_krbticketflags(ldap, entry_attrs, attrs_list, options, False)
         if 'krbticketflags' in entry_attrs:
             entry_attrs['objectclass'].append('krbticketpolicyaux')
+        validate_auth_indicator(entry_attrs)
+
         return dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
@@ -789,20 +802,13 @@ class host_del(LDAPDelete):
 
     def pre_callback(self, ldap, dn, *keys, **options):
         assert isinstance(dn, DN)
-        # If we aren't given a fqdn, find it
-        config = ldap.get_ipa_config()
-        maxlen = int(config.get('ipamaxhostnamelength')[0])
-        if hostname_validator(None, keys[-1], maxlen=maxlen) is not None:
-            hostentry = api.Command['host_show'](keys[-1])['result']
-            fqdn = hostentry['fqdn'][0]
-        else:
-            fqdn = keys[-1]
+        fqdn = resolve_fqdn(keys[-1])
         host_is_master(ldap, fqdn)
         # Remove all service records for this host
         truncated = True
         while truncated:
             try:
-                ret = api.Command['service_find'](fqdn)
+                ret = api.Command['service_find'](fqdn, pkey_only=True)
                 truncated = ret['truncated']
                 services = ret['result']
             except errors.NotFound:
@@ -888,6 +894,7 @@ class host_mod(LDAPUpdate):
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
+        fqdn = resolve_fqdn(keys[-1])
         # Allow an existing OTP to be reset but don't allow a OTP to be
         # added to an enrolled host.
         if options.get('userpassword') or options.get('random'):
@@ -953,7 +960,7 @@ class host_mod(LDAPUpdate):
                 entry_attrs['objectclass'] = obj_classes
 
         if options.get('updatedns', False) and dns_container_exists(ldap):
-            parts = keys[-1].split('.')
+            parts = fqdn.split('.')
             domain = unicode('.'.join(parts[1:]))
             try:
                 result = api.Command['dnszone_show'](domain)['result']
@@ -988,6 +995,7 @@ class host_mod(LDAPUpdate):
             if 'krbprincipalaux' not in (item.lower() for item in
                                          entry_attrs['objectclass']):
                 entry_attrs['objectclass'].append('krbprincipalaux')
+            validate_auth_indicator(entry_attrs)
 
         add_sshpubkey_to_attrs_pre(self.context, attrs_list)
 
@@ -1184,13 +1192,7 @@ class host_disable(LDAPQuery):
     def execute(self, *keys, **options):
         ldap = self.obj.backend
 
-        # If we aren't given a fqdn, find it
-        if hostname_validator(None, keys[-1]) is not None:
-            hostentry = api.Command['host_show'](keys[-1])['result']
-            fqdn = hostentry['fqdn'][0]
-        else:
-            fqdn = keys[-1]
-
+        fqdn = resolve_fqdn(keys[-1])
         host_is_master(ldap, fqdn)
 
         # See if we actually do anthing here, and if not raise an exception
@@ -1199,7 +1201,7 @@ class host_disable(LDAPQuery):
         truncated = True
         while truncated:
             try:
-                ret = api.Command['service_find'](fqdn)
+                ret = api.Command['service_find'](fqdn, pkey_only=True)
                 truncated = ret['truncated']
                 services = ret['result']
             except errors.NotFound:
