@@ -22,14 +22,15 @@ import six
 from ipalib.plugable import Registry
 from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                      LDAPRetrieve, LDAPSearch, LDAPUpdate)
-from ipalib import api, Int, Str, StrEnum, _, ngettext
+from ipalib import api, Int, Str, StrEnum, _, ngettext, messages
 from ipalib import errors
+from ipaplatform import services
 from ipapython.dn import DN
 
 if six.PY3:
     unicode = str
 
-if api.env.in_server and api.env.context in ['lite', 'server']:
+if api.env.in_server:
     try:
         import ipaserver.dcerpc
         _dcerpc_bindings_installed = True
@@ -79,6 +80,9 @@ and an ID range of a trusted domain must set
  - rid-base: the first RID of the corresponding RID range
  - sid: domain SID of the trusted domain
 
+and an ID range of a trusted domain may set
+ - auto-private-groups: [true|false|hybrid] automatic creation of private groups
+
 
 
 EXAMPLE: Add a new ID range for a trusted domain
@@ -157,7 +161,7 @@ for a different domain.
 
 (*) The RID is typically the last integer of a user or group SID which follows
 the domain SID. E.g. if the domain SID is S-1-5-21-123-456-789 and a user from
-this domain has the SID S-1-5-21-123-456-789-1010 then 1010 id the RID of the
+this domain has the SID S-1-5-21-123-456-789-1010 then 1010 is the RID of the
 user. RIDs are unique in a domain, 32bit values and are used for users and
 groups.
 
@@ -180,7 +184,7 @@ class idrange(LDAPObject):
     possible_objectclasses = ['ipadomainidrange', 'ipatrustedaddomainrange']
     default_attributes = ['cn', 'ipabaseid', 'ipaidrangesize', 'ipabaserid',
                           'ipasecondarybaserid', 'ipanttrusteddomainsid',
-                          'iparangetype']
+                          'iparangetype', 'ipaautoprivategroups']
     managed_permissions = {
         'System: Read ID Ranges': {
             'replaces_global_anonymous_aci': True,
@@ -190,6 +194,7 @@ class idrange(LDAPObject):
                 'cn', 'objectclass',
                 'ipabaseid', 'ipaidrangesize', 'iparangetype',
                 'ipabaserid', 'ipasecondarybaserid', 'ipanttrusteddomainsid',
+                'ipaautoprivategroups',
             },
         },
     }
@@ -200,6 +205,7 @@ class idrange(LDAPObject):
     # The commented range types are planned but not yet supported
     range_types = {
         u'ipa-local': unicode(_('local domain range')),
+        # u'ipa-local-subid': unicode(_('local domain subid range')),
         # u'ipa-ad-winsync': unicode(_('Active Directory winsync range')),
         u'ipa-ad-trust': unicode(_('Active Directory domain range')),
         u'ipa-ad-trust-posix': unicode(_('Active Directory trust range with '
@@ -216,10 +222,14 @@ class idrange(LDAPObject):
         Int('ipabaseid',
             cli_name='base_id',
             label=_("First Posix ID of the range"),
+            minvalue=1,
+            maxvalue=Int.MAX_UINT32
         ),
         Int('ipaidrangesize',
             cli_name='range_size',
             label=_("Number of IDs in the range"),
+            minvalue=1,
+            maxvalue=Int.MAX_UINT32
         ),
         Int('ipabaserid?',
             cli_name='rid_base',
@@ -245,7 +255,13 @@ class idrange(LDAPObject):
                 doc=_('ID range type, one of allowed values'),
                 values=sorted(range_types),
                 flags=['no_update'],
-                )
+                ),
+        StrEnum('ipaautoprivategroups?',
+                label=_('Auto private groups'),
+                cli_name='auto_private_groups',
+                doc=_('Auto creation of private groups, one of allowed values'),
+                values=(u'true', u'false', u'hybrid'),
+                ),
     )
 
     def handle_iparangetype(self, entry_attrs, options,
@@ -393,6 +409,10 @@ class idrange_add(LDAPCreate):
 
     may be given for a new ID range for the local domain while
 
+        --auto-private-groups
+
+    may be given for a new ID range for a trusted AD domain and
+
         --rid-base
         --dom-sid
 
@@ -481,6 +501,16 @@ class idrange_add(LDAPCreate):
                             'or ipa-ad-trust-posix when SID of the trusted '
                             'domain is not specified.'))
 
+            # auto private group is possible only for ad trusts
+            if is_set('ipaautoprivategroups') and \
+               entry_attrs['iparangetype'] not in (u'ipa-ad-trust',
+                                                   u'ipa-ad-trust-posix'):
+                raise errors.ValidationError(
+                    name='ID Range setup',
+                    error=_('IPA Range type must be one of ipa-ad-trust '
+                            'or ipa-ad-trust-posix when '
+                            'auto-private-groups is specified'))
+
             # secondary base rid must be set if and only if base rid is set
             if is_set('ipasecondarybaserid') != is_set('ipabaserid'):
                 raise errors.ValidationError(name='ID Range setup',
@@ -644,7 +674,10 @@ class idrange_mod(LDAPUpdate):
         except errors.NotFound:
             raise self.obj.handle_not_found(*keys)
 
-        if old_attrs['iparangetype'][0] == 'ipa-local':
+        if (
+            old_attrs['iparangetype'][0] in {'ipa-local', 'ipa-local-subid'}
+            or old_attrs['cn'][0] == f'{self.api.env.realm}_subid_range'
+        ):
             raise errors.ExecutionError(
                 message=_('This command can not be used to change ID '
                           'allocation for local IPA domain. Run '
@@ -708,7 +741,7 @@ class idrange_mod(LDAPUpdate):
                     entry_attrs['ipanttrusteddomainsid'])
 
             # Add trusted AD domain range object class, if it wasn't there
-            if not 'ipatrustedaddomainrange' in old_attrs['objectclass']:
+            if 'ipatrustedaddomainrange' not in old_attrs['objectclass']:
                 entry_attrs['objectclass'].append('ipatrustedaddomainrange')
 
         else:
@@ -764,4 +797,10 @@ class idrange_mod(LDAPUpdate):
         assert isinstance(dn, DN)
         self.obj.handle_ipabaserid(entry_attrs, options)
         self.obj.handle_iparangetype(entry_attrs, options)
+        self.add_message(
+            messages.ServiceRestartRequired(
+                service=services.knownservices['sssd'].systemd_name,
+                server=keys[0]
+            )
+        )
         return dn
