@@ -22,13 +22,17 @@ import re
 import six
 
 from .baseldap import (LDAPQuery, LDAPObject, LDAPCreate,
-                                     LDAPDelete, LDAPUpdate, LDAPSearch,
-                                     LDAPAddAttributeViaOption,
-                                     LDAPRemoveAttributeViaOption,
-                                     LDAPRetrieve, global_output_params)
+                       LDAPDelete, LDAPUpdate, LDAPSearch,
+                       LDAPAddAttributeViaOption,
+                       LDAPRemoveAttributeViaOption,
+                       LDAPRetrieve, global_output_params,
+                       host_is_master,
+                       add_missing_object_class, member_validator)
 from .hostgroup import get_complete_hostgroup_member_list
-from .service import validate_certificate
-from ipalib import api, Str, Int, Bytes, Flag, _, ngettext, errors, output
+from ipalib import (
+    api, Str, Int, Flag, _, ngettext, errors, output
+)
+from ipalib.parameters import Certificate
 from ipalib.constants import (
     IPA_ANCHOR_PREFIX,
     SID_ANCHOR_PREFIX,
@@ -45,7 +49,7 @@ if six.PY3:
 
 _dcerpc_bindings_installed = False
 
-if api.env.in_server and api.env.context in ['lite', 'server']:
+if api.env.in_server:
     try:
         import ipaserver.dcerpc
         _dcerpc_bindings_installed = True
@@ -54,7 +58,9 @@ if api.env.in_server and api.env.context in ['lite', 'server']:
 
 __doc__ = _("""
 ID Views
+
 Manage ID Views
+
 IPA allows to override certain properties of users and groups per each host.
 This functionality is primarily used to allow migration from older systems or
 other Identity Management solutions.
@@ -85,6 +91,28 @@ ANCHOR_REGEX = re.compile(
 )
 
 
+def normalize_idview_name(value):
+    if value in (None, '',):
+        return DEFAULT_TRUST_VIEW_NAME
+    return value
+
+
+def handle_idoverride_memberof(self, ldap, dn, found, not_found,
+                               *keys, **options):
+    if ('idoverrideuser' in options) and ('member' in found):
+        for id in found['member'].get('idoverrideuser', []):
+            try:
+                e = ldap.get_entry(id, ['*'])
+                if not self.obj.has_objectclass(e['objectclass'],
+                                                'nsmemberof'):
+                    add_missing_object_class(ldap, 'nsmemberof',
+                                             id, entry_attrs=e, update=True)
+            except errors.NotFound:
+                # We are not adding an object here, only modifying existing
+                continue
+    return dn
+
+
 @register()
 class idview(LDAPObject):
     """
@@ -95,8 +123,9 @@ class idview(LDAPObject):
     object_name = _('ID View')
     object_name_plural = _('ID Views')
     object_class = ['ipaIDView', 'top']
-    default_attributes = ['cn', 'description']
-    rdn_is_primary_key = True
+    possible_objectclasses = ['ipaNameResolutionData']
+    default_attributes = ['cn', 'description', 'ipadomainresolutionorder']
+    allow_rename = True
 
     label = _('ID Views')
     label_singular = _('ID View')
@@ -106,6 +135,7 @@ class idview(LDAPObject):
             cli_name='name',
             label=_('ID View Name'),
             primary_key=True,
+            normalizer=normalize_idview_name,
         ),
         Str('description?',
             cli_name='desc',
@@ -123,6 +153,14 @@ class idview(LDAPObject):
             label=_('Hosts the view applies to'),
             flags={'virtual_attribute', 'no_create', 'no_update', 'no_search'},
         ),
+        Str(
+            'ipadomainresolutionorder?',
+            cli_name='domain_resolution_order',
+            label=_('Domain resolution order'),
+            doc=_('colon-separated list of domains used for short name'
+                  ' qualification'),
+            flags={'no_search'}
+        )
     )
 
     permission_filter_objectclasses = ['nsContainer']
@@ -131,17 +169,43 @@ class idview(LDAPObject):
             'ipapermbindruletype': 'all',
             'ipapermright': {'read', 'search', 'compare'},
             'ipapermdefaultattr': {
-                'cn', 'description', 'objectClass',
+                'cn', 'description', 'ipadomainresolutionorder', 'objectClass',
             },
         },
     }
 
+    def ensure_possible_objectclasses(self, ldap, dn, entry_attrs, *keys):
+        try:
+            orig_entry_attrs = ldap.get_entry(dn, ['objectclass'])
+        except errors.NotFound:
+            raise self.handle_not_found(*keys)
+
+        orig_objectclasses = {
+            o.lower() for o in orig_entry_attrs.get('objectclass', [])}
+
+        entry_attrs['objectclass'] = orig_entry_attrs['objectclass']
+
+        for obj_class_name in self.possible_objectclasses:
+            if obj_class_name.lower() not in orig_objectclasses:
+                entry_attrs['objectclass'].append(obj_class_name)
+
 
 @register()
 class idview_add(LDAPCreate):
     __doc__ = _('Add a new ID View.')
     msg_summary = _('Added ID View "%(value)s"')
 
+    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
+        self.api.Object.config.validate_domain_resolution_order(entry_attrs)
+
+        # The objectclass ipaNameResolutionData may not be present on
+        # the id view. We need to add it if we define a new
+        # value for ipaDomainResolutionOrder
+        if 'ipadomainresolutionorder' in entry_attrs:
+            add_missing_object_class(ldap, u'ipanameresolutiondata', dn,
+                                     entry_attrs, update=False)
+        return dn
+
 
 @register()
 class idview_del(LDAPDelete):
@@ -166,6 +230,9 @@ class idview_mod(LDAPUpdate):
             if key.lower() == DEFAULT_TRUST_VIEW_NAME:
                 raise protected_default_trust_view_error
 
+        self.api.Object.config.validate_domain_resolution_order(entry_attrs)
+        self.obj.ensure_possible_objectclasses(ldap, dn, entry_attrs, *keys)
+
         return dn
 
 
@@ -199,7 +266,7 @@ class idview_show(LDAPRetrieve):
             attr_name = obj_type + 'overrides'
 
             try:
-                (overrides, truncated) = ldap.find_entries(
+                overrides, _truncated = ldap.find_entries(
                     filter="objectclass=%s" % objectclass,
                     attrs_list=['ipaanchoruuid'],
                     base_dn=dn,
@@ -234,7 +301,7 @@ class idview_show(LDAPRetrieve):
         }
 
         try:
-            (hosts, truncated) = ldap.find_entries(
+            hosts, _truncated = ldap.find_entries(
                 filter=ldap.make_filter(filter_params, rules=ldap.MATCH_ALL),
                 attrs_list=['cn'],
                 base_dn=api.env.container_host + api.env.basedn,
@@ -317,6 +384,16 @@ class baseidview_apply(LDAPQuery):
 
         for host in hosts_to_apply:
             try:
+                # Check that the host is not a master
+                # IDView must not be applied to masters
+                try:
+                    host_is_master(ldap, host)
+                except errors.ValidationError:
+                    failed['host'].append(
+                        (host,
+                         unicode(_("ID View cannot be applied to IPA master")))
+                    )
+                    continue
                 host_dn = api.Object['host'].get_dn_if_exists(host)
 
                 host_entry = ldap.get_entry(host_dn,
@@ -325,7 +402,7 @@ class baseidview_apply(LDAPQuery):
 
                 ldap.update_entry(host_entry)
 
-                # If no exception was raised, view assigment went well
+                # If no exception was raised, view assignment went well
                 completed = completed + 1
                 succeeded['host'].append(host)
             except errors.EmptyModlist:
@@ -495,7 +572,7 @@ def resolve_object_to_anchor(ldap, obj_type, obj, fallback_to_ldap):
     Takes options:
         ldap - the backend
         obj_type - either 'user' or 'group'
-        obj - the name of the object, e.g 'admin' or 'testuser'
+        obj - the name of the object, e.g. 'admin' or 'testuser'
     """
 
     try:
@@ -511,10 +588,11 @@ def resolve_object_to_anchor(ldap, obj_type, obj, fallback_to_ldap):
             'group': 'ipausergroup',
         }[obj_type]
 
-        if required_objectclass not in entry['objectclass']:
+        if not api.Object[obj_type].has_objectclass(entry['objectclass'],
+                                                    required_objectclass):
             raise errors.ValidationError(
                     name=_('IPA object'),
-                    error=_('system IPA objects (e.g system groups, user '
+                    error=_('system IPA objects (e.g. system groups, user '
                             'private groups) cannot be overridden')
                 )
 
@@ -527,7 +605,7 @@ def resolve_object_to_anchor(ldap, obj_type, obj, fallback_to_ldap):
     except errors.NotFound:
         pass
 
-    # If not successfull, try looking up the object in the trusted domain
+    # If not successful, try looking up the object in the trusted domain
     try:
         if _dcerpc_bindings_installed:
             domain_validator = ipaserver.dcerpc.DomainValidator(api)
@@ -550,7 +628,7 @@ def resolve_object_to_anchor(ldap, obj_type, obj, fallback_to_ldap):
         pass
 
     # No acceptable object was found
-    api.Object[obj_type].handle_not_found(obj)
+    raise api.Object[obj_type].handle_not_found(obj)
 
 
 def resolve_anchor_to_object_name(ldap, obj_type, anchor):
@@ -624,7 +702,7 @@ def remove_ipaobject_overrides(ldap, api, dn):
     override_filter = '(ipaanchoruuid=:IPA:{0}:{1})'.format(api.env.domain,
                                                             object_uuid)
     try:
-        entries, truncated = ldap.find_entries(
+        entries, _truncated = ldap.find_entries(
             override_filter,
             base_dn=DN(api.env.container_views, api.env.basedn),
             paged_search=True
@@ -637,6 +715,41 @@ def remove_ipaobject_overrides(ldap, api, dn):
             ldap.delete_entry(entry)
 
 
+# Extended user validator that allows to resolve users from trusted domains
+def validate_external_object(ldap, dn, keys, options, value, object_type):
+    trusted_objects = options.get('trusted_objects', [])
+    param = api.Object[object_type].primary_key
+    if object_type == 'group' and value.startswith('%'):
+        value = value[1:]
+    o_id = param(value)
+
+    try:
+        param.validate(o_id)
+    except errors.ValidationError as e:
+        membertype = object_type
+        try:
+            id = resolve_object_to_anchor(ldap, object_type, o_id, False)
+            if id.startswith(SID_ANCHOR_PREFIX):
+                if object_type == 'group':
+                    trusted_objects.append('%' + value)
+                else:
+                    trusted_objects.append(value)
+                options['trusted_objects'] = trusted_objects
+                return
+        except errors.NotFound:
+            raise errors.ValidationError(name=membertype, error=e.error)
+
+
+@member_validator(membertype='user')
+def validate_external_user(ldap, dn, keys, options, value):
+    validate_external_object(ldap, dn, keys, options, value, 'user')
+
+
+@member_validator(membertype='group')
+def validate_external_ipasudorunas(ldap, dn, keys, options, value):
+    validate_external_object(ldap, dn, keys, options, value, 'group')
+
+
 # This is not registered on purpose, it's a base class for ID overrides
 class baseidoverride(LDAPObject):
     """
@@ -678,8 +791,12 @@ class baseidoverride(LDAPObject):
                 self.backend,
                 self.override_object,
                 keys[-1],
-                fallback_to_ldap=options['fallback_to_ldap']
+                fallback_to_ldap=options.get('fallback_to_ldap', False)
             )
+            if all([len(keys[:-1]) == 0,
+                    self.override_object == 'user',
+                    anchor.startswith(SID_ANCHOR_PREFIX)]):
+                keys = (DEFAULT_TRUST_VIEW_NAME, ) + keys
 
         keys = keys[:-1] + (anchor, )
         return super(baseidoverride, self).get_dn(*keys, **options)
@@ -722,6 +839,46 @@ class baseidoverride(LDAPObject):
                     error=_('Default Trust View cannot contain IPA users')
                     )
 
+    def filter_for_anchor(self, ldap, filter, options, obj_type):
+        """Modify filter to support user and group names
+
+        Allow users to pass in an IPA user/group name and resolve it to an
+        anchor name.
+
+        :param ldap: ldap connection
+        :param filter: pre_callback filter
+        :param options: option dict
+        :param obj_type: 'user' or 'group'
+        :return: modified or same filter
+        """
+        anchor = options.get('ipaanchoruuid', None)
+        # return original filter if anchor is absent or correct
+        if anchor is None or ANCHOR_REGEX.match(anchor):
+            return filter
+        try:
+            resolved_anchor = resolve_object_to_anchor(
+                ldap, obj_type, anchor,
+                options.get('fallback_to_ldap', False)
+            )
+        except (errors.NotFound, errors.ValidationError):
+            # anchor cannot be resolved, let it pass through
+            return filter
+        else:
+            return ldap.make_filter(
+                {
+                    'objectClass': self.object_class,
+                    'ipaanchoruuid': resolved_anchor,
+                },
+                rules=ldap.MATCH_ALL
+            )
+
+    def get_primary_key_from_dn(self, dn):
+        return resolve_anchor_to_object_name(self.backend,
+                                             self.override_object,
+                                             dn[0].value)
+
+
+
 class baseidoverride_add(LDAPCreate):
     __doc__ = _('Add a new ID override.')
     msg_summary = _('Added ID override "%(value)s"')
@@ -752,14 +909,12 @@ class baseidoverride_del(LDAPDelete):
         try:
             entry = ldap.get_entry(dn, ['objectclass'])
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
-
-        required_object_classes = set(self.obj.object_class)
-        actual_object_classes = set(entry['objectclass'])
+            raise self.obj.handle_not_found(*keys)
 
         # If not, treat it as a failed search
-        if not required_object_classes.issubset(actual_object_classes):
-            self.obj.handle_not_found(*keys)
+        for required_oc in self.obj.object_class:
+            if not self.obj.has_objectclass(entry['objectclass'], required_oc):
+                raise self.obj.handle_not_found(*keys)
 
         return dn
 
@@ -816,7 +971,12 @@ class idoverrideuser(baseidoverride):
 
     label = _('User ID overrides')
     label_singular = _('User ID override')
-    rdn_is_primary_key = True
+    allow_rename = True
+
+    # ID user overrides are bindable because we map SASL GSSAPI
+    # authentication of trusted users to ID user overrides in the
+    # default trust view.
+    bindable = True
 
     permission_filter_objectclasses = ['ipaUserOverride']
     managed_permissions = {
@@ -826,16 +986,18 @@ class idoverrideuser(baseidoverride):
             'ipapermdefaultattr': {
                 'objectClass', 'ipaAnchorUUID', 'uidNumber', 'description',
                 'homeDirectory', 'uid', 'ipaOriginalUid', 'loginShell', 'gecos',
-                'gidNumber', 'ipaSshPubkey', 'usercertificate'
+                'gidNumber', 'ipaSshPubkey', 'usercertificate', 'memberof'
             },
         },
     }
 
     object_class = baseidoverride.object_class + ['ipaUserOverride']
-    possible_objectclasses = ['ipasshuser', 'ipaSshGroupOfPubKeys']
+    possible_objectclasses = ['ipasshuser', 'ipaSshGroupOfPubKeys',
+                              'nsmemberof']
     default_attributes = baseidoverride.default_attributes + [
        'homeDirectory', 'uidNumber', 'uid', 'ipaOriginalUid', 'loginShell',
        'ipaSshPubkey', 'gidNumber', 'gecos', 'usercertificate;binary',
+       'memberofindirect', 'memberof'
     ]
 
     search_display_attributes = baseidoverride.default_attributes + [
@@ -843,6 +1005,11 @@ class idoverrideuser(baseidoverride):
        'ipaSshPubkey', 'gidNumber', 'gecos',
     ]
 
+    attribute_members = {
+        'memberof': ['group', 'role'],
+        'memberofindirect': ['group', 'role'],
+    }
+
     takes_params = baseidoverride.takes_params + (
         Str('uid?',
             pattern=PATTERN_GROUPUSER_NAME,
@@ -883,7 +1050,7 @@ class idoverrideuser(baseidoverride):
             normalizer=normalize_sshpubkey,
             flags=['no_search'],
         ),
-        Bytes('usercertificate*', validate_certificate,
+        Certificate('usercertificate*',
               cli_name='certificate',
               label=_('Certificate'),
               doc=_('Base-64 encoded user certificate'),
@@ -927,7 +1094,7 @@ class idoverridegroup(baseidoverride):
 
     label = _('Group ID overrides')
     label_singular = _('Group ID override')
-    rdn_is_primary_key = True
+    allow_rename = True
 
     permission_filter_objectclasses = ['ipaGroupOverride']
     managed_permissions = {
@@ -1081,6 +1248,15 @@ class idoverrideuser_find(baseidoverride_find):
     msg_summary = ngettext('%(count)d User ID override matched',
                            '%(count)d User ID overrides matched', 0)
 
+    def pre_callback(self, ldap, filter, attrs_list, base_dn, scope, *args,
+                     **options):
+        result = super(idoverrideuser_find, self).pre_callback(
+            ldap, filter, attrs_list, base_dn, scope, *args, **options
+        )
+        filter, base_dn, scope = result
+        filter = self.obj.filter_for_anchor(ldap, filter, options, 'user')
+        return filter, base_dn, scope
+
     def post_callback(self, ldap, entries, truncated, *args, **options):
         truncated = super(idoverrideuser_find, self).post_callback(
             ldap, entries, truncated, *args, **options)
@@ -1126,6 +1302,15 @@ class idoverridegroup_find(baseidoverride_find):
     msg_summary = ngettext('%(count)d Group ID override matched',
                            '%(count)d Group ID overrides matched', 0)
 
+    def pre_callback(self, ldap, filter, attrs_list, base_dn, scope, *args,
+                     **options):
+        result = super(idoverridegroup_find, self).pre_callback(
+            ldap, filter, attrs_list, base_dn, scope, *args, **options
+        )
+        filter, base_dn, scope = result
+        filter = self.obj.filter_for_anchor(ldap, filter, options, 'group')
+        return filter, base_dn, scope
+
 
 @register()
 class idoverridegroup_show(baseidoverride_show):
