@@ -18,8 +18,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import logging
+
 from ipalib import api
-from ipalib import Int, Str, DNParam
+from ipalib import Int, Str, DNParam, Bool
 from ipalib import errors
 from .baseldap import (
     LDAPObject,
@@ -31,9 +33,7 @@ from .baseldap import (
 from ipalib import _
 from ipalib.plugable import Registry
 from ipalib.request import context
-from ipapython.ipautil import run
 from ipapython.dn import DN
-from distutils import version
 
 import six
 
@@ -83,13 +83,14 @@ EXAMPLES:
    ipa pwpolicy-mod --minclasses=2 localadmins
 """)
 
+logger = logging.getLogger(__name__)
+
 register = Registry()
 
 @register()
 class cosentry(LDAPObject):
-    """
-    Class of Service object used for linking policies with groups
-    """
+    __doc__ = _('Class of Service object used for linking policies with'
+                ' groups')
     NO_CLI = True
 
     container_dn = DN(('cn', 'costemplates'), api.env.container_accounts)
@@ -167,6 +168,7 @@ class cosentry(LDAPObject):
 
 @register()
 class cosentry_add(LDAPCreate):
+    __doc__ = _('Add Class of Service entry')
     NO_CLI = True
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
@@ -177,7 +179,7 @@ class cosentry_add(LDAPCreate):
         try:
             result = ldap.get_entry(group_dn, ['objectclass'])
         except errors.NotFound:
-            self.api.Object.group.handle_not_found(keys[-1])
+            raise self.api.Object.group.handle_not_found(keys[-1])
 
         oc = [x.lower() for x in result['objectclass']]
         if 'mepmanagedentry' in oc:
@@ -189,11 +191,13 @@ class cosentry_add(LDAPCreate):
 
 @register()
 class cosentry_del(LDAPDelete):
+    __doc__ = _('Delete Class of Service entry')
     NO_CLI = True
 
 
 @register()
 class cosentry_mod(LDAPUpdate):
+    __doc__ = _('Modify Class of Service entry')
     NO_CLI = True
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
@@ -211,11 +215,13 @@ class cosentry_mod(LDAPUpdate):
 
 @register()
 class cosentry_show(LDAPRetrieve):
+    __doc__ = _('Display Class of Service entry')
     NO_CLI = True
 
 
 @register()
 class cosentry_find(LDAPSearch):
+    __doc__ = _('Search for Class of Service entry')
     NO_CLI = True
 
 
@@ -230,13 +236,15 @@ class pwpolicy(LDAPObject):
     container_dn = DN(('cn', api.env.realm), ('cn', 'kerberos'))
     object_name = _('password policy')
     object_name_plural = _('password policies')
-    object_class = ['top', 'nscontainer', 'krbpwdpolicy']
-    permission_filter_objectclasses = ['krbpwdpolicy']
+    object_class = ['top', 'nscontainer', 'krbpwdpolicy', 'ipapwdpolicy']
+    permission_filter_objectclasses = ['krbpwdpolicy', 'ipapwdpolicy']
     default_attributes = [
         'cn', 'cospriority', 'krbmaxpwdlife', 'krbminpwdlife',
         'krbpwdhistorylength', 'krbpwdmindiffchars', 'krbpwdminlength',
         'krbpwdmaxfailure', 'krbpwdfailurecountinterval',
-        'krbpwdlockoutduration',
+        'krbpwdlockoutduration', 'ipapwdmaxrepeat',
+        'ipapwdmaxsequence', 'ipapwddictcheck',
+        'ipapwdusercheck',
     ]
     managed_permissions = {
         'System: Read Group Password Policy': {
@@ -248,6 +256,8 @@ class pwpolicy(LDAPObject):
                 'krbpwdfailurecountinterval', 'krbpwdhistorylength',
                 'krbpwdlockoutduration', 'krbpwdmaxfailure',
                 'krbpwdmindiffchars', 'krbpwdminlength', 'objectclass',
+                'ipapwdmaxrepeat', 'ipapwdmaxsequence', 'ipapwddictcheck',
+                'ipapwdusercheck',
             },
             'default_privileges': {
                 'Password Policy Readers',
@@ -273,7 +283,9 @@ class pwpolicy(LDAPObject):
             'ipapermdefaultattr': {
                 'krbmaxpwdlife', 'krbminpwdlife', 'krbpwdfailurecountinterval',
                 'krbpwdhistorylength', 'krbpwdlockoutduration',
-                'krbpwdmaxfailure', 'krbpwdmindiffchars', 'krbpwdminlength'
+                'krbpwdmaxfailure', 'krbpwdmindiffchars', 'krbpwdminlength',
+                'ipapwdmaxrepeat', 'ipapwdmaxsequence', 'ipapwddictcheck',
+                'ipapwdusercheck',
             },
             'replaces': [
                 '(targetattr = "krbmaxpwdlife || krbminpwdlife || krbpwdhistorylength || krbpwdmindiffchars || krbpwdminlength || krbpwdmaxfailure || krbpwdfailurecountinterval || krbpwdlockoutduration")(target = "ldap:///cn=*,cn=$REALM,cn=kerberos,$SUFFIX")(version 3.0;acl "permission:Modify Group Password Policy";allow (write) groupdn = "ldap:///cn=Modify Group Password Policy,cn=permissions,cn=pbac,$SUFFIX";)',
@@ -282,40 +294,6 @@ class pwpolicy(LDAPObject):
         },
     }
 
-    MIN_KRB5KDC_WITH_LOCKOUT = "1.8"
-    has_lockout = False
-    lockout_params = ()
-
-    result = run(['klist', '-V'], raiseonerr=False, capture_output=True)
-    if result.returncode == 0:
-        verstr = result.output.split()[-1]
-        ver = version.LooseVersion(verstr)
-        min = version.LooseVersion(MIN_KRB5KDC_WITH_LOCKOUT)
-        if ver >= min:
-                has_lockout = True
-
-    if has_lockout:
-        lockout_params = (
-            Int('krbpwdmaxfailure?',
-                cli_name='maxfail',
-                label=_('Max failures'),
-                doc=_('Consecutive failures before lockout'),
-                minvalue=0,
-            ),
-            Int('krbpwdfailurecountinterval?',
-                cli_name='failinterval',
-                label=_('Failure reset interval'),
-                doc=_('Period after which failure count will be reset (seconds)'),
-                minvalue=0,
-            ),
-            Int('krbpwdlockoutduration?',
-                cli_name='lockouttime',
-                label=_('Lockout duration'),
-                doc=_('Period for which lockout is enforced (seconds)'),
-                minvalue=0,
-            ),
-        )
-
     label = _('Password Policies')
     label_singular = _('Password Policy')
 
@@ -365,7 +343,60 @@ class pwpolicy(LDAPObject):
             minvalue=0,
             flags=('virtual_attribute',),
         ),
-    ) + lockout_params
+        Int(
+            'krbpwdmaxfailure?',
+            cli_name='maxfail',
+            label=_('Max failures'),
+            doc=_('Consecutive failures before lockout'),
+            minvalue=0,
+        ),
+        Int(
+            'krbpwdfailurecountinterval?',
+            cli_name='failinterval',
+            label=_('Failure reset interval'),
+            doc=_('Period after which failure count will be reset (seconds)'),
+            minvalue=0,
+        ),
+        Int(
+            'krbpwdlockoutduration?',
+            cli_name='lockouttime',
+            label=_('Lockout duration'),
+            doc=_('Period for which lockout is enforced (seconds)'),
+            minvalue=0,
+        ),
+        Int(
+            'ipapwdmaxrepeat?',
+            cli_name='maxrepeat',
+            label=_('Max repeat'),
+            doc=_('Maximum number of same consecutive characters'),
+            minvalue=0,
+            maxvalue=256,
+            default=0,
+        ),
+        Int(
+            'ipapwdmaxsequence?',
+            cli_name='maxsequence',
+            label=_('Max sequence'),
+            doc=_('The max. length of monotonic character sequences (abcd)'),
+            minvalue=0,
+            maxvalue=256,
+            default=0,
+        ),
+        Bool(
+            'ipapwddictcheck?',
+            cli_name='dictcheck',
+            label=_('Dictionary check'),
+            doc=_('Check if the password is a dictionary word'),
+            default=False,
+        ),
+        Bool(
+            'ipapwdusercheck?',
+            cli_name='usercheck',
+            label=_('User check'),
+            doc=_('Check if the password contains the username'),
+            default=False,
+        ),
+    )
 
     def get_dn(self, *keys, **options):
         if keys[-1] is not None:
@@ -394,6 +425,51 @@ class pwpolicy(LDAPObject):
         if 'krbminpwdlife' in entry_attrs and entry_attrs['krbminpwdlife']:
             entry_attrs['krbminpwdlife'] = entry_attrs['krbminpwdlife'] * 3600
 
+    def validate_minlength(self, ldap, entry_attrs, add=False, *keys):
+        """
+        If any of the libpwquality options are used then the minimum
+        length must be >= 6 which is the built-in default of libpwquality.
+        Allowing a lower value to be set will result in a failed policy
+        check and a generic error message.
+        """
+        def get_val(entry, attr):
+            """Get a single value from a list or a string"""
+            val = entry.get(attr, 0)
+            if isinstance(val, list):
+                val = val[0]
+            return val
+
+        def has_pwquality_set(entry):
+            for attr in ['ipapwdmaxrepeat', 'ipapwdmaxsequence',
+                         'ipapwddictcheck', 'ipapwdusercheck']:
+                val = get_val(entry, attr)
+                if val not in ('FALSE', '0', 0, None):
+                    return True
+            return False
+
+        has_pwquality_value = False
+        if not add:
+            if len(keys) > 0:
+                existing_entry = self.api.Command.pwpolicy_show(
+                    keys[-1], all=True,)['result']
+            else:
+                existing_entry = self.api.Command.pwpolicy_show(
+                    all=True,)['result']
+            existing_entry.update(entry_attrs)
+            min_length = int(get_val(existing_entry, 'krbpwdminlength'))
+
+            has_pwquality_value = has_pwquality_set(existing_entry)
+        else:
+            min_length = int(get_val(entry_attrs, 'krbpwdminlength'))
+            has_pwquality_value = has_pwquality_set(entry_attrs)
+
+        if min_length and min_length < 6 and has_pwquality_value:
+            raise errors.ValidationError(
+                name='minlength',
+                error=_('Minimum length must be >= 6 if maxrepeat, '
+                        'maxsequence, dictcheck or usercheck are defined')
+            )
+
     def validate_lifetime(self, entry_attrs, add=False, *keys):
         """
         Ensure that the maximum lifetime is greater than the minimum.
@@ -415,7 +491,10 @@ class pwpolicy(LDAPObject):
             if minlife > maxlife:
                 raise errors.ValidationError(
                     name='maxlife',
-                    error=_('Maximum password life must be greater than minimum.'),
+                    error=_(
+                        "Maximum password life must be equal to "
+                        "or greater than the minimum."
+                    ),
                 )
 
     def add_cospriority(self, entry, pwpolicy_name, rights=True):
@@ -442,6 +521,7 @@ class pwpolicy_add(LDAPCreate):
         assert isinstance(dn, DN)
         self.obj.convert_time_on_input(entry_attrs)
         self.obj.validate_lifetime(entry_attrs, True)
+        self.obj.validate_minlength(ldap, entry_attrs, True)
         self.api.Command.cosentry_add(
             keys[-1], krbpwdpolicyreference=dn,
             cospriority=options.get('cospriority')
@@ -450,7 +530,7 @@ class pwpolicy_add(LDAPCreate):
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
         assert isinstance(dn, DN)
-        self.log.info('%r' % entry_attrs)
+        logger.info('%r', entry_attrs)
         # attribute rights are not allowed for pwpolicy_add
         self.obj.add_cospriority(entry_attrs, keys[-1], rights=False)
         self.obj.convert_time_for_output(entry_attrs, **options)
@@ -493,7 +573,12 @@ class pwpolicy_mod(LDAPUpdate):
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
+        old_entry_attrs = ldap.get_entry(dn, ['objectclass'])
+        if 'ipapwdpolicy' not in old_entry_attrs['objectclass']:
+            old_entry_attrs['objectclass'].append('ipapwdpolicy')
+            entry_attrs['objectclass'] = old_entry_attrs['objectclass']
         self.obj.convert_time_on_input(entry_attrs)
+        self.obj.validate_minlength(ldap, entry_attrs, False, *keys)
         self.obj.validate_lifetime(entry_attrs, False, *keys)
         setattr(context, 'cosupdate', False)
         if options.get('cospriority') is not None:
