@@ -335,8 +335,9 @@ class selinuxusermap_add(LDAPCreate):
         entry_attrs['ipaenabledflag'] = 'TRUE'
         validate_selinuxuser_inlist(ldap, entry_attrs['ipaselinuxuser'])
 
-        # hbacrule is not allowed when usercat or hostcat is set
-        is_to_be_set = lambda x: x in entry_attrs and entry_attrs[x] != None
+        def is_to_be_set(x):
+            """hbacrule is not allowed when usercat or hostcat is set"""
+            return x in entry_attrs and entry_attrs[x] is not None
 
         are_local_members_to_be_set = any(is_to_be_set(attr)
                                           for attr in ('usercategory',
@@ -453,12 +454,20 @@ class selinuxusermap_find(LDAPSearch):
         if options.get('seealso'):
             hbacrule = options['seealso']
 
+            # If a complete DN is passed we can skip calling hbacrule-show
             try:
-                hbac = api.Command['hbacrule_show'](hbacrule,
-all=True)['result']
-                dn = hbac['dn']
-            except errors.NotFound:
-                return dict(count=0, result=[], truncated=False)
+                tmpdn = DN(hbacrule)
+            except ValueError:
+                tmpdn = DN()
+            if DN(api.env.container_hbac, api.env.basedn) not in tmpdn:
+                try:
+                    hbac = api.Command['hbacrule_show'](hbacrule,
+                                                        all=True)['result']
+                    dn = hbac['dn']
+                except errors.NotFound:
+                    return dict(count=0, result=[], truncated=False)
+            else:
+                dn = tmpdn
             options['seealso'] = dn
 
         return super(selinuxusermap_find, self).execute(*args, **options)
