@@ -50,7 +50,8 @@ from .baseuser import (
     baseuser_add_principal,
     baseuser_remove_principal,
     baseuser_add_certmapdata,
-    baseuser_remove_certmapdata)
+    baseuser_remove_certmapdata,
+)
 from .idviews import remove_ipaobject_overrides
 from ipalib.plugable import Registry
 from .baseldap import (
@@ -125,6 +126,12 @@ register = Registry()
 
 user_output_params = baseuser_output_params
 
+MEMBEROF_ADMINS = "(memberOf={})".format(
+    DN('cn=admins', api.env.container_group, api.env.basedn)
+)
+
+NOT_MEMBEROF_ADMINS = '(!{})'.format(MEMBEROF_ADMINS)
+
 
 def check_protected_member(user, protected_group_name=u'admins'):
     '''
@@ -155,6 +162,7 @@ class user(baseuser):
     label_singular            = _('User')
     object_name               = _('user')
     object_name_plural        = _('users')
+    permission_filter_objectclasses_string = '(objectclass=posixaccount)'
     managed_permissions = {
         'System: Read User Standard Attributes': {
             'replaces_global_anonymous_aci': True,
@@ -259,10 +267,8 @@ class user(baseuser):
         'System: Change User password': {
             'ipapermright': {'write'},
             'ipapermtargetfilter': [
-                '(objectclass=posixaccount)',
-                '(!(memberOf=%s))' % DN('cn=admins',
-                                        api.env.container_group,
-                                        api.env.basedn),
+                permission_filter_objectclasses_string,
+                NOT_MEMBEROF_ADMINS,
             ],
             'ipapermdefaultattr': {
                 'krbprincipalkey', 'passwordhistory', 'sambalmpassword',
@@ -279,8 +285,25 @@ class user(baseuser):
                 'PassSync Service',
             },
         },
+        'System: Change Admin User password': {
+            'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                MEMBEROF_ADMINS,
+            ],
+            'ipapermdefaultattr': {
+                'krbprincipalkey', 'passwordhistory', 'sambalmpassword',
+                'sambantpassword', 'userpassword', 'krbpasswordexpiration'
+            },
+            'default_privileges': {
+                'PassSync Service',
+            },
+        },
         'System: Manage User SSH Public Keys': {
             'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                permission_filter_objectclasses_string,
+                NOT_MEMBEROF_ADMINS,
+            ],
             'ipapermdefaultattr': {'ipasshpubkey'},
             'replaces': [
                 '(targetattr = "ipasshpubkey")(target = "ldap:///uid=*,cn=users,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Manage User SSH Public Keys";allow (write) groupdn = "ldap:///cn=Manage User SSH Public Keys,cn=permissions,cn=pbac,$SUFFIX";)',
@@ -289,6 +312,10 @@ class user(baseuser):
         },
         'System: Manage User Certificates': {
             'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                permission_filter_objectclasses_string,
+                NOT_MEMBEROF_ADMINS,
+            ],
             'ipapermdefaultattr': {'usercertificate'},
             'default_privileges': {
                 'User Administrators',
@@ -297,6 +324,10 @@ class user(baseuser):
         },
         'System: Manage User Principals': {
             'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                permission_filter_objectclasses_string,
+                NOT_MEMBEROF_ADMINS,
+            ],
             'ipapermdefaultattr': {'krbprincipalname', 'krbcanonicalname'},
             'default_privileges': {
                 'User Administrators',
@@ -305,6 +336,10 @@ class user(baseuser):
         },
         'System: Modify Users': {
             'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                permission_filter_objectclasses_string,
+                NOT_MEMBEROF_ADMINS,
+            ],
             'ipapermdefaultattr': {
                 'businesscategory', 'carlicense', 'cn', 'departmentnumber',
                 'description', 'displayname', 'employeetype',
@@ -326,6 +361,10 @@ class user(baseuser):
         },
         'System: Remove Users': {
             'ipapermright': {'delete'},
+            'ipapermtargetfilter': [
+                permission_filter_objectclasses_string,
+                NOT_MEMBEROF_ADMINS,
+            ],
             'replaces': [
                 '(target = "ldap:///uid=*,cn=users,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Remove Users";allow (delete) groupdn = "ldap:///cn=Remove Users,cn=permissions,cn=pbac,$SUFFIX";)',
             ],
@@ -333,6 +372,10 @@ class user(baseuser):
         },
         'System: Unlock User': {
             'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                permission_filter_objectclasses_string,
+                NOT_MEMBEROF_ADMINS,
+            ],
             'ipapermdefaultattr': {
                 'krblastadminunlock', 'krbloginfailedcount', 'nsaccountlock',
             },
@@ -401,15 +444,16 @@ class user(baseuser):
 
     def get_either_dn(self, *keys, **options):
         '''
-        Returns the DN of a user
+        Returns the DN of a user and their objectclasses
         The user can be active (active container) or delete (delete container)
         If the user does not exist, returns the Active user DN
         '''
         ldap = self.backend
+        oc = []
         # Check that this value is a Active user
         try:
             active_dn = self.get_dn(*keys, **options)
-            ldap.get_entry(active_dn, ['dn'])
+            oc = ldap.get_entry(active_dn, ['dn', 'objectclass'])['objectclass']
 
             # The Active user exists
             dn = active_dn
@@ -417,7 +461,9 @@ class user(baseuser):
             # Check that this value is a Delete user
             delete_dn = self.get_delete_dn(*keys, **options)
             try:
-                ldap.get_entry(delete_dn, ['dn'])
+                oc = ldap.get_entry(
+                    delete_dn, ['dn', 'objectclass']
+                )['objectclass']
 
                 # The Delete user exists
                 dn = delete_dn
@@ -425,7 +471,7 @@ class user(baseuser):
                 # The user is neither Active/Delete -> returns that Active DN
                 dn = active_dn
 
-        return dn
+        return dn, oc
 
     def _normalize_manager(self, manager):
         """
@@ -572,10 +618,6 @@ class user_add(baseuser_add):
            'ipauser' not in entry_attrs['objectclass']:
             entry_attrs['objectclass'].append('ipauser')
 
-        if 'ipauserauthtype' in entry_attrs and \
-           'ipauserauthtypeclass' not in entry_attrs['objectclass']:
-            entry_attrs['objectclass'].append('ipauserauthtypeclass')
-
         rcl = entry_attrs.get('ipatokenradiusconfiglink', None)
         if rcl:
             if 'ipatokenradiusproxyuser' not in entry_attrs['objectclass']:
@@ -617,6 +659,10 @@ class user_add(baseuser_add):
 
         entry_attrs.update(newentry)
 
+        # delete ipantsecurityidentifier if present
+        if ('ipantsecurityidentifier' in entry_attrs):
+            del entry_attrs['ipantsecurityidentifier']
+
         if options.get('random', False):
             try:
                 entry_attrs['randompassword'] = unicode(getattr(context, 'randompassword'))
@@ -624,6 +670,17 @@ class user_add(baseuser_add):
                 # if both randompassword and userpassword options were used
                 pass
 
+        # generate subid
+        default_subid = config.single_value.get(
+            'ipaUserDefaultSubordinateId', 'FALSE'
+        )
+        if default_subid == 'TRUE':
+            result = self.api.Command.subid_generate(
+                ipaowner=entry_attrs.single_value['uid'],
+                version=options['version']
+            )
+            entry_attrs["memberOf"].append(result['result']['dn'])
+
         self.obj.get_preserved_attribute(entry_attrs, options)
 
         self.post_common_callback(ldap, dn, entry_attrs, *keys, **options)
@@ -646,7 +703,7 @@ class user_del(baseuser_del):
     def _preserve_user(self, pkey, delete_container, **options):
         assert isinstance(delete_container, DN)
 
-        dn = self.obj.get_either_dn(pkey, **options)
+        dn, _oc = self.obj.get_either_dn(pkey, **options)
         delete_dn = DN(dn[0], delete_container)
         ldap = self.obj.backend
         logger.debug("preserve move %s -> %s", dn, delete_dn)
@@ -702,7 +759,7 @@ class user_del(baseuser_del):
             self._exc_wrapper(pkey, options, ldap.update_entry)(entry_attrs)
 
     def pre_callback(self, ldap, dn, *keys, **options):
-        dn = self.obj.get_either_dn(*keys, **options)
+        dn, _oc = self.obj.get_either_dn(*keys, **options)
 
         # For User life Cycle: user-del is a common plugin
         # command to delete active user (active container) and
@@ -711,7 +768,9 @@ class user_del(baseuser_del):
         # of OTP tokens.
         check_protected_member(keys[-1])
 
-        if not options.get('preserve', False):
+        preserve = options.get('preserve', False)
+
+        if not preserve:
             # Remove any ID overrides tied with this user
             try:
                 remove_ipaobject_overrides(self.obj.backend, self.obj.api, dn)
@@ -734,6 +793,15 @@ class user_del(baseuser_del):
             else:
                 self.api.Command.otptoken_del(token)
 
+        # XXX: preserving doesn't work yet, see subordinate-ids.md
+        # Delete all subid entries owned by this user.
+        results = self.api.Command.subid_find(ipaowner=owner)["result"]
+        for subid_entry in results:
+            subid_pkey = self.api.Object.subid.get_primary_key_from_dn(
+                subid_entry["dn"]
+            )
+            self.api.Command.subid_del(subid_pkey)
+
         return dn
 
     def execute(self, *keys, **options):
@@ -777,10 +845,13 @@ class user_mod(baseuser_mod):
             yield option
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
-        dn = self.obj.get_either_dn(*keys, **options)
+        dn, oc = self.obj.get_either_dn(*keys, **options)
+        if 'objectclass' not in entry_attrs and 'rename' not in options:
+            entry_attrs.update({'objectclass': oc})
         self.pre_common_callback(ldap, dn, entry_attrs, attrs_list, *keys,
                                  **options)
         validate_nsaccountlock(entry_attrs)
+        # TODO: forward uidNumber changes and rename to subids
         return dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
@@ -859,7 +930,7 @@ class user_show(baseuser_show):
     )
 
     def pre_callback(self, ldap, dn, attrs_list, *keys, **options):
-        dn = self.obj.get_either_dn(*keys, **options)
+        dn, _oc = self.obj.get_either_dn(*keys, **options)
         self.pre_common_callback(ldap, dn, attrs_list, *keys, **options)
         return dn
 
@@ -881,7 +952,7 @@ class user_undel(LDAPQuery):
         ldap = self.obj.backend
 
         # First check that the user exists and is a delete one
-        delete_dn = self.obj.get_either_dn(*keys, **options)
+        delete_dn, _oc = self.obj.get_either_dn(*keys, **options)
         try:
             self._exc_wrapper(keys, options, ldap.get_entry)(delete_dn)
         except errors.NotFound:
@@ -935,6 +1006,7 @@ class user_stage(LDAPMultiQuery):
                     u'ipauniqueid', u'krbcanonicalname',
                     u'sshpubkeyfp', u'krbextradata',
                     u'ipacertmapdata',
+                    'ipantsecurityidentifier',
                     u'nsaccountlock']
 
     def execute(self, *keys, **options):
@@ -1021,7 +1093,7 @@ class user_disable(LDAPQuery):
 
         check_protected_member(keys[-1])
 
-        dn = self.obj.get_either_dn(*keys, **options)
+        dn, _oc = self.obj.get_either_dn(*keys, **options)
         ldap.deactivate_entry(dn)
 
         return dict(
@@ -1041,7 +1113,7 @@ class user_enable(LDAPQuery):
     def execute(self, *keys, **options):
         ldap = self.obj.backend
 
-        dn = self.obj.get_either_dn(*keys, **options)
+        dn, _oc = self.obj.get_either_dn(*keys, **options)
 
         ldap.activate_entry(dn)
 
@@ -1065,7 +1137,7 @@ class user_unlock(LDAPQuery):
     msg_summary = _('Unlocked account "%(value)s"')
 
     def execute(self, *keys, **options):
-        dn = self.obj.get_either_dn(*keys, **options)
+        dn, _oc = self.obj.get_either_dn(*keys, **options)
         entry = self.obj.backend.get_entry(
             dn, ['krbLastAdminUnlock', 'krbLoginFailedCount'])
 
@@ -1149,7 +1221,7 @@ class user_status(LDAPQuery):
 
     def execute(self, *keys, **options):
         ldap = self.obj.backend
-        dn = self.api.Object.user.get_either_dn(*keys, **options)
+        dn, _oc = self.api.Object.user.get_either_dn(*keys, **options)
         attr_list = ['krbloginfailedcount', 'krblastsuccessfulauth', 'krblastfailedauth', 'nsaccountlock']
 
         disabled = False
