@@ -46,6 +46,7 @@ from cryptography import x509
 from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.asymmetric import rsa
 from cryptography.hazmat.backends import default_backend
+from datetime import datetime, timedelta
 
 from ipapython import certdb
 from ipapython import ipautil
@@ -54,7 +55,7 @@ from ipaplatform.paths import paths
 from ipaplatform.services import knownservices
 from ipapython.dn import DN
 from ipalib import errors
-from ipalib.util import get_reverse_zone_default, verify_host_resolvable
+from ipalib.util import get_reverse_zone_default
 from ipalib.constants import (
     DEFAULT_CONFIG, DOMAIN_SUFFIX_NAME, DOMAIN_LEVEL_0,
     MIN_DOMAIN_LEVEL, MAX_DOMAIN_LEVEL
@@ -64,6 +65,7 @@ from ipatests.create_external_ca import ExternalCA
 from .env_config import env_to_script
 from .host import Host
 from .firewall import Firewall
+from .resolver import ResolvedResolver
 
 logger = logging.getLogger(__name__)
 
@@ -142,10 +144,49 @@ def rpcbind_kadmin_workaround(host):
             break
 
 
+def disable_systemd_resolved_cache(host):
+    if not ResolvedResolver.is_our_resolver(host):
+        return
+    resolved_conf_file = (
+        '/etc/systemd/resolved.conf.d/zzzz-ipatests-disable-cache.conf')
+    resolved_conf = textwrap.dedent('''
+        # generated by IPA tests
+        [Resolve]
+        Cache=no
+    ''')
+    host.run_command(['mkdir', '-p', os.path.dirname(resolved_conf_file)])
+    host.put_file_contents(resolved_conf_file, resolved_conf)
+    host.run_command(['systemctl', 'restart', 'systemd-resolved'])
+
+
 def apply_common_fixes(host):
     prepare_host(host)
     fix_hostname(host)
     rpcbind_kadmin_workaround(host)
+    disable_systemd_resolved_cache(host)
+
+
+def prepare_dse_changes(host, log_level=8192):
+    """Put custom changes for dse.ldif on the host
+    """
+    ipatests_dse_path = os.path.join(host.config.test_dir, "ipatests_dse.ldif")
+    ldif = textwrap.dedent(
+        """\
+        # replication debugging
+        dn: cn=config
+        changetype: modify
+        replace: nsslapd-errorlog-level
+        nsslapd-errorlog-level: {log_level}
+
+        # server writes all access log entries directly to disk
+        dn: cn=config
+        changetype: modify
+        replace: nsslapd-accesslog-logbuffering
+        nsslapd-accesslog-logbuffering: off
+        """
+    ).format(log_level=log_level)
+    host.put_file_contents(ipatests_dse_path, ldif)
+    return ipatests_dse_path
 
 
 def allow_sync_ptr(host):
@@ -249,17 +290,6 @@ def restore_hostname(host):
         host.run_command(['rm', backupname])
 
 
-def enable_replication_debugging(host, log_level=0):
-    logger.info('Set LDAP debug level')
-    logging_ldif = textwrap.dedent("""
-        dn: cn=config
-        changetype: modify
-        replace: nsslapd-errorlog-level
-        nsslapd-errorlog-level: {log_level}
-        """.format(log_level=log_level))
-    ldapmodify_dm(host, logging_ldif)
-
-
 def enable_ds_audit_log(host, enabled='on'):
     """Enable 389-ds audit log and auditfail log
 
@@ -298,6 +328,10 @@ def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
         domain_level = host.config.domain_level
     check_domain_level(domain_level)
     apply_common_fixes(host)
+    if "--dirsrv-config-file" not in extra_args:
+        ipatests_dse = prepare_dse_changes(host)
+    else:
+        ipatests_dse = None
     fix_apache_semaphores(host)
     fw = Firewall(host)
     fw_services = ["freeipa-ldap", "freeipa-ldaps"]
@@ -310,6 +344,9 @@ def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
         '-a', host.config.admin_password,
         "--domain-level=%i" % domain_level,
     ]
+    if ipatests_dse:
+        args.extend(["--dirsrv-config-file", ipatests_dse])
+
     if unattended:
         args.append('-U')
 
@@ -335,11 +372,11 @@ def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
         fw.enable_services(fw_services)
     if result.returncode == 0 and not external_ca:
         # external CA step 1 doesn't have DS and KDC fully configured, yet
-        enable_replication_debugging(host)
         enable_ds_audit_log(host, 'on')
-        setup_sssd_debugging(host)
+        setup_sssd_conf(host)
         kinit_admin(host)
         if setup_dns:
+            setup_named_debugging(host)
             # fixup DNS zone default TTL for IPA DNS zone
             # For tests we should not wait too long
             set_default_ttl_for_ipa_dns_zone(host, raiseonerr=raiseonerr)
@@ -387,27 +424,31 @@ def master_authoritative_for_client_domain(master, client):
     return result.returncode == 0
 
 
-def config_host_resolvconf_with_master_data(master, host):
-    """
-    Configure host /etc/resolv.conf to use master as DNS server
-    """
-    content = ('search {domain}\nnameserver {master_ip}'
-               .format(domain=master.domain.name, master_ip=master.ip))
-    host.put_file_contents(paths.RESOLV_CONF, content)
-
-
 def install_replica(master, replica, setup_ca=True, setup_dns=False,
                     setup_kra=False, setup_adtrust=False, extra_args=(),
                     domain_level=None, unattended=True, stdin_text=None,
-                    raiseonerr=True, promote=True):
+                    raiseonerr=True, promote=True, nameservers='master'):
     """
     This task installs client and then promote it to the replica
+
+    :param nameservers: nameservers to write in resolver config. Possible
+       values:
+       * "master" - use ip of `master` parameter
+       * None - do not setup resolver
+       * IP_ADDRESS or [IP_ADDRESS, ...] - use this address as resolver
+
     """
     replica_args = list(extra_args)  # needed for client's ntp options
     if domain_level is None:
         domain_level = domainlevel(master)
     check_domain_level(domain_level)
     apply_common_fixes(replica)
+
+    if "--dirsrv-config-file" not in extra_args:
+        ipatests_dse = prepare_dse_changes(replica)
+    else:
+        ipatests_dse = None
+
     allow_sync_ptr(master)
     fw = Firewall(replica)
     fw_services = ["freeipa-ldap", "freeipa-ldaps"]
@@ -426,12 +467,18 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
         for ntp_arg in ntp_args:
             replica_args.remove(ntp_arg)
 
-        install_client(master, replica, extra_args=ntp_args)
+        install_client(master, replica, extra_args=ntp_args,
+                       nameservers=nameservers)
     else:
         # for one step installation of replica we need authorized user
         # to enroll a replica and master server to contact
         args.extend(['--principal', replica.config.admin_name,
                      '--server', master.hostname])
+        replica.resolver.backup()
+        if nameservers is not None:
+            if nameservers == 'master':
+                nameservers = master.ip
+            replica.resolver.setup_resolver(nameservers, master.domain.name)
 
     if unattended:
         args.append('-U')
@@ -457,22 +504,34 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
     fix_apache_semaphores(replica)
     args.extend(['--realm', replica.domain.realm,
                  '--domain', replica.domain.name])
+    if ipatests_dse:
+        args.extend(["--dirsrv-config-file", ipatests_dse])
+
     fw.enable_services(fw_services)
 
     result = replica.run_command(args, raiseonerr=raiseonerr,
                                  stdin_text=stdin_text)
     if result.returncode == 0:
-        enable_replication_debugging(replica)
         enable_ds_audit_log(replica, 'on')
-        setup_sssd_debugging(replica)
+        setup_sssd_conf(replica)
         kinit_admin(replica)
+        if setup_dns:
+            setup_named_debugging(replica)
     else:
         fw.disable_services(fw_services)
     return result
 
 
 def install_client(master, client, extra_args=[], user=None,
-                   password=None, unattended=True, stdin_text=None):
+                   password=None, unattended=True, stdin_text=None,
+                   nameservers='master'):
+    """
+    :param nameservers: nameservers to write in resolver config. Possible
+           values:
+           * "master" - use ip of `master` parameter
+           * None - do not setup resolver
+           * IP_ADDRESS or [IP_ADDRESS, ...] - use this address as resolver
+    """
     apply_common_fixes(client)
     allow_sync_ptr(master)
     # Now, for the situations where a client resides in a different subnet from
@@ -482,6 +541,11 @@ def install_client(master, client, extra_args=[], user=None,
     if not error:
         master.run_command(["ipa", "dnszone-mod", zone,
                             "--dynamic-update=TRUE"])
+    if nameservers is not None:
+        client.resolver.backup()
+        if nameservers == 'master':
+            nameservers = master.ip
+        client.resolver.setup_resolver(nameservers, master.domain.name)
     if user is None:
         user = client.config.admin_name
     if password is None:
@@ -503,7 +567,7 @@ def install_client(master, client, extra_args=[], user=None,
 
     result = client.run_command(args, stdin_text=stdin_text)
 
-    setup_sssd_debugging(client)
+    setup_sssd_conf(client)
     kinit_admin(client)
 
     return result
@@ -532,7 +596,9 @@ def install_adtrust(host):
     dig_command = ['dig', 'SRV', '+short', '@localhost',
                    '_ldap._tcp.%s' % host.domain.name]
     dig_output = '0 100 389 %s.' % host.hostname
-    dig_test = lambda x: re.search(re.escape(dig_output), x)
+
+    def dig_test(x):
+        return re.search(re.escape(dig_output), x)
 
     run_repeatedly(host, dig_command, test=dig_test)
 
@@ -635,7 +701,7 @@ def unconfigure_windows_dns_for_trust(ad, master):
     ad.run_command(['dnscmd', '/zonedelete', master.domain.name, '/f'])
 
 
-def establish_trust_with_ad(master, ad_domain, extra_args=(),
+def establish_trust_with_ad(master, ad_domain, ad_admin=None, extra_args=(),
                             shared_secret=None):
     """
     Establishes trust with Active Directory. Trust type is detected depending
@@ -643,6 +709,9 @@ def establish_trust_with_ad(master, ad_domain, extra_args=(),
 
     Use extra arguments to pass extra arguments to the trust-add command, such
     as --range-type="ipa-ad-trust" to enforce a particular range type.
+
+    If ad_admin is not provided, name will be constructed as
+    "Administrator@<ad_domain>".
     """
 
     # Force KDC to reload MS-PAC info by trying to get TGT for HTTP
@@ -660,7 +729,9 @@ def establish_trust_with_ad(master, ad_domain, extra_args=(),
         extra_args += ['--trust-secret']
         stdin_text = shared_secret
     else:
-        extra_args += ['--admin', 'Administrator', '--password']
+        if ad_admin is None:
+            ad_admin = 'Administrator@{}'.format(ad_domain)
+        extra_args += ['--admin', ad_admin, '--password']
         stdin_text = master.config.ad_admin_password
     run_repeatedly(
         master, ['ipa', 'trust-add', '--type', 'ad', ad_domain] + extra_args,
@@ -671,11 +742,13 @@ def establish_trust_with_ad(master, ad_domain, extra_args=(),
     time.sleep(60)
 
 
-def remove_trust_with_ad(master, ad_domain):
+def remove_trust_with_ad(master, ad_domain, ad_hostname):
     """
     Removes trust with Active Directory. Also removes the associated ID range.
     """
 
+    remove_trust_info_from_ad(master, ad_domain, ad_hostname)
+
     kinit_admin(master)
 
     # Remove the trust
@@ -685,14 +758,19 @@ def remove_trust_with_ad(master, ad_domain):
     range_name = ad_domain.upper() + '_id_range'
     master.run_command(['ipa', 'idrange-del', range_name])
 
-    remove_trust_info_from_ad(master, ad_domain)
 
-
-def remove_trust_info_from_ad(master, ad_domain):
+def remove_trust_info_from_ad(master, ad_domain, ad_hostname):
     # Remove record about trust from AD
-    master.run_command(['rpcclient', ad_domain,
-                        '-U\\Administrator%{}'.format(
-                            master.config.ad_admin_password),
+    kinit_as_user(master,
+                  'Administrator@{}'.format(ad_domain.upper()),
+                  master.config.ad_admin_password)
+    # Detect whether rpcclient supports -k or --use-kerberos option
+    res = master.run_command(['rpcclient', '-h'], raiseonerr=False)
+    if "--use-kerberos" in res.stderr_text:
+        rpcclient_krb5_knob = "--use-kerberos=desired"
+    else:
+        rpcclient_krb5_knob = "-k"
+    master.run_command(['rpcclient', rpcclient_krb5_knob, ad_hostname,
                         '-c', 'deletetrustdom {}'.format(master.domain.name)],
                        raiseonerr=False)
 
@@ -720,28 +798,59 @@ def configure_auth_to_local_rule(master, ad):
     master.run_command(['systemctl', 'restart', 'sssd'])
 
 
-def setup_sssd_debugging(host):
+def setup_sssd_conf(host):
     """
-    Sets debug level to 7 in each section of sssd.conf file.
+    Configures sssd
     """
+    # sssd in not published on PyPI
+    from SSSDConfig import NoOptionError
 
-    # Set debug level in each section of sssd.conf file to 7
-    # First, remove any previous occurences
-    host.run_command(['sed', '-i',
-                      '/debug_level = 7/d',
-                      paths.SSSD_CONF],
-                     raiseonerr=False)
+    with remote_sssd_config(host) as sssd_config:
+        # sssd 2.5.0 https://github.com/SSSD/sssd/issues/5635
+        try:
+            sssd_config.edit_domain(host.domain, "ldap_sudo_random_offset", 0)
+        except NoOptionError:
+            # sssd doesn't support ldap_sudo_random_offset
+            pass
 
-    # Add the debug directive to each section
-    host.run_command(['sed', '-i',
-                      r'/\[*\]/ a\debug_level = 7',
-                      paths.SSSD_CONF],
-                     raiseonerr=False)
+        for sssd_service_name in sssd_config.list_services():
+            sssd_config.edit_service(sssd_service_name, "debug_level", 7)
+
+        for sssd_domain_name in sssd_config.list_domains():
+            sssd_config.edit_domain(sssd_domain_name, "debug_level", 7)
 
     # Clear the cache and restart SSSD
     clear_sssd_cache(host)
 
 
+def setup_named_debugging(host):
+    """
+    Sets debug level to debug in each section of custom logging config
+    """
+
+    host.run_command(
+        [
+            "sed",
+            "-i",
+            "s/severity info;/severity debug;/",
+            paths.NAMED_LOGGING_OPTIONS_CONF,
+        ],
+    )
+    host.run_command(["cat", paths.NAMED_LOGGING_OPTIONS_CONF])
+    result = host.run_command(
+        [
+            "python3",
+            "-c",
+            (
+                "from ipaplatform.services import knownservices; "
+                "print(knownservices.named.systemd_name)"
+            ),
+        ]
+    )
+    service_name = result.stdout_text.strip()
+    host.run_command(["systemctl", "restart", service_name])
+
+
 @contextmanager
 def remote_sssd_config(host):
     """Context manager for editing sssd config file on a remote host.
@@ -872,6 +981,7 @@ def clear_sssd_cache(host):
                      "xargs rm -fv")
     host.run_command(['rm', '-fv', paths.SSSD_MC_GROUP])
     host.run_command(['rm', '-fv', paths.SSSD_MC_PASSWD])
+    host.run_command(['rm', '-fv', paths.SSSD_MC_INITGROUPS])
 
     if systemd_available:
         host.run_command(['systemctl', 'start', 'sssd'])
@@ -935,9 +1045,14 @@ def disconnect_replica(master, replica, domain_level=None,
                             ])
 
 
+def kinit_user(host, user, password, raiseonerr=True):
+    return host.run_command(['kinit', user], raiseonerr=raiseonerr,
+                            stdin_text=password)
+
+
 def kinit_admin(host, raiseonerr=True):
-    return host.run_command(['kinit', 'admin'], raiseonerr=raiseonerr,
-                            stdin_text=host.config.admin_password)
+    return kinit_user(host, 'admin', host.config.admin_password,
+                      raiseonerr=raiseonerr)
 
 
 def uninstall_master(host, ignore_topology_disconnect=True,
@@ -957,6 +1072,17 @@ def uninstall_master(host, ignore_topology_disconnect=True,
 
     result = host.run_command(uninstall_cmd)
     assert "Traceback" not in result.stdout_text
+
+    # Check that IPA certs have been deleted after uninstall
+    # Related: https://pagure.io/freeipa/issue/8614
+    assert host.run_command(['test', '-f', paths.IPA_CA_CRT],
+                            raiseonerr=False).returncode == 1
+    assert host.run_command(['test', '-f', paths.IPA_P11_KIT],
+                            raiseonerr=False).returncode == 1
+    assert "IPA CA" not in host.run_command(
+        ["trust", "list"], log_stdout=False
+    ).stdout_text
+
     if clean:
         Firewall(host).disable_services(["freeipa-ldap", "freeipa-ldaps",
                                          "freeipa-trust", "dns"])
@@ -972,10 +1098,12 @@ def uninstall_master(host, ignore_topology_disconnect=True,
                       paths.IPA_RENEWAL_LOCK,
                       paths.REPLICA_INFO_GPG_TEMPLATE % host.hostname],
                      raiseonerr=False)
-    host.run_command("find /var/lib/sss/keytabs -name '*.keytab' | "
-                     "xargs rm -fv", raiseonerr=False)
+    host.run_command("find %s -name '*.keytab' | "
+                     "xargs rm -fv" % paths.SSSD_KEYTABS_DIR, raiseonerr=False)
     host.run_command("find /run/ipa -name 'krb5*' | xargs rm -fv",
                      raiseonerr=False)
+    while host.resolver.has_backups():
+        host.resolver.restore()
     if clean:
         unapply_fixes(host)
 
@@ -983,6 +1111,8 @@ def uninstall_master(host, ignore_topology_disconnect=True,
 def uninstall_client(host):
     host.run_command(['ipa-client-install', '--uninstall', '-U'],
                      raiseonerr=False)
+    while host.resolver.has_backups():
+        host.resolver.restore()
     unapply_fixes(host)
 
 
@@ -1297,18 +1427,35 @@ def install_topo(topo, master, replicas, clients, domain_level=None,
             install_replica(
                 parent, child,
                 setup_ca=setup_replica_cas,
-                setup_kra=setup_replica_kras
+                setup_kra=setup_replica_kras,
+                nameservers=master.ip,
             )
         installed.add(child)
     install_clients([master] + replicas, clients, clients_extra_args)
 
 
-def install_clients(servers, clients, extra_args=()):
-    """Install IPA clients, distributing them among the given servers"""
+def install_clients(servers, clients, extra_args=(),
+                    nameservers='first'):
+    """Install IPA clients, distributing them among the given servers
+
+       :param nameservers: nameservers to write in resolver config on clients.
+       Possible values:
+       * "first" - use ip of the first item in `servers` parameter
+       * "distribute" - use ip of master/replica which is used for client
+                        installation
+       * None - do not setup resolver
+       * IP_ADDRESS or [IP_ADDRESS, ...] - use this address as resolver
+    """
     izip = getattr(itertools, 'izip', zip)
+    client_nameservers = nameservers
     for server, client in izip(itertools.cycle(servers), clients):
         logger.info('Installing client %s on %s', server, client)
-        install_client(server, client, extra_args)
+        if nameservers == 'distribute':
+            client_nameservers = server.ip
+        if nameservers == 'first':
+            client_nameservers = servers[0].ip
+        install_client(server, client, extra_args,
+                       nameservers=client_nameservers)
 
 
 def _entries_to_ldif(entries):
@@ -1404,14 +1551,7 @@ def add_a_records_for_hosts_in_master_domain(master):
     for host in master.domain.hosts:
         # We don't need to take care of the zone creation since it is master
         # domain
-        try:
-            verify_host_resolvable(host.hostname)
-            logger.debug("The host (%s) is resolvable.", host.hostname)
-        except errors.DNSNotARecordError:
-            logger.debug("Hostname (%s) does not have A/AAAA record. Adding "
-                         "new one.",
-                         host.hostname)
-            add_a_record(master, host)
+        add_a_record(master, host)
 
 
 def add_a_record(master, host):
@@ -1751,7 +1891,8 @@ def get_host_ip_with_hostmask(host):
         return None
 
 
-def ldappasswd_user_change(user, oldpw, newpw, master, use_dirman=False):
+def ldappasswd_user_change(user, oldpw, newpw, master, use_dirman=False,
+                           raiseonerr=True):
     container_user = dict(DEFAULT_CONFIG)['container_user']
     basedn = master.domain.basedn
 
@@ -1766,7 +1907,7 @@ def ldappasswd_user_change(user, oldpw, newpw, master, use_dirman=False):
     else:
         args = [paths.LDAPPASSWD, '-D', userdn, '-w', oldpw, '-a', oldpw,
                 '-s', newpw, '-x', '-ZZ', '-H', master_ldap_uri]
-    master.run_command(args)
+    return master.run_command(args, raiseonerr=raiseonerr)
 
 
 def ldappasswd_sysaccount_change(user, oldpw, newpw, master, use_dirman=False):
@@ -1901,6 +2042,11 @@ def user_add(host, login, first='test', last='user', extra_args=(),
     return host.run_command(cmd, stdin_text=stdin_text)
 
 
+def user_del(host, login):
+    cmd = ["ipa", "user-del", login]
+    return host.run_command(cmd)
+
+
 def group_add(host, groupname, extra_args=()):
     cmd = [
         "ipa", "group-add", groupname,
@@ -1953,13 +2099,15 @@ def ldapsearch_dm(host, base, ldap_args, scope='sub', **kwargs):
     return host.run_command(args, **kwargs)
 
 
-def create_temp_file(host, directory=None, create_file=True):
-    """Creates temporary file using mktemp."""
+def create_temp_file(host, directory=None, suffix=None, create_file=True):
+    """Creates temporary file using mktemp. See `man 1 mktemp`."""
     cmd = ['mktemp']
     if create_file is False:
         cmd += ['--dry-run']
     if directory is not None:
         cmd += ['-p', directory]
+    if suffix is not None:
+        cmd.extend(["--suffix", suffix])
     return host.run_command(cmd).stdout_text.strip()
 
 
@@ -1971,10 +2119,25 @@ def create_active_user(host, login, password, first='test', last='user',
     user_add(host, login, first=first, last=last, extra_args=extra_args,
              password=temp_password)
     if krb5_trace:
-        host.run_command(
-            "KRB5_TRACE=/dev/stdout kinit %s" % login,
-            stdin_text='{0}\n{1}\n{1}\n'.format(temp_password, password)
+        # Retrieve kdcinfo.$REALM before changing the user's password.
+        get_kdcinfo(host)
+        # This tends to fail when the KDC the password is
+        # reset on is not the same as the one we immediately
+        # request a TGT from. This should not be the case as SSSD
+        # tries to pin itself to an IPA server.
+        #
+        # Note raiseonerr=False:
+        # the assert is located after kdcinfo retrieval.
+        result = host.run_command(
+            f"KRB5_TRACE=/dev/stdout SSSD_KRB5_LOCATOR_DEBUG=1 kinit {login}",
+            stdin_text='{0}\n{1}\n{1}\n'.format(
+                temp_password, password
+            ), raiseonerr=False
         )
+        # Retrieve kdc.$REALM after the password change, just in case SSSD
+        # domain status flipped to online during the password change.
+        get_kdcinfo(host)
+        assert result.returncode == 0
     else:
         host.run_command(
             ['kinit', login],
@@ -2001,14 +2164,76 @@ def run_command_as_user(host, user, command, *args, **kwargs):
     return host.run_command(command, *args, **kwargs)
 
 
-def kinit_as_user(host, user, password, krb5_trace=False):
+def kinit_as_user(host, user, password, krb5_trace=False, raiseonerr=True):
+    """Launch kinit as user on host.
+    If krb5_trace, then set KRB5_TRACE=/dev/stdout, SSSD_KRB5_LOCATOR_DEBUG=1
+    and collect /var/lib/sss/pubconf/kdcinfo.$REALM
+    as this file contains the list of KRB5KDC IPs SSSD uses.
+    https://pagure.io/freeipa/issue/8510
+    """
+
     if krb5_trace:
-        host.run_command(
-            "KRB5_TRACE=/dev/stdout kinit %s" % user,
-            stdin_text='{0}\n'.format(password)
+        # Retrieve kdcinfo.$REALM before changing the user's password.
+        get_kdcinfo(host)
+        # This tends to fail when the KDC the password is
+        # reset on is not the same as the one we immediately
+        # request a TGT from. This should not be the case as SSSD
+        # tries to pin itself to an IPA server.
+        #
+        # Note raiseonerr=False:
+        # the assert is located after kdcinfo retrieval.
+        result = host.run_command(
+            f"KRB5_TRACE=/dev/stdout SSSD_KRB5_LOCATOR_DEBUG=1 kinit {user}",
+            stdin_text='{0}\n'.format(password),
+            raiseonerr=False
         )
+        # Retrieve kdc.$REALM after the password change, just in case SSSD
+        # domain status flipped to online during the password change.
+        get_kdcinfo(host)
+        if raiseonerr:
+            assert result.returncode == 0
+        return result
     else:
-        host.run_command(['kinit', user], stdin_text='{0}\n'.format(password))
+        return host.run_command(
+            ['kinit', user],
+            stdin_text='{0}\n'.format(password),
+            raiseonerr=raiseonerr)
+
+
+def get_kdcinfo(host):
+    """Retrieve /var/lib/sss/pubconf/kdcinfo.$REALM on host.
+    That file contains the IP of the KDC SSSD should be pinned to.
+    """
+    logger.info(
+        'Collecting kdcinfo log from: %s', host.hostname
+    )
+    if check_if_sssd_is_online(host):
+        logger.info("SSSD considers domain %s online.", host.domain.realm)
+    else:
+        logger.warning(
+            "SSSD considers domain %s offline.", host.domain.realm
+        )
+    kdcinfo = None
+    try:
+        kdcinfo = host.get_file_contents(
+            "/var/lib/sss/pubconf/kdcinfo.{}".format(host.domain.realm)
+        )
+        logger.info(
+            'kdcinfo %s contains:\n%s', host.hostname, kdcinfo
+        )
+        if check_if_sssd_is_online(host) is False:
+            logger.warning(
+                "SSSD still considers domain %s offline.",
+                host.domain.realm
+            )
+    except (OSError, IOError) as e:
+        logger.warning(
+            "Exception collecting kdcinfo.%s: %s\n"
+            "SSSD is able to function without this file but logon "
+            "attempts immediately after a password change might break.",
+            host.domain.realm, e
+        )
+    return kdcinfo
 
 
 KeyEntry = collections.namedtuple('KeyEntry',
@@ -2048,10 +2273,10 @@ class KerberosKeyCopier:
             [paths.KLIST, "-eK", "-k", keytab], log_stdout=False)
 
         keys_to_sync = []
-        for l in result.stdout_text.splitlines():
-            if (princ in l and any(e in l for e in self.valid_etypes)):
+        for line in result.stdout_text.splitlines():
+            if (princ in line and any(e in line for e in self.valid_etypes)):
 
-                els = l.split()
+                els = line.split()
                 els[-2] = els[-2].strip('()')
                 els[-1] = els[-1].strip('()')
                 keys_to_sync.append(KeyEntry._make(els))
@@ -2059,19 +2284,6 @@ class KerberosKeyCopier:
         return keys_to_sync
 
     def copy_key(self, keytab, keyentry):
-        # keyentry.key is a hex value of the actual key
-        # prefixed with 0x, as produced by klist -K -k.
-        # However, ktutil accepts hex value without 0x, so
-        # we should strip first two characters.
-        stdin = textwrap.dedent("""\
-        rkt {keytab}
-        addent -key -p {principal} -k {kvno} -e {etype}
-        {key}
-        wkt {keytab}
-        """).format(keytab=keytab, principal=keyentry.principal,
-                    kvno=keyentry.kvno, etype=keyentry.etype,
-                    key=keyentry.key[2:])
-
         def get_keytab_mtime():
             """Get keytab file mtime.
 
@@ -2086,8 +2298,26 @@ class KerberosKeyCopier:
 
         mtime_before = get_keytab_mtime()
 
-        self.host.run_command([paths.KTUTIL], stdin_text=stdin,
-                              log_stdout=False)
+        with self.host.spawn_expect(paths.KTUTIL, default_timeout=5,
+                                    extra_ssh_options=['-t']) as e:
+            e.expect_exact('ktutil:')
+            e.sendline('rkt {}'.format(keytab))
+            e.expect_exact('ktutil:')
+            e.sendline(
+                'addent -key -p {principal} -k {kvno} -e {etype}'
+                .format(principal=keyentry.principal, kvno=keyentry.kvno,
+                        etype=keyentry.etype,))
+            e.expect('Key for.+:')
+            # keyentry.key is a hex value of the actual key
+            # prefixed with 0x, as produced by klist -K -k.
+            # However, ktutil accepts hex value without 0x, so
+            # we should strip first two characters.
+            e.sendline(keyentry.key[2:])
+            e.expect_exact('ktutil:')
+            e.sendline('wkt {}'.format(keytab))
+            e.expect_exact('ktutil:')
+            e.sendline('q')
+
         if mtime_before == get_keytab_mtime():
             raise Exception('{} did not update keytab file "{}"'.format(
                 paths.KTUTIL, keytab))
@@ -2218,9 +2448,9 @@ def install_packages(host, pkgs):
     :param pkgs: packages to install, provided as a list of strings
     """
     platform = get_platform(host)
-    if platform in ('rhel', 'fedora'):
+    if platform in {'rhel', 'fedora'}:
         install_cmd = ['/usr/bin/dnf', 'install', '-y']
-    elif platform in ('ubuntu'):
+    elif platform in {'debian', 'ubuntu'}:
         install_cmd = ['apt-get', 'install', '-y']
     else:
         raise ValueError('install_packages: unknown platform %s' % platform)
@@ -2252,20 +2482,29 @@ def download_packages(host, pkgs):
     return tmpdir
 
 
-def uninstall_packages(host, pkgs):
+def uninstall_packages(host, pkgs, nodeps=False):
     """Uninstall packages on a remote host.
-    :param host: the host where the uninstallation takes place
-    :param pkgs: packages to uninstall, provided as a list of strings
+    :param host: the host where the uninstallation takes place.
+    :param pkgs: packages to uninstall, provided as a list of strings.
+    :param nodeps: ignore dependencies (dangerous!).
     """
     platform = get_platform(host)
-    # Only supports RHEL 8+ and Fedora for now
-    if platform in ('rhel', 'fedora'):
-        install_cmd = ['/usr/bin/dnf', 'remove', '-y']
-    elif platform in ('ubuntu'):
-        install_cmd = ['apt-get', 'remove', '-y']
+    if platform not in {"rhel", "fedora", "debian", "ubuntu"}:
+        raise ValueError(f"uninstall_packages: unknown platform {platform}")
+    if nodeps:
+        if platform in {"rhel", "fedora"}:
+            cmd = ["rpm", "-e", "--nodeps"]
+        elif platform in {"debian", "ubuntu"}:
+            cmd = ["dpkg", "-P", "--force-depends"]
+        for package in pkgs:
+            # keep raiseonerr=True here. --fcami
+            host.run_command(cmd + [package])
     else:
-        raise ValueError('install_packages: unknown platform %s' % platform)
-    host.run_command(install_cmd + pkgs, raiseonerr=False)
+        if platform in {"rhel", "fedora"}:
+            cmd = ["/usr/bin/dnf", "remove", "-y"]
+        elif platform in {"debian", "ubuntu"}:
+            cmd = ["apt-get", "remove", "-y"]
+        host.run_command(cmd + pkgs, raiseonerr=False)
 
 
 def wait_for_request(host, request_id, timeout=120):
@@ -2315,6 +2554,23 @@ def wait_for_certmonger_status(host, status, request_id, timeout=120):
     return state
 
 
+def check_if_sssd_is_online(host):
+    """Check whether SSSD considers the IPA domain online.
+
+    Analyse sssctl domain-status <domain>'s output to see if SSSD considers
+    the IPA domain of the host online.
+
+    Could be extended for Trust domains as well.
+    """
+    pattern = re.compile(r'Online status: (?P<state>.*)\n')
+    result = host.run_command(
+        [paths.SSSCTL, "domain-status", host.domain.name, "-o"]
+    )
+    match = pattern.search(result.stdout_text)
+    state = match.group('state')
+    return state == 'Online'
+
+
 def wait_for_sssd_domain_status_online(host, timeout=120):
     """Wait up to timeout (in seconds) for sssd domain status to become Online
 
@@ -2325,14 +2581,8 @@ def wait_for_sssd_domain_status_online(host, timeout=120):
     as SSSD may need a while before it reconnects and switches from Offline
     mode to Online.
     """
-    pattern = re.compile(r'Online status: (?P<state>.*)\n')
     for _i in range(0, timeout, 5):
-        result = host.run_command(
-            [paths.SSSCTL, "domain-status", host.domain.name, "-o"]
-        )
-        match = pattern.search(result.stdout_text)
-        state = match.group('state')
-        if state == 'Online':
+        if check_if_sssd_is_online(host):
             break
         time.sleep(5)
     else:
@@ -2345,6 +2595,69 @@ def get_sssd_version(host):
     return parse_version(version)
 
 
+def get_pki_version(host):
+    """Get pki version on remote host."""
+    data = host.get_file_contents("/usr/share/pki/VERSION", encoding="utf-8")
+
+    groups = re.match(r'.*\nSpecification-Version: ([\d+\.]*)\n.*', data)
+    if groups:
+        version_string = groups.groups(0)[0]
+        return parse_version(version_string)
+    else:
+        raise ValueError("get_pki_version: pki is not installed")
+
+
+def get_healthcheck_version(host):
+    """
+    Function to get healthcheck version on fedora and rhel
+    """
+    platform = get_platform(host)
+    if platform in ("rhel", "fedora"):
+        cmd = host.run_command(
+            ["rpm", "-qa", "--qf", "%{VERSION}", "*ipa-healthcheck"]
+        )
+        healthcheck_version = cmd.stdout_text
+        if not healthcheck_version:
+            raise ValueError(
+                "get_healthcheck_version: "
+                "ipa-healthcheck package is not installed"
+            )
+    else:
+        raise ValueError(
+            "get_healthcheck_version: unknown platform %s" % platform
+        )
+    return healthcheck_version
+
+
+def wait_for_ipa_to_start(host, timeout=60):
+    """Wait up to timeout seconds for ipa to start on a given host.
+
+    If DS is restarted, and SSSD must be online, please consider using
+    wait_for_sssd_domain_status_online(host) in the test after calling
+    this method.
+    """
+    interval = 1
+    end_time = datetime.now() + timedelta(seconds=timeout)
+    for _i in range(0, timeout, interval):
+        if datetime.now() > end_time:
+            raise RuntimeError("Request timed out")
+        time.sleep(interval)
+        result = host.run_command(
+            [paths.IPACTL, "status"], raiseonerr=False
+        )
+        if result.returncode == 0:
+            break
+
+
+def dns_update_system_records(host):
+    """Runs "ipa dns-update-system-records" on "host".
+    """
+    kinit_admin(host)
+    host.run_command(
+        ["ipa", "dns-update-system-records"]
+    )
+
+
 def run_ssh_cmd(
     from_host=None, to_host=None, username=None, cmd=None,
     auth_method=None, password=None, private_key_path=None,
@@ -2456,8 +2769,11 @@ def run_ssh_cmd(
 
     if auth_method == "password":
         if expect_auth_success is True:
-            assert "Authentication succeeded (keyboard-interactive)" in \
-                stderr
+            patterns = [
+                r'Authenticated to .* using "keyboard-interactive"',
+                r'Authentication succeeded \(keyboard-interactive\)'
+            ]
+            assert any(re.search(pattern, stderr) for pattern in patterns)
             # do not assert the return code:
             # it can be >0 if the command failed.
         elif expect_auth_failure is True:
@@ -2466,7 +2782,11 @@ def run_ssh_cmd(
             assert "Authentication succeeded" not in stderr
     elif auth_method == "key":
         if expect_auth_success is True:
-            assert "Authentication succeeded (publickey)" in stderr
+            patterns = [
+                r'Authenticated to .* using "publickey"',
+                r'Authentication succeeded \(publickey\)'
+            ]
+            assert any(re.search(pattern, stderr) for pattern in patterns)
             # do not assert the return code:
             # it can be >0 if the command failed.
         elif expect_auth_failure is True:
@@ -2475,3 +2795,31 @@ def run_ssh_cmd(
             assert "Authentication succeeded" not in stderr
             assert "No more authentication methods to try." in stderr
     return (return_code, stdout, stderr)
+
+
+def is_package_installed(host, pkg):
+    platform = get_platform(host)
+    if platform in {'rhel', 'fedora'}:
+        result = host.run_command(
+            ['rpm', '-q', pkg], raiseonerr=False
+        )
+    elif platform in {'debian', 'ubuntu'}:
+        result = host.run_command(
+            ['dpkg', '-s', pkg], raiseonerr=False
+        )
+    else:
+        raise ValueError(
+            'is_package_installed: unknown platform %s' % platform
+        )
+    return result.returncode == 0
+
+
+def move_date(host, chrony_cmd, date_str):
+    """Helper method to move system date
+    :param host: host on which date is to be manipulated
+    :param chrony_cmd: systemctl command to apply to
+                       chrony service, for instance 'start', 'stop'
+    :param date_str: date string to change the date i.e '3years2months1day1'
+    """
+    host.run_command(['systemctl', chrony_cmd, 'chronyd'])
+    host.run_command(['date', '-s', date_str])
