@@ -130,7 +130,7 @@ class TestNFS(IntegrationTest):
         nfsclt = self.clients[1]
 
         # for journalctl --since
-        since = time.strftime('%H:%M:%S')
+        since = time.strftime('%Y-%m-%d %H:%M:%S')
         nfsclt.run_command(["systemctl", "restart", "rpc-gssd"])
         time.sleep(WAIT_AFTER_INSTALL)
         mountpoints = ("/mnt/krb", "/mnt/std", "/home")
@@ -294,7 +294,7 @@ class TestIpaClientAutomountFileRestore(IntegrationTest):
     ):
 
         # In order to get a more pure sum, one that ignores the Generated
-        # header and any white space we have to do a bit of work...
+        # header and any whitespace we have to do a bit of work...
         sha256nsswitch_cmd = \
             'egrep -v "Generated|^$" /etc/nsswitch.conf | sed "s/\\s//g" ' \
             '| sort | sha256sum'
