@@ -11,12 +11,15 @@ from __future__ import absolute_import
 from functools import partial
 import textwrap
 import re
+import os
 
 import pytest
 
 from ipatests.test_integration.base import IntegrationTest
 from ipatests.pytest_ipa.integration import tasks
+from ipaplatform.osinfo import osinfo
 from ipaplatform.paths import paths
+from ipatests.pytest_ipa.integration import skip_if_fips
 
 
 def wait_smbd_functional(host):
@@ -59,8 +62,6 @@ class TestSMB(IntegrationTest):
         cls.smbclient = cls.clients[1]
         cls.ad_user = '{}@{}'.format(cls.ad_user_login, cls.ad.domain.name)
 
-        tasks.config_host_resolvconf_with_master_data(cls.master,
-                                                      cls.smbclient)
         tasks.install_adtrust(cls.master)
         tasks.configure_dns_for_trust(cls.master, cls.ad)
         tasks.configure_windows_dns_for_trust(cls.ad, cls.master)
@@ -75,18 +76,7 @@ class TestSMB(IntegrationTest):
         for user in [cls.ipa_user1, cls.ipa_user2, cls.ad_user]:
             tasks.run_command_as_user(cls.smbserver, user, ['stat', '.'])
 
-    @pytest.yield_fixture
-    def enable_smb_client_dns_lookup_kdc(self):
-        smbclient = self.smbclient
-        with tasks.FileBackup(smbclient, paths.KRB5_CONF):
-            krb5_conf = smbclient.get_file_contents(
-                paths.KRB5_CONF, encoding='utf-8')
-            krb5_conf = krb5_conf.replace(
-                'dns_lookup_kdc = false', 'dns_lookup_kdc = true')
-            smbclient.put_file_contents(paths.KRB5_CONF, krb5_conf)
-            yield
-
-    @pytest.yield_fixture
+    @pytest.fixture
     def samba_share_public(self):
         """Setup share outside /home on samba server."""
         share_name = 'shared'
@@ -176,9 +166,28 @@ class TestSMB(IntegrationTest):
                 encoding='utf-8')
             assert file_contents_at_server == test_string
 
-            # check access using smbclient utility
+            # Detect whether smbclient uses -k or --use-kerberos=required
+            # https://pagure.io/freeipa/issue/8926
+            # then check access using smbclient.
             res = run_smb_client(
-                ['smbclient', '-k', share['unc'], '-c', 'dir'])
+                [
+                    "smbclient",
+                    "-h",
+                ], raiseonerr=False
+            )
+            if "[-k|--kerberos]" in res.stderr_text:
+                smbclient_krb5_knob = "-k"
+            else:
+                smbclient_krb5_knob = "--use-kerberos=desired"
+            res = run_smb_client(
+                [
+                    "smbclient",
+                    smbclient_krb5_knob,
+                    share["unc"],
+                    "-c",
+                    "dir",
+                ]
+            )
             assert test_dir in res.stdout_text
 
             # check file and dir removal from client side
@@ -250,6 +259,30 @@ class TestSMB(IntegrationTest):
         # started - this way we prevent other tests from failing
         self.smb_installation_check(samba_install_result)
 
+    def test_authentication_with_smb_cifs_principal_alias(self):
+        """Test that we can auth as NetBIOS alias
+
+        cifs/... principal on SMB server side has NetBIOS name of the SMB
+        server as its alias. Test that we can actually initialize credentials
+        using this alias. We don't need to use it anywhere in Samba, just
+        verify that alias works.
+        Test for  https://pagure.io/freeipa/issue/8291"""
+        netbiosname = self.smbserver.hostname.split('.')[0].upper() + '$'
+        copier = tasks.KerberosKeyCopier(self.smbserver)
+
+        principal = 'cifs/{hostname}@{realm}'.format(
+            hostname=self.smbserver.hostname, realm=copier.realm)
+        alias = '{netbiosname}@{realm}'.format(
+            netbiosname=netbiosname, realm=copier.realm)
+        replacement = {principal: alias}
+        tmpname = tasks.create_temp_file(self.smbserver, create_file=False)
+        try:
+            copier.copy_keys(paths.SAMBA_KEYTAB, tmpname, principal=principal,
+                             replacement=replacement)
+            self.smbserver.run_command(['kinit', '-kt', tmpname, netbiosname])
+        finally:
+            self.smbserver.run_command(['rm', '-f', tmpname])
+
     def test_samba_service_listed(self):
         """Check samba service is listed.
 
@@ -287,8 +320,7 @@ class TestSMB(IntegrationTest):
         self.check_smb_access_at_ipa_client(
             self.ipa_user1, self.ipa_user1_password, samba_share)
 
-    def test_smb_access_for_ad_user_at_ipa_client(
-            self, enable_smb_client_dns_lookup_kdc):
+    def test_smb_access_for_ad_user_at_ipa_client(self):
         samba_share = {
             'name': 'homes',
             'server_path': '/home/{}/{}'.format(self.ad.domain.name,
@@ -316,6 +348,40 @@ class TestSMB(IntegrationTest):
         finally:
             self.cleanup_mount(mount_point)
 
+    @pytest.mark.skipif(
+        osinfo.id == 'fedora' and osinfo.version_number <= (31,),
+        reason='Test requires krb 1.18')
+    def test_smb_service_s4u2self(self):
+        """Test S4U2Self operation by IPA service
+           against both AD and IPA users
+        """
+        script = textwrap.dedent("""export KRB5_TRACE=/dev/stderr
+        kdestroy -A
+        kinit -kt /etc/samba/samba.keytab {principal}
+        klist -f
+        {print_pac} -k /etc/samba/samba.keytab -E impersonate {user_princ}
+        klist -f
+        """)
+        principal = 'cifs/{hostname}'.format(
+            hostname=self.smbserver.hostname)
+        # Copy ipa-print-pac to SMB server
+        # We can do so because Samba and GSSAPI libraries
+        # are present there
+        print_pac = self.master.get_file_contents(
+            os.path.join(paths.LIBEXEC_IPA_DIR, "ipa-print-pac"))
+        result = self.smbserver.run_command(['mktemp'])
+        tmpname = result.stdout_text.strip()
+        self.smbserver.put_file_contents(tmpname, print_pac)
+        self.smbserver.run_command(['chmod', 'a+x', tmpname])
+        for user in (self.ad_user, self.ipa_user1,):
+            shell_script = script.format(principal=principal,
+                                         user_princ=user,
+                                         print_pac=tmpname)
+            self.smbserver.run_command(['/bin/bash', '-s', '-e'],
+                                       stdin_text=shell_script)
+        self.smbserver.run_command(['rm', '-f', tmpname])
+        tasks.kdestroy_all(self.smbserver)
+
     def test_smb_mount_fails_without_kerberos_ticket(self, samba_share_public):
         mountpoint = '/mnt/smb'
         try:
@@ -329,6 +395,68 @@ class TestSMB(IntegrationTest):
         finally:
             self.cleanup_mount(mountpoint)
 
+    def check_repeated_smb_mount(self, options):
+        mountpoint = '/mnt/smb'
+        unc = '//{}/homes'.format(self.smbserver.hostname)
+        test_file = 'ntlm_test'
+        test_file_server_path = '/home/{}/{}'.format(self.ipa_user1, test_file)
+        test_file_client_path = '{}/{}'.format(mountpoint, test_file)
+
+        self.smbclient.run_command(['mkdir', '-p', mountpoint])
+        self.smbserver.put_file_contents(test_file_server_path, '')
+        try:
+            for i in [1, 2]:
+                res = self.smbclient.run_command([
+                    'mount', '-t', 'cifs', unc, mountpoint, '-o', options],
+                    raiseonerr=False)
+                assert res.returncode == 0, (
+                    'Mount failed at iteration {}. Output: {}'
+                    .format(i, res.stdout_text + res.stderr_text))
+                assert self.smbclient.transport.file_exists(
+                    test_file_client_path)
+                self.smbclient.run_command(['umount', mountpoint])
+        finally:
+            self.cleanup_mount(mountpoint)
+            self.smbserver.run_command(['rm', '-f', test_file_server_path])
+
+    @skip_if_fips()
+    def test_ntlm_authentication_with_auto_domain(self):
+        """Repeatedly try to authenticate with username and password with
+        automatic domain discovery.
+
+        This is a regression test for https://pagure.io/freeipa/issue/8636
+        """
+        tasks.kdestroy_all(self.smbclient)
+
+        mount_options = 'user={user},pass={password},domainauto'.format(
+            user=self.ipa_user1,
+            password=self.ipa_user1_password
+        )
+
+        self.check_repeated_smb_mount(mount_options)
+
+    @skip_if_fips()
+    def test_ntlm_authentication_with_upn_with_lowercase_domain(self):
+        tasks.kdestroy_all(self.smbclient)
+
+        mount_options = 'user={user}@{domain},pass={password}'.format(
+            user=self.ipa_user1,
+            password=self.ipa_user1_password,
+            domain=self.master.domain.name.lower()
+        )
+        self.check_repeated_smb_mount(mount_options)
+
+    @skip_if_fips()
+    def test_ntlm_authentication_with_upn_with_uppercase_domain(self):
+        tasks.kdestroy_all(self.smbclient)
+
+        mount_options = 'user={user}@{domain},pass={password}'.format(
+            user=self.ipa_user1,
+            password=self.ipa_user1_password,
+            domain=self.master.domain.name.upper()
+        )
+        self.check_repeated_smb_mount(mount_options)
+
     def test_uninstall_samba(self):
         self.smbserver.run_command(['ipa-client-samba', '--uninstall', '-U'])
         res = self.smbserver.run_command(
