@@ -3,6 +3,8 @@
 # Copyright (C) 2015  FreeIPA Contributors see COPYING for license
 #
 
+from __future__ import absolute_import
+
 import os
 import pytest
 import tempfile
@@ -16,6 +18,7 @@ from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import rsa
 
 from ipalib import api, errors
+from ipaplatform.paths import paths
 from ipatests.util import (
     prepare_config, unlock_principal_password, change_principal,
     host_keytab)
@@ -48,7 +51,7 @@ def generate_user_csr(username, domain=None):
         username=username)
 
     with tempfile.NamedTemporaryFile(mode='w') as csr_file:
-        run(['openssl', 'req', '-new', '-key', CERT_RSA_PRIVATE_KEY_PATH,
+        run([paths.OPENSSL, 'req', '-new', '-key', CERT_RSA_PRIVATE_KEY_PATH,
              '-out', csr_file.name,
              '-config', prepare_config(
                  CERT_OPENSSL_CONFIG_TEMPLATE, csr_values)])
@@ -60,7 +63,7 @@ def generate_user_csr(username, domain=None):
 
 
 @pytest.fixture(scope='class')
-def smime_profile(request):
+def smime_profile(request, xmlrpc_setup):
     profile_path = prepare_config(
             SMIME_PROFILE_TEMPLATE,
             dict(ipadomain=api.env.domain, iparealm=api.env.realm))
@@ -73,7 +76,7 @@ def smime_profile(request):
 
 
 @pytest.fixture(scope='class')
-def smime_acl(request):
+def smime_acl(request, xmlrpc_setup):
     tracker = CAACLTracker(u'smime_acl')
 
     return tracker.make_fixture(request)
@@ -83,7 +86,7 @@ def smime_acl(request):
 # UserTracker has problems while setting passwords.
 # Until fixed, will use this fixture.
 @pytest.fixture(scope='class')
-def smime_user(request):
+def smime_user(request, xmlrpc_setup):
     username = u'alice'
     api.Command.user_add(uid=username, givenname=u'Alice', sn=u'SMIME',
                          userpassword=SMIME_USER_INIT_PW)
@@ -98,7 +101,7 @@ def smime_user(request):
 
 
 @pytest.fixture(scope='class')
-def smime_group(request):
+def smime_group(request, xmlrpc_setup):
     api.Command.group_add(u'smime_users')
 
     def fin():
@@ -256,7 +259,7 @@ class TestSignWithChangedProfile(XMLRPC_test):
 
 
 @pytest.fixture(scope='class')
-def smime_signing_ca(request):
+def smime_signing_ca(request, xmlrpc_setup):
     name = u'smime-signing-ca'
     subject = u'CN=SMIME CA,O=test industries Inc.'
     return CATracker(name, subject).make_fixture(request)
@@ -365,7 +368,7 @@ class TestCertSignMIMEwithSubCA(XMLRPC_test):
 
 
 @pytest.fixture(scope='class')
-def santest_subca(request):
+def santest_subca(request, xmlrpc_setup):
     name = u'default-profile-subca'
     subject = u'CN={},O=test'.format(name)
     tr = CATracker(name, subject)
@@ -373,19 +376,19 @@ def santest_subca(request):
 
 
 @pytest.fixture(scope='class')
-def santest_subca_acl(request):
+def santest_subca_acl(request, xmlrpc_setup):
     tr = CAACLTracker(u'default_profile_subca')
     return tr.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def santest_host_1(request):
+def santest_host_1(request, xmlrpc_setup):
     tr = HostTracker(u'santest-host-1')
     return tr.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def santest_host_2(request):
+def santest_host_2(request, xmlrpc_setup):
     tr = HostTracker(u'santest-host-2')
     return tr.make_fixture(request)
 
@@ -434,10 +437,10 @@ def santest_csr(request, santest_host_1, santest_host_2):
         pkey, hashes.SHA256(), backend
     ).public_bytes(serialization.Encoding.PEM)
 
-    return unicode(csr)
+    return csr.decode('ascii')
 
 
-class CAACLEnforcementOnCertBase(XMLRPC_test):
+class SubjectAltNameOneServiceBase(XMLRPC_test):
     """Base setup class for tests with SAN in CSR
 
     The class prepares an environment for test cases based
@@ -445,12 +448,10 @@ class CAACLEnforcementOnCertBase(XMLRPC_test):
 
     The class creates following entries:
 
-        * two host entries
+        * host entry
             * santest-host-1
-            * santest-host-2
-        * two service entries
+        * service entry
             * srv/santest-host-1
-            * srv/santest-host-2
         * Sub CA
             * default-profile-subca
 
@@ -467,12 +468,9 @@ class CAACLEnforcementOnCertBase(XMLRPC_test):
             * default-profile-subca -- CA
             * caIPAServiceCert      -- profile
     """
-
-    def test_prepare_caacl_hosts(self, santest_subca_acl,
-                                 santest_host_1, santest_host_2):
+    def test_prepare_caacl_hosts(self, santest_subca_acl, santest_host_1):
         santest_subca_acl.ensure_exists()
         santest_host_1.ensure_exists()
-        santest_host_2.ensure_exists()
         santest_subca_acl.add_host(santest_host_1.name)
 
     def test_prepare_caacl_CA(self, santest_subca_acl, santest_subca):
@@ -483,12 +481,78 @@ class CAACLEnforcementOnCertBase(XMLRPC_test):
         santest_subca_acl.add_profile(u'caIPAserviceCert')
 
     def test_prepare_caacl_services(self, santest_subca_acl,
-                                    santest_service_host_1,
-                                    santest_service_host_2):
+                                    santest_service_host_1):
         santest_service_host_1.ensure_exists()
+        santest_subca_acl.add_service(santest_service_host_1.name)
+
+
+class CAACLEnforcementOnCertBase(SubjectAltNameOneServiceBase):
+    """
+    Base setup class for tests with SAN in CSR, where
+    multiple hosts and services are in play.
+
+    In addition to the host and service created in the base class,
+    this class adds the following entries to the environment:
+
+        * host entry
+            * santest-host-2
+        * service entry
+            * srv/santest-host-2
+
+    """
+    def test_prepare_add_host_2(self, santest_host_2, santest_service_host_2):
+        santest_host_2.ensure_exists()
         santest_service_host_2.ensure_exists()
 
-        santest_subca_acl.add_service(santest_service_host_1.name)
+
+@pytest.mark.tier1
+class TestNoMatchForSubjectAltNameDnsName(SubjectAltNameOneServiceBase):
+    """Sign certificate request with an invalid SAN dnsName.
+
+    The CSR includes a DNS name that does not correspond to a
+    principal alias or alternative principal.
+
+    """
+    def test_request_cert_with_not_allowed_SAN(
+            self, santest_subca, santest_host_1,
+            santest_service_host_1, santest_csr):
+
+        with host_keytab(santest_host_1.name) as keytab_filename:
+            with change_principal(santest_host_1.attrs['krbcanonicalname'][0],
+                                  keytab=keytab_filename):
+                with pytest.raises(errors.NotFound):
+                    api.Command.cert_request(
+                        santest_csr,
+                        principal=santest_service_host_1.name,
+                        cacn=santest_subca.name
+                    )
+
+
+@pytest.mark.tier1
+class TestPrincipalAliasForSubjectAltNameDnsName(SubjectAltNameOneServiceBase):
+    """Test cert-request with SAN dnsName corresponding to a princpial alias.
+
+    Request should succeed.
+
+    """
+    def test_add_principal_alias(
+            self, santest_service_host_1, santest_service_host_2):
+        api.Command.service_add_principal(
+            santest_service_host_1.name,
+            santest_service_host_2.name)
+
+    def test_request_cert_with_SAN_matching_principal_alias(
+            self, santest_subca, santest_host_1,
+            santest_service_host_1, santest_csr):
+        with host_keytab(santest_host_1.name) as keytab_filename:
+            with change_principal(
+                    santest_host_1.attrs['krbcanonicalname'][0],
+                    keytab=keytab_filename):
+                api.Command.cert_request(
+                    santest_csr,
+                    principal=santest_service_host_1.name,
+                    cacn=santest_subca.name
+                )
 
 
 @pytest.mark.tier1
