@@ -130,8 +130,15 @@ class test_dns_realmdomains_integration(Declarative):
                     'idnsallowtransfer': [u'none;'],
                     'idnsallowquery': [u'any;'],
                     'objectclass': objectclasses.dnszone,
-
                 },
+                'messages': ({
+                    'code': 13004,
+                    'data': {'additional_info': 'Value will be ignored.',
+                             'option': 'idnssoaserial'},
+                    'message': "'idnssoaserial' option is deprecated. "
+                               "Value will be ignored.",
+                    'name': 'OptionDeprecatedWarning',
+                    'type': 'warning'},),
             },
             extra_check=assert_realmdomain_and_txt_record_present,
         ),
@@ -149,16 +156,26 @@ class test_dns_realmdomains_integration(Declarative):
             expected={
                 'value': DNSName(dnszone_2_absolute),
                 'summary': None,
-                'messages': ({
-                    u'message': u'DNS forwarder semantics changed since '
-                    u'IPA 4.0.\nYou may want to use forward zones '
-                    u'(dnsforwardzone-*) instead.\nFor more details read '
-                    u'the docs.',
-                    u'code': 13002,
-                    u'type': u'warning',
-                    u'name': u'ForwardersWarning',
-                    u'data': {}
-                },),
+                'messages': (
+                    {
+                        'code': 13004,
+                        'data': {'additional_info': 'Value will be ignored.',
+                                 'option': 'idnssoaserial'},
+                        'message': "'idnssoaserial' option is deprecated. "
+                                   "Value will be ignored.",
+                        'name': 'OptionDeprecatedWarning',
+                        'type': 'warning'
+                    },
+                    {
+                        u'message': u'DNS forwarder semantics changed since '
+                        u'IPA 4.0.\nYou may want to use forward zones '
+                        u'(dnsforwardzone-*) instead.\nFor more details read '
+                        u'the docs.',
+                        u'code': 13002,
+                        u'type': u'warning',
+                        u'name': u'ForwardersWarning',
+                        u'data': {}
+                    },),
                 'result': {
                     'dn': dnszone_2_dn,
                     'idnsname': [DNSName(dnszone_2_absolute)],
