@@ -27,7 +27,8 @@ import pytest
 from ipalib import errors
 from ipatests.test_xmlrpc import objectclasses
 from ipatests.test_xmlrpc.xmlrpc_test import (
-    fuzzy_digits, fuzzy_uuid, fuzzy_set_ci, add_oc,
+    fuzzy_digits, fuzzy_uuid, fuzzy_set_ci,
+    fuzzy_user_or_group_sid,
     XMLRPC_test, raises_exact
 )
 from ipatests.test_xmlrpc.tracker.group_plugin import GroupTracker
@@ -41,13 +42,13 @@ external_sid1 = u'S-1-1-123456-789-1'
 
 
 @pytest.fixture(scope='class')
-def group(request):
+def group(request, xmlrpc_setup):
     tracker = GroupTracker(name=u'testgroup1', description=u'Test desc1')
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def group2(request):
+def group2(request, xmlrpc_setup):
     tracker = GroupTracker(name=u'testgroup2', description=u'Test desc2')
     return tracker.make_fixture(request)
 
@@ -61,11 +62,13 @@ def managed_group(request, user):
     tracker.exists = True
     # Managed group gets created when user is created
     tracker.track_create()
+    # Managed groups don't have a SID
+    del tracker.attrs['ipantsecurityidentifier']
     return tracker
 
 
 @pytest.fixture(scope='class')
-def user(request):
+def user(request, xmlrpc_setup):
     tracker = UserTracker(name=u'user1', givenname=u'Test', sn=u'User1')
     return tracker.make_fixture(request)
 
@@ -79,13 +82,13 @@ def user_npg2(request, group):
     del tracker.attrs['mepmanagedentry']
     tracker.attrs.update(
         gidnumber=[u'1000'], description=[], memberof_group=[group.cn],
-        objectclass=add_oc(objectclasses.user_base, u'ipantuserattrs')
+        objectclass=objectclasses.user_base + ['ipantuserattrs']
     )
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def admins(request):
+def admins(request, xmlrpc_setup):
     # Track the admins group
     tracker = GroupTracker(
         name=u'admins', description=u'Account administrators group'
@@ -97,7 +100,7 @@ def admins(request):
 
 
 @pytest.fixture(scope='class')
-def trustadmins(request):
+def trustadmins(request, xmlrpc_setup):
     # Track the 'trust admins' group
     tracker = GroupTracker(
         name=u'trust admins', description=u'Trusts administrators group'
@@ -314,9 +317,9 @@ class TestFindGroup(XMLRPC_test):
                     'gidnumber': [fuzzy_digits],
                     'cn': [u'admins'],
                     'description': [u'Account administrators group'],
-                    'objectclass': fuzzy_set_ci(add_oc(
-                        objectclasses.posixgroup, u'ipantgroupattrs')),
+                    'objectclass': fuzzy_set_ci(objectclasses.posixgroup),
                     'ipauniqueid': [fuzzy_uuid],
+                    'ipantsecurityidentifier': [fuzzy_user_or_group_sid],
                 },
                 {
                     'dn': get_group_dn('editors'),
@@ -324,27 +327,27 @@ class TestFindGroup(XMLRPC_test):
                     'cn': [u'editors'],
                     'description':
                         [u'Limited admins who can edit other users'],
-                    'objectclass': fuzzy_set_ci(add_oc(
-                        objectclasses.posixgroup, u'ipantgroupattrs')),
+                    'objectclass': fuzzy_set_ci(objectclasses.posixgroup),
                     'ipauniqueid': [fuzzy_uuid],
+                    'ipantsecurityidentifier': [fuzzy_user_or_group_sid],
                 },
                 {
                     'dn': get_group_dn(group.cn),
                     'cn': [group.cn],
                     'description': [u'Test desc1'],
                     'gidnumber': [fuzzy_digits],
-                    'objectclass': fuzzy_set_ci(add_oc(
-                        objectclasses.posixgroup, u'ipantgroupattrs')),
+                    'objectclass': fuzzy_set_ci(objectclasses.posixgroup),
                     'ipauniqueid': [fuzzy_uuid],
+                    'ipantsecurityidentifier': [fuzzy_user_or_group_sid],
                 },
                 {
                     'dn': get_group_dn(group2.cn),
                     'cn': [group2.cn],
                     'description': [u'Test desc2'],
                     'gidnumber': [fuzzy_digits],
-                    'objectclass': fuzzy_set_ci(add_oc(
-                        objectclasses.posixgroup, u'ipantgroupattrs')),
+                    'objectclass': fuzzy_set_ci(objectclasses.posixgroup),
                     'ipauniqueid': [fuzzy_uuid],
+                    'ipantsecurityidentifier': [fuzzy_user_or_group_sid],
                 },
             ]), result)
 
@@ -388,6 +391,15 @@ class TestNonexistentGroup(XMLRPC_test):
 
 @pytest.mark.tier1
 class TestNonposixGroup(XMLRPC_test):
+    def test_create_nonposix_with_gid(self, group):
+        """ Try to create non-posix group with GID """
+        command = group.make_create_command(**dict(nonposix=True,
+                                                   gidnumber=10011))
+
+        with raises_exact(errors.ObjectclassViolation(
+                info=u'attribute "gidNumber" not allowed with --nonposix')):
+            command()
+
     def test_create_nonposix(self, group):
         """ Create a non-posix group """
         group.track_create()
@@ -489,6 +501,8 @@ class TestExternalGroup(XMLRPC_test):
         group.track_create()
         del group.attrs['gidnumber']
         group.attrs.update(objectclass=objectclasses.externalgroup)
+        # External group don't have a SID
+        del group.attrs['ipantsecurityidentifier']
         command = group.make_create_command(**dict(external=True))
         result = command()
         group.check_create(result)
@@ -757,3 +771,35 @@ class TestTrustAdminGroup(XMLRPC_test):
                           key=trustadmins.cn,
                           reason='Cannot support external non-IPA members')):
             command()
+
+
+@pytest.mark.tier1
+class TestGroupMemberManager(XMLRPC_test):
+    def test_add_member_manager_user(self, user, group):
+        user.ensure_exists()
+        group.ensure_exists()
+        group.add_member_manager({"user": user.uid})
+
+    def test_remove_member_manager_user(self, user, group):
+        user.ensure_exists()
+        group.ensure_exists()
+        group.remove_member_manager({"user": user.uid})
+
+    def test_add_member_manager_group(self, group, group2):
+        group.ensure_exists()
+        group2.ensure_exists()
+        group.add_member_manager({"group": group2.cn})
+
+    def test_remove_member_manager_group(self, group, group2):
+        group.ensure_exists()
+        group2.ensure_exists()
+        group.remove_member_manager({"group": group2.cn})
+
+    def test_member_manager_delete_user(self, user, group):
+        user.ensure_exists()
+        group.ensure_exists()
+        group.add_member_manager({"user": user.uid})
+        user.delete()
+        # deleting a user also deletes member manager reference
+        group.attrs.pop("membermanager_user")
+        group.retrieve()
