@@ -12,6 +12,7 @@ import six
 
 from collections import OrderedDict
 from ipalib import api, errors
+from ipaplatform.constants import constants as platformconstants
 
 from ipatests.test_xmlrpc.xmlrpc_test import XMLRPC_test, raises_exact
 
@@ -19,6 +20,13 @@ from ipatests.test_xmlrpc.tracker.user_plugin import UserTracker
 from ipatests.test_xmlrpc.tracker.group_plugin import GroupTracker
 from ipatests.test_xmlrpc.tracker.stageuser_plugin import StageUserTracker
 
+try:
+    from ipaserver.plugins.ldap2 import ldap2
+except ImportError:
+    have_ldap2 = False
+else:
+    have_ldap2 = True
+
 if six.PY3:
     unicode = str
 
@@ -40,7 +48,7 @@ sshpubkey = (u'ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDGAX3xAeLeaJggwTqMjxNwa6X'
              'cSIn3JrXynlvui4MixvrtX6zx+O/bBo68o8/eZD26QrahVbA09fivrn/4h3TM01'
              '9Eu/c2jOdckfU3cHUV/3Tno5d6JicibyaoDDK7S/yjdn5jhaz8MSEayQvFkZkiF'
              '0L public key test')
-sshpubkeyfp = (u'13:67:6B:BF:4E:A2:05:8E:AE:25:8B:A1:31:DE:6F:1B '
+sshpubkeyfp = (u'SHA256:cStA9o5TRSARbeketEOooMUMSWRSsArIAXloBZ4vNsE '
                'public key test (ssh-rsa)')
 
 options_def = OrderedDict([
@@ -49,7 +57,7 @@ options_def = OrderedDict([
     ('display name', {u'displayname': u'display'}),
     ('home directory', {u'homedirectory': u'/home/homedir'}),
     ('GECOS', {u'gecos': u'gecos'}),
-    ('shell', {u'loginshell': u'/bin/shell'}),
+    ('shell', {u'loginshell': platformconstants.DEFAULT_SHELL}),
     ('email address', {u'mail': u'email@email.email'}),
     ('job title', {u'title': u'newbie'}),
     ('kerberos principal', {
@@ -80,73 +88,95 @@ options_ids = list(options_def.keys())
 
 
 @pytest.fixture(scope='class')
-def stageduser(request):
+def stageduser(request, xmlrpc_setup):
     tracker = StageUserTracker(name=u'suser1', givenname=u'staged', sn=u'user')
     return tracker.make_fixture(request)
 
 
+@pytest.fixture(scope='class')
+def stageduser_min(request, xmlrpc_setup):
+    tracker = StageUserTracker(givenname=u'stagedmin', sn=u'usermin')
+    return tracker.make_fixture(request)
+
 @pytest.fixture(scope='class', params=options_ok, ids=options_ids)
-def stageduser2(request):
+def stageduser2(request, xmlrpc_setup):
     tracker = StageUserTracker(u'suser2', u'staged', u'user', **request.param)
     return tracker.make_fixture_activate(request)
 
 
 @pytest.fixture(scope='class')
-def user_activated(request):
+def user_activated(request, xmlrpc_setup):
     tracker = UserTracker(u'suser2', u'staged', u'user')
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def stageduser3(request):
+def stageduser3(request, xmlrpc_setup):
     tracker = StageUserTracker(name=u'suser3', givenname=u'staged', sn=u'user')
     return tracker.make_fixture_activate(request)
 
 
 @pytest.fixture(scope='class')
-def stageduser4(request):
+def stageduser4(request, xmlrpc_setup):
     tracker = StageUserTracker(u'tuser', u'test', u'user')
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def user(request):
+def stageduser_notposix(request, xmlrpc_setup):
+    tracker = StageUserTracker(u'notposix', u'notposix', u'notposix')
+    return tracker.make_fixture(request)
+
+
+@pytest.fixture(scope='class')
+def stageduser_customattr(request, xmlrpc_setup):
+    tracker = StageUserTracker(u'customattr', u'customattr', u'customattr',
+                               setattr=u'businesscategory=BusinessCat')
+    tracker.track_create()
+    tracker.attrs.update(
+        businesscategory=[u'BusinessCat']
+    )
+    return tracker.make_fixture(request)
+
+
+@pytest.fixture(scope='class')
+def user(request, xmlrpc_setup):
     tracker = UserTracker(u'auser1', u'active', u'user')
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def user2(request):
+def user2(request, xmlrpc_setup):
     tracker = UserTracker(u'suser3', u'staged', u'user')
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def user3(request):
+def user3(request, xmlrpc_setup):
     tracker = UserTracker(u'auser2', u'active', u'user')
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def user4(request):
+def user4(request, xmlrpc_setup):
     tracker = UserTracker(u'tuser', u'test', u'user')
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def user5(request):
+def user5(request, xmlrpc_setup):
     tracker = UserTracker(u'tuser', u'test', u'user')
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def user6(request):
+def user6(request, xmlrpc_setup):
     tracker = UserTracker(u'suser2', u'staged', u'user')
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def user7(request):
+def user7(request, xmlrpc_setup):
     tracker = UserTracker(u'puser1', u'preserved', u'user')
     return tracker.make_fixture_restore(request)
 
@@ -191,6 +221,12 @@ class TestNonexistentStagedUser(XMLRPC_test):
 
 @pytest.mark.tier1
 class TestStagedUser(XMLRPC_test):
+    def test_create_with_min_values(self, stageduser_min):
+        """ Create user with uid not specified """
+        stageduser_min.ensure_missing()
+        command = stageduser_min.make_create_command()
+        command()
+
     def test_create_duplicate(self, stageduser):
         stageduser.ensure_exists()
         command = stageduser.make_create_command()
@@ -255,6 +291,7 @@ class TestStagedUser(XMLRPC_test):
         stageduser.delete()
 
     def test_find_stageduser(self, stageduser):
+        stageduser.ensure_exists()
         stageduser.find()
 
     def test_findall_stageduser(self, stageduser):
@@ -281,6 +318,37 @@ class TestStagedUser(XMLRPC_test):
                               uidnumber=[uid], gidnumber=[gid]))
         stageduser.retrieve()
 
+    def test_without_posixaccount(self, stageduser_notposix):
+        """Test stageuser-find when the staged user is not a posixaccount.
+        """
+        stageduser_notposix.ensure_missing()
+
+        # Directly create the user using ldapmod
+        # without the posixaccount objectclass
+        if not have_ldap2:
+            pytest.skip('server plugin not available')
+        ldap = ldap2(api)
+        ldap.connect()
+        ldap.create(
+            dn=stageduser_notposix.dn,
+            objectclass=[u'inetorgperson', u'organizationalperson', u'person'],
+            uid=stageduser_notposix.uid,
+            sn=stageduser_notposix.sn,
+            givenname=stageduser_notposix.givenname,
+            cn=stageduser_notposix.uid
+        )
+        # Check that stageuser-find correctly finds the user
+        command = stageduser_notposix.make_find_command(
+            uid=stageduser_notposix.uid)
+        result = command()
+        assert result['count'] == 1
+
+    def test_create_withuserauthtype(self, stageduser):
+        stageduser.ensure_missing()
+        command = stageduser.make_create_command(
+            options={u'ipauserauthtype': u'password'})
+        command()
+
 
 @pytest.mark.tier1
 class TestCreateInvalidAttributes(XMLRPC_test):
@@ -522,6 +590,59 @@ class TestPreserved(XMLRPC_test):
         stageduser.delete()
 
 
+@pytest.mark.tier1
+class TestCustomAttr(XMLRPC_test):
+    """Test for pagure ticket 7597
+
+    When a staged user is activated, preserved and finally staged again,
+    the custom attributes are lost.
+    """
+    def test_stageduser_customattr(self, stageduser_customattr):
+        # Create a staged user with attributes not accessible
+        # through the options
+        # --setattr is needed here
+        command = stageduser_customattr.make_create_command()
+        result = command()
+        stageduser_customattr.check_create(result, [u'businesscategory'])
+
+        # Activate the staged user
+        user_customattr = UserTracker(
+            stageduser_customattr.uid, stageduser_customattr.givenname,
+            stageduser_customattr.sn)
+        user_customattr.create_from_staged(stageduser_customattr)
+        user_customattr.attrs[u'businesscategory'] = [u'BusinessCat']
+
+        command = stageduser_customattr.make_activate_command()
+        result = command()
+        user_customattr.check_activate(result)
+
+        # Check that the user contains businesscategory
+        command = user_customattr.make_retrieve_command(all=True)
+        result = command()
+        assert 'BusinessCat' in result['result'][u'businesscategory']
+
+        # delete the user with --preserve
+        command = user_customattr.make_delete_command(no_preserve=False,
+                                                      preserve=True)
+        result = command()
+        user_customattr.check_delete(result)
+
+        # Check that the preserved user contains businesscategory
+        command = user_customattr.make_retrieve_command(all=True)
+        result = command()
+        assert 'BusinessCat' in result['result'][u'businesscategory']
+
+        # Move the user from preserved to stage
+        command = user_customattr.make_stage_command()
+        result = command()
+        stageduser_customattr.check_restore_preserved(result)
+
+        # Check that the stage user contains businesscategory
+        command = stageduser_customattr.make_retrieve_command(all=True)
+        result = command()
+        assert 'BusinessCat' in result['result'][u'businesscategory']
+
+
 @pytest.mark.tier1
 class TestManagers(XMLRPC_test):
     def test_staged_manager(self, user, stageduser):
@@ -618,7 +739,7 @@ class TestDuplicates(XMLRPC_test):
 
 
 @pytest.fixture(scope='class')
-def group(request):
+def group(request, xmlrpc_setup):
     tracker = GroupTracker(u'testgroup')
     return tracker.make_fixture(request)
 
